@@ -7,15 +7,15 @@
  *
  */
 
-#ifndef __COMMANDS_H__
-#define __COMMANDS_H__
+#ifndef C_COMMANDS_H
+#define C_COMMANDS_H
 
-typedef enum {
+enum ExResult {
     ErFAIL = 0,
     ErOK = 1
-} ExResult;
+};
 
-typedef enum {
+enum ExCommands {
     ExNop,
     ExFail,
 
@@ -542,6 +542,28 @@ typedef enum {
     ExViewCvsLog,
     /// View CVS log
 
+    //<cmd_svn> SVN Support
+    ExSvn,
+    /// Ask for SVN options and run SVN
+    ExRunSvn,
+    /// Run configured SVN command
+    ExViewSvn,
+    /// View SVN output
+    ExClearSvnMessages,
+    /// Clear SVN messages
+    ExSvnDiff,
+    /// Ask for SVN diff options and run SVN
+    ExRunSvnDiff,
+    /// Run configured SVN diff command
+    ExViewSvnDiff,
+    /// View SVN diff output
+    ExSvnCommit,
+    /// Ask for SVN commit options and run SVN
+    ExRunSvnCommit,
+    /// Run configured SVN commit command
+    ExViewSvnLog,
+    /// View SVN log
+
     //<cmd_tags> TAGS Commands
     /// fte supports TAGS files generated by programs like ctags.
     ExTagFind,
@@ -598,6 +620,8 @@ typedef enum {
     ///
     ExToggleShowBookmarks,
     ///
+    ExToggleMakeBackups,
+    ///
     ExSetLeftMargin,
     ///
     ExSetRightMargin,
@@ -747,6 +771,6 @@ typedef enum {
 
     ExBlockPasteOver
     /// Delete content's of selection and paste clipboard to current position
-} ExCommands;
+};
 
-#endif
+#endif // C_COMMANDS_H
