@@ -7,16 +7,34 @@
  *
  */
 
-#include "fte.h"
+#include "c_config.h"
 
-typedef struct _CurPos {
-    int sz;
+#include "c_bind.h"
+#include "c_color.h"
+#include "c_fconfig.h"
+#include "ftever.h"
+#include "log.h"
+#include "o_buflist.h"
+#include "o_cvsbase.h"
+#include "o_svnbase.h"
+#include "s_string.h"
+
+#include <fcntl.h>
+
+struct GUICharactersEntry {
+    struct GUICharactersEntry *next;
+    char *name;
+    char *chars;
+};
+
+struct CurPos {
+    off_t sz;
     const char *a;
     const char *c;
     const char *z;
     int line;
     const char *name; // filename
-} CurPos;
+};
 
 #ifdef CONFIG_INDENT_C
 extern int C_Indent;
@@ -32,6 +50,7 @@ extern int C_FirstLevelWidth;
 extern int C_FirstLevelIndent;
 extern int C_Continuation;
 extern int C_ParenDelta;
+extern int FunctionUsesContinuation;
 #endif
 
 #ifdef CONFIG_INDENT_REXX
@@ -48,10 +67,11 @@ int ScreenSizeX = -1, ScreenSizeY = -1;
 int ScrollBarWidth = 1;
 int CursorInsSize[2] = { 90, 100 };
 int CursorOverSize[2] = { 0, 100 };
+bool CursorBlink = 0; // default is "no" (same as before)
 int OpenAfterClose = 1;
 int SelectPathname = 0;
 char DefaultModeName[32] = "";
-RxNode *CompletionFilter;
+RxNode *CompletionFilter = NULL;
 #if defined(DOS) || defined(DOSP32)
 char PrintDevice[MAXPATH] = "PRN";
 #else
@@ -62,6 +82,10 @@ int KeepHistory = 0;
 int LoadDesktopOnEntry = 0;
 int SaveDesktopOnExit = 0;
 char WindowFont[64] = "";
+// Custom RGB colors (if console driver supports them)
+TRGBColor RGBColor[16];
+// true if corresponding triplet in RGBColor is valid
+bool RGBColorValid [16];
 int KeepMessages = 0;
 int ScrollBorderX = 0;
 int ScrollBorderY = 0;
@@ -74,17 +98,92 @@ int WeirdScroll = 0;
 int LoadDesktopMode = 0;
 char HelpCommand[128] = "man -a";
 char *ConfigSourcePath = 0;
+int IgnoreBufferList = 0;
+static GUICharactersEntry *GUICharacters = NULL;
+#ifdef CONFIG_OBJ_CVS
+char CvsCommand[256] = "cvs";
+char CvsLogMode[32] = "PLAIN";
+#endif
+#ifdef CONFIG_OBJ_SVN
+char SvnCommand[256] = "svn";
+char SvnLogMode[32] = "PLAIN";
+#endif
+int ReassignModelIds = 0;
+int RecheckReadOnly = 0;
+char XShellCommand[256] = "xterm";
+
+// Which characters to get. defaultCharacters if not set, rest filled
+// with defaultCharacters if too short
+// List of GUICharacters is freed, only one item remains
+const char *GetGUICharacters(const char *which, const char *defChars) {
+    GUICharactersEntry *g, *gg, *found = NULL;
+    char *s;
+    size_t i;
+
+    for (g = GUICharacters; g; g=gg) {
+        gg = g->next;
+        if (strcmp(g->name, which) == 0) {
+            if ((i = strlen(g->chars)) < strlen(defChars)) {
+                s = new char [strlen(defChars) + 1];
+                assert(s != NULL);
+                strcpy(s, g->chars);
+                strcpy(s + i, defChars + i);
+                delete g->chars;
+                g->chars = s;
+            }
+            if (found) {
+                free(found->chars); free(found->name); free(found);
+            }
+            found = g;
+        } else {
+            free(g->name); free(g->chars); free(g);
+        }
+    }
+    GUICharacters = found;
+    return found ? found->chars : defChars;
+}
+
+static void AppendGUICharacters(const char *string) {
+    const char *s;
+    GUICharactersEntry *g;
+
+    s = strchr(string, ':');
+    if (s) {
+        g = new GUICharactersEntry;
+        assert(g != NULL);
+
+        // allocate memory for name  +1 for strncat
+        g->name = (char *)malloc((s-string) + 1);
+        assert(g->name != NULL);
+
+        // make sure we have zero at start of string
+        *(g->name) = 0;
+
+        // strncat makes sure that we have zero at the end...
+        strncat(g->name, string, (s-string));
+
+        // copy text after ':' to chars...
+        g->chars = strdup(s+1);
+        assert(g->chars != NULL);
+
+        g->next = GUICharacters;
+        GUICharacters = g;
+    }
+}
 
 #ifdef CONFIG_SYNTAX_HILIT
-int AddKeyword(ColorKeywords *tab, char color, const char *keyword) {
-    int len;
-
-    len = strlen(keyword);
+static int AddKeyword(ColorKeywords *tab, char color, const char *keyword) {
+    size_t len = strlen(keyword);
     if (len < 1 || len >= CK_MAXLEN) return 0;
 
     if (tab->key[len]) {
-        int lx = strlen(tab->key[len]);
-        tab->key[len] = (char *)realloc(tab->key[len], lx + len + 1 + 1);
+        size_t lx = strlen(tab->key[len]);
+        char *key;
+
+        key = (char *)realloc(tab->key[len], lx + len + 1 + 1);
+        assert(key != NULL);
+
+        tab->key[len] = key;
         assert(tab->key[len] != 0);
         strcpy(tab->key[len] + lx, keyword);
         tab->key[len][lx + len] = color;
@@ -102,7 +201,7 @@ int AddKeyword(ColorKeywords *tab, char color, const char *keyword) {
 }
 #endif
 
-int SetModeNumber(EMode *mode, int what, int number) {
+static int SetModeNumber(EMode *mode, int what, int number) {
     int j = what;
 
     if (j == BFI_LeftMargin || j == BFI_RightMargin) number--;
@@ -110,7 +209,7 @@ int SetModeNumber(EMode *mode, int what, int number) {
     return 0;
 }
 
-int SetModeString(EMode *mode, int what, const char *string) {
+static int SetModeString(EMode *mode, int what, const char *string) {
     int j = what;
 
 #ifdef CONFIG_SYNTAX_HILIT
@@ -120,8 +219,10 @@ int SetModeString(EMode *mode, int what, const char *string) {
 #endif
         if (j == BFI_EventMap) {
             mode->fEventMap = FindEventMap(string);
+#ifdef CONFIG_INDENT
         } else if (j == BFI_IndentMode) {
             mode->Flags.num[j] = GetIndentMode(string);
+#endif
         } else if (j == BFS_WordChars) {
             SetWordChars(mode->Flags.WordChars, string);
         } else if (j == BFS_CapitalChars) {
@@ -141,12 +242,18 @@ int SetModeString(EMode *mode, int what, const char *string) {
             mode->MatchLine = strdup(string);
             mode->MatchLineRx = RxCompile(string);
         } else {
+            if (mode->Flags.str[j & 0xFF])
+                free(mode->Flags.str[j & 0xFF]);
             mode->Flags.str[j & 0xFF] = strdup(string);
         }
     return 0;
 }
 
-int SetGlobalNumber(int what, int number) {
+// *INDENT-OFF*
+static int SetGlobalNumber(int what, int number)
+{
+    STARTFUNC("SetGlobalNumber");
+    LOG << "What: " << what << " Number: " << number << ENDLINE;
     switch (what) {
 #ifdef CONFIG_INDENT_C
     case FLAG_C_Indent:          C_Indent = number; break;
@@ -162,6 +269,7 @@ int SetGlobalNumber(int what, int number) {
     case FLAG_C_FirstLevelWidth: C_FirstLevelWidth = number; break;
     case FLAG_C_Continuation:    C_Continuation = number; break;
     case FLAG_C_ParenDelta:      C_ParenDelta = number; break;
+    case FLAG_FunctionUsesContinuation: FunctionUsesContinuation = number; break;
 #endif
 #ifdef CONFIG_INDENT_REXX
     case FLAG_REXX_Indent:       REXX_Base_Indent = number; break;
@@ -173,6 +281,7 @@ int SetGlobalNumber(int what, int number) {
     case FLAG_CursorInsertEnd:   CursorInsSize[1] = number; break;
     case FLAG_CursorOverStart:   CursorOverSize[0] = number; break;
     case FLAG_CursorOverEnd:     CursorOverSize[1] = number; break;
+    case FLAG_CursorBlink:       CursorBlink = number; break;
     case FLAG_SysClipboard:      SystemClipboard = number; break;
     case FLAG_OpenAfterClose:    OpenAfterClose = number; break;
     case FLAG_ShowVScroll:       ShowVScroll = number; break;
@@ -194,58 +303,104 @@ int SetGlobalNumber(int what, int number) {
     case FLAG_SevenBit:          SevenBit = number; break;
     case FLAG_WeirdScroll:       WeirdScroll = number; break;
     case FLAG_LoadDesktopMode:   LoadDesktopMode = number; break;
+    case FLAG_IgnoreBufferList:  IgnoreBufferList = number; break;
+    case FLAG_ReassignModelIds:  ReassignModelIds = number; break;
+    case FLAG_RecheckReadOnly:   RecheckReadOnly = number; break;
     default:
-        return -1;
+        //printf("Unknown global number: %d\n", what);
+        ENDFUNCRC(-1);
     }
-    return 0;
+    ENDFUNCRC(0);
+}
+// *INDENT-ON*
+
+static void SetRGBColor(const char *string) {
+    int idx,r,g,b;
+    if (sscanf (string, "%x:%x,%x,%x", &idx, &r, &g, &b) != 4) {
+        fprintf(stderr, "Invalid RGB Definition: %s\n", string);
+        return;
+    }
+    if (idx < 0 || idx > 15) {
+        fprintf(stderr, "Invalid RGB index: (0-f only) (%s)\n", string);
+        return;
+    }
+    if (r < 0 || r > 255 ||
+        g < 0 || g > 255 ||
+        b < 0 || b > 255) {
+        fprintf(stderr, "Invalid RGB palette values (00-ff only): %s\n", string);
+        return;
+    }
+    RGBColorValid[idx] = true;
+    RGBColor[idx].r = (unsigned char) r;
+    RGBColor[idx].g = (unsigned char) g;
+    RGBColor[idx].b = (unsigned char) b;
 }
 
-int SetGlobalString(long what, const char *string) {
+static int SetGlobalString(long what, const char *string) {
+    STARTFUNC("SetGlobalString");
+    LOG << "What: " << what << " String: " << string << ENDLINE;
+
     switch (what) {
-    case FLAG_DefaultModeName: strcpy(DefaultModeName, string); break;
+    case FLAG_DefaultModeName: strlcpy(DefaultModeName, string, sizeof(DefaultModeName)); break;
     case FLAG_CompletionFilter: if ((CompletionFilter = RxCompile(string)) == NULL) return -1; break;
-    case FLAG_PrintDevice: strcpy(PrintDevice, string); break;
-    case FLAG_CompileCommand: strcpy(CompileCommand, string); break;
-    case FLAG_WindowFont: strcpy(WindowFont, string); break;
-    case FLAG_HelpCommand: strcpy(HelpCommand, string); break;
+    case FLAG_PrintDevice: strlcpy(PrintDevice, string, sizeof(PrintDevice)); break;
+    case FLAG_CompileCommand: strlcpy(CompileCommand, string, sizeof(CompileCommand)); break;
+    case FLAG_WindowFont: strlcpy(WindowFont, string, sizeof(WindowFont)); break;
+    case FLAG_HelpCommand: strlcpy(HelpCommand, string, sizeof(HelpCommand)); break;
+    case FLAG_GUICharacters: AppendGUICharacters (string); break;
+#ifdef CONFIG_OBJ_CVS
+    case FLAG_CvsCommand: strlcpy(CvsCommand, string, sizeof(CvsCommand)); break;
+    case FLAG_CvsLogMode: strlcpy(CvsLogMode, string, sizeof(CvsLogMode)); break;
+#endif
+#ifdef CONFIG_OBJ_SVN
+    case FLAG_SvnCommand: strlcpy(SvnCommand, string, sizeof(SvnCommand)); break;
+    case FLAG_SvnLogMode: strlcpy(SvnLogMode, string, sizeof(SvnLogMode)); break;
+#endif
+    case FLAG_RGBColor: SetRGBColor(string); break;
+    case FLAG_XShellCommand: strlcpy(XShellCommand, string, sizeof(XShellCommand)); break;
     default:
-        return -1;
+        //printf("Unknown global string: %ld\n", what);
+        ENDFUNCRC(-1);
     }
-    return 0;
+    ENDFUNCRC(0);
 }
 
-int SetEventString(EEventMap *Map, long what, const char *string) {
+static int SetEventString(EEventMap *Map, int what, const char *string) {
+    STARTFUNC("SetEventString");
+    LOG << "What: " << what << " String: " << string << ENDLINE;
     switch (what) {
     case EM_MainMenu:
     case EM_LocalMenu:
         Map->SetMenu(what, string);
         break;
     default:
-        return -1;
+        ENDFUNCRC(-1);
     }
-    return 0;
+    ENDFUNCRC(0);
 }
 
 #ifdef CONFIG_SYNTAX_HILIT
-int SetColorizeString(EColorize *Colorize, long what, const char *string) {
+static int SetColorizeString(EColorize *Colorize, long what, const char *string) {
+    STARTFUNC("SetColorizeString");
+    LOG << "What: " << what << " String: " << string << ENDLINE;
     switch (what) {
     case COL_SyntaxParser:
         Colorize->SyntaxParser = GetHilitMode(string);
         break;
     default:
-        return -1;
+        ENDFUNCRC(-1);
     }
-    return 0;
+    ENDFUNCRC(0);
 }
 #endif
 
-unsigned char GetObj(CurPos &cp, unsigned short &len) {
+static unsigned char GetObj(CurPos &cp, unsigned short &len) {
     len = 0;
     if (cp.c + 3 <= cp.z) {
         unsigned char c;
         unsigned char l[2];
         c = *cp.c++;
-        memcpy(&l, cp.c, 2);
+        memcpy(l, cp.c, 2);
         len = (l[1] << 8) + l[0];
         cp.c += 2;
         return c;
@@ -253,18 +408,24 @@ unsigned char GetObj(CurPos &cp, unsigned short &len) {
     return 0xFF;
 }
 
-const char *GetCharStr(CurPos &cp, unsigned short len) {
+static const char *GetCharStr(CurPos &cp, unsigned short len) {
+    STARTFUNC("GetCharStr");
+    LOG << "Length: " << len << ENDLINE;
+
     const char *p = cp.c;
     if (cp.c + len > cp.z)
-        return 0;
+    {
+        LOG << "End of config file in GetCharStr" << ENDLINE;
+        ENDFUNCRC(0);
+    }
     cp.c += len;
-    return p;
+    ENDFUNCRC(p);
 }
 
-int GetNum(CurPos &cp, long &num) {
+static int GetNum(CurPos &cp, long &num) {
     unsigned char n[4];
     if (cp.c + 4 > cp.z) return 0;
-    memcpy(&n, cp.c, 4);
+    memcpy(n, cp.c, 4);
     num =
         (n[3] << 24) +
         (n[2] << 16) +
@@ -277,17 +438,20 @@ int GetNum(CurPos &cp, long &num) {
     return 1;
 }
 
-int ReadCommands(CurPos &cp, const char *Name) {
+static int ReadCommands(CurPos &cp, const char *Name) {
+    STARTFUNC("ReadCommands");
+    LOG << "Name = " << (Name != NULL ? Name : "(null)") << ENDLINE;
+
     unsigned char obj;
     unsigned short len;
     long Cmd = NewCommand(Name);
     long cmdno;
 
-    if (GetObj(cp, len) != CF_INT) return-1;
-    if (GetNum(cp, cmdno) == 0) return -1;
+    if (GetObj(cp, len) != CF_INT) ENDFUNCRC(-1);
+    if (GetNum(cp, cmdno) == 0) ENDFUNCRC(-1);
     if (cmdno != (Cmd | CMD_EXT)) {
         fprintf(stderr, "Bad Command map %s -> %ld != %ld\n", Name, Cmd, cmdno);
-        return -1;
+        ENDFUNCRC(-1);
     }
 
     while ((obj = GetObj(cp, len)) != 0xFF) {
@@ -300,11 +464,11 @@ int ReadCommands(CurPos &cp, const char *Name) {
                 long cmd;
 
                 //                if ((s = GetCharStr(cp, len)) == 0) return -1;
-                if (GetNum(cp, cmd) == 0) return -1;
-                if (GetObj(cp, len) != CF_INT) return -1;
-                if (GetNum(cp, cnt) == 0) return -1;
-                if (GetObj(cp, len) != CF_INT) return -1;
-                if (GetNum(cp, ign) == 0) return -1;
+                if (GetNum(cp, cmd) == 0) ENDFUNCRC(-1);
+                if (GetObj(cp, len) != CF_INT) ENDFUNCRC(-1);
+                if (GetNum(cp, cnt) == 0) ENDFUNCRC(-1);
+                if (GetObj(cp, len) != CF_INT) ENDFUNCRC(-1);
+                if (GetNum(cp, ign) == 0) ENDFUNCRC(-1);
 
                 //                if (cmd != CmdNum(s)) {
                 //                    fprintf(stderr, "Bad Command Id: %s -> %d\n", s, cmd);
@@ -314,7 +478,7 @@ int ReadCommands(CurPos &cp, const char *Name) {
                 if (AddCommand(Cmd, cmd, cnt, ign) == 0) {
                     if (Name == 0 || strcmp(Name, "xx") != 0) {
                         fprintf(stderr, "Bad Command Id: %ld\n", cmd);
-                        return -1;
+                        ENDFUNCRC(-1);
                     }
                 }
             }
@@ -323,39 +487,39 @@ int ReadCommands(CurPos &cp, const char *Name) {
             {
                 const char *s = GetCharStr(cp, len);
 
-                if (s == 0) return -1;
-                if (AddString(Cmd, s) == 0) return -1;
+                if (s == 0) ENDFUNCRC(-1);
+                if (AddString(Cmd, s) == 0) ENDFUNCRC(-1);
             }
             break;
         case CF_INT:
             {
                 long num;
 
-                if (GetNum(cp, num) == 0) return -1;
-                if (AddNumber(Cmd, num) == 0) return -1;
+                if (GetNum(cp, num) == 0) ENDFUNCRC(-1);
+                if (AddNumber(Cmd, num) == 0) ENDFUNCRC(-1);
             }
             break;
         case CF_VARIABLE:
             {
                 long num;
 
-                if (GetNum(cp, num) == 0) return -1;
-                if (AddVariable(Cmd, num) == 0) return -1;
+                if (GetNum(cp, num) == 0) ENDFUNCRC(-1);
+                if (AddVariable(Cmd, num) == 0) ENDFUNCRC(-1);
             }
             break;
         case CF_CONCAT:
-            if (AddConcat(Cmd) == 0) return -1;
+            if (AddConcat(Cmd) == 0) ENDFUNCRC(-1);
             break;
         case CF_END:
-            return Cmd;
+            ENDFUNCRC(Cmd);
         default:
-            return -1;
+            ENDFUNCRC(-1);
         }
     }
-    return -1;
+    ENDFUNCRC(-1);
 }
 
-int ReadMenu(CurPos &cp, const char *MenuName) {
+static int ReadMenu(CurPos &cp, const char *MenuName) {
     unsigned char obj;
     unsigned short len;
 
@@ -411,7 +575,7 @@ int ReadMenu(CurPos &cp, const char *MenuName) {
     return -1;
 }
 
-int ReadColors(CurPos &cp, const char *ObjName) {
+static int ReadColors(CurPos &cp, const char *ObjName) {
     unsigned char obj;
     unsigned short len;
 
@@ -419,16 +583,19 @@ int ReadColors(CurPos &cp, const char *ObjName) {
         switch (obj) {
         case CF_STRING:
             {
-                char cl[30];
                 const char *sname = GetCharStr(cp, len);
                 const char *svalue;
                 if (sname == 0) return -1;
                 if ((obj = GetObj(cp, len)) != CF_STRING) return -1;
                 if ((svalue = GetCharStr(cp, len)) == 0) return -1;
-                strcpy(cl, ObjName);
-                strcat(cl, ".");
-                strcat(cl, sname);
-                if (SetColor(cl, svalue) == 0) return -1;
+
+                StlString cl(ObjName);
+
+                cl += '.';
+                cl += sname;
+
+                if (SetColor(cl.c_str(), svalue) == 0)
+                   return -1;
             }
             break;
         case CF_END:
@@ -441,7 +608,7 @@ int ReadColors(CurPos &cp, const char *ObjName) {
 }
 
 #ifdef CONFIG_SYNTAX_HILIT
-int ReadHilitColors(CurPos &cp, EColorize *Colorize, const char * /*ObjName*/) {
+static int ReadHilitColors(CurPos &cp, EColorize *Colorize, const char * /*ObjName*/) {
     unsigned char obj;
     unsigned short len;
 
@@ -470,7 +637,7 @@ int ReadHilitColors(CurPos &cp, EColorize *Colorize, const char * /*ObjName*/) {
     return -1;
 }
 
-int ReadKeywords(CurPos &cp, ColorKeywords *keywords, int color) {
+static int ReadKeywords(CurPos &cp, ColorKeywords *keywords, int color) {
     unsigned char obj;
     unsigned short len;
 
@@ -493,7 +660,7 @@ int ReadKeywords(CurPos &cp, ColorKeywords *keywords, int color) {
 }
 #endif
 
-int ReadEventMap(CurPos &cp, EEventMap *Map, const char * /*MapName*/) {
+static int ReadEventMap(CurPos &cp, EEventMap *Map, const char * /*MapName*/) {
     unsigned char obj;
     unsigned short len;
 
@@ -574,7 +741,7 @@ int ReadEventMap(CurPos &cp, EEventMap *Map, const char * /*MapName*/) {
 }
 
 #ifdef CONFIG_SYNTAX_HILIT
-int ReadColorize(CurPos &cp, EColorize *Colorize, const char *ModeName) {
+static int ReadColorize(CurPos &cp, EColorize *Colorize, const char *ModeName) {
     unsigned char obj;
     unsigned short len;
 
@@ -656,7 +823,7 @@ int ReadColorize(CurPos &cp, EColorize *Colorize, const char *ModeName) {
                 if (GetNum(cp, color) == 0)
                     return -1;
                 obj = GetObj(cp, len);
-                assert(obj == CF_STRING);
+                assert(matchFlags & MATCH_REGEXP ? obj == CF_REGEXP : obj == CF_STRING);
                 if ((match = GetCharStr(cp, len)) == 0)
                     return -1;
 
@@ -666,11 +833,15 @@ int ReadColorize(CurPos &cp, EColorize *Colorize, const char *ModeName) {
                 newTrans.nextState = nextState;
                 newTrans.color = color;
 
-                if ((newTrans.matchFlags & MATCH_SET) ||
-                    (newTrans.matchFlags & MATCH_NOTSET))
+                if (newTrans.matchFlags & MATCH_REGEXP) {
+                    newTrans.regexp = RxCompile(match);
+                    newTrans.matchLen = 0;
+                } else if ((newTrans.matchFlags & MATCH_SET) ||
+                           (newTrans.matchFlags & MATCH_NOTSET))
                 {
                     newTrans.matchLen = 1;
-                    assert((newTrans.match = (char *)malloc(256/8)) != 0);
+                    newTrans.match = (char *)malloc(256/8);
+                    assert(newTrans.match != NULL);
                     SetWordChars(newTrans.match, match);
                 } else {
                     newTrans.match = strdup(match);
@@ -720,7 +891,8 @@ int ReadColorize(CurPos &cp, EColorize *Colorize, const char *ModeName) {
                 Colorize->hm->LastState()->nextKwdNoCharState = nextKwdNoCharState;
 
                 if (wordChars && *wordChars) {
-                    assert((Colorize->hm->LastState()->wordChars = (char *)malloc(256/8)) != 0);
+                    Colorize->hm->LastState()->wordChars = (char *)malloc(256/8);
+                    assert(Colorize->hm->LastState()->wordChars != NULL);
                     SetWordChars(Colorize->hm->LastState()->wordChars, wordChars);
                 }
             }
@@ -748,7 +920,12 @@ int ReadColorize(CurPos &cp, EColorize *Colorize, const char *ModeName) {
                         Col <<= 4;
                         Col |= (hcPlain_Background & 0x0F);
                     } else {
-                        if (sscanf(Value, "%2X", &Col) != 1) return -1;
+                        unsigned int ColBg, ColFg;
+
+                        if (sscanf(colorstr, "%1X %1X", &ColFg, &ColBg) != 2)
+                            return 0;
+
+                        Col = ColFg | (ColBg << 4);
                     }
                     color = Col;
                 }
@@ -792,7 +969,7 @@ int ReadColorize(CurPos &cp, EColorize *Colorize, const char *ModeName) {
 }
 #endif
 
-int ReadMode(CurPos &cp, EMode *Mode, const char * /*ModeName*/) {
+static int ReadMode(CurPos &cp, EMode *Mode, const char * /*ModeName*/) {
     unsigned char obj;
     unsigned short len;
 
@@ -833,7 +1010,7 @@ int ReadMode(CurPos &cp, EMode *Mode, const char * /*ModeName*/) {
     return -1;
 }
 
-int ReadObject(CurPos &cp, const char *ObjName) {
+static int ReadObject(CurPos &cp, const char *ObjName) {
     unsigned char obj;
     unsigned short len;
 
@@ -860,6 +1037,31 @@ int ReadObject(CurPos &cp, const char *ObjName) {
                 if (AddCRegexp(file, line, msg, regexp) == 0) return -1;
             }
             break;
+#endif
+#ifdef CONFIG_OBJ_CVS
+        case CF_CVSIGNRX:
+            {
+                const char *regexp;
+
+                if (GetObj(cp, len) != CF_REGEXP) return -1;
+                if ((regexp = GetCharStr(cp, len)) == 0) return -1;
+
+                if (AddCvsIgnoreRegexp(regexp) == 0) return -1;
+            }
+            break;
+#endif
+
+#ifdef CONFIG_OBJ_SVN
+        case CF_SVNIGNRX:
+            {
+                const char *regexp;
+
+                if (GetObj(cp, len) != CF_REGEXP) return -1;
+                if ((regexp = GetCharStr(cp, len)) == 0) return -1;
+
+                if (AddSvnIgnoreRegexp(regexp) == 0) return -1;
+            }
+            break;
 #endif
         case CF_SETVAR:
             {
@@ -896,7 +1098,7 @@ int ReadObject(CurPos &cp, const char *ObjName) {
     return -1;
 }
 
-int ReadConfigFile(CurPos &cp) {
+static int ReadConfigFile(CurPos &cp) {
     unsigned char obj;
     unsigned short len;
 
@@ -907,6 +1109,10 @@ int ReadConfigFile(CurPos &cp) {
         assert(obj == CF_STRING);
         if ((p = GetCharStr(cp, len)) == 0)
             return -1;
+
+        if (ConfigSourcePath)
+            free(ConfigSourcePath);
+
         ConfigSourcePath = strdup(p);
     }
 
@@ -950,7 +1156,7 @@ int ReadConfigFile(CurPos &cp) {
                 if (ReadEventMap(cp, EventMap, MapName) == -1) return -1;
             }
             break;
-
+#ifdef CONFIG_SYNTAX_HILIT
         case CF_COLORIZE:
             {
                 EColorize *Mode = 0;
@@ -972,7 +1178,7 @@ int ReadConfigFile(CurPos &cp) {
                     return -1;
             }
             break;
-
+#endif
         case CF_MODE:
             {
                 EMode *Mode = 0;
@@ -1029,25 +1235,37 @@ int ReadConfigFile(CurPos &cp) {
 }
 
 int LoadConfig(int /*argc*/, char ** /*argv*/, char *CfgFileName) {
+    STARTFUNC("LoadConfig");
+    LOG << "Config file: " << CfgFileName << ENDLINE;
+
     int fd, rc;
     char *buffer = 0;
     struct stat statbuf;
     CurPos cp;
 
     if ((fd = open(CfgFileName, O_RDONLY | O_BINARY)) == -1)
-        return -1;
+        ENDFUNCRC(-1);
     if (fstat(fd, &statbuf) != 0) {
         close(fd);
-        return -1;
+        ENDFUNCRC(-1);
     }
-    buffer = (char *) malloc(statbuf.st_size);
+
+    // check that we have enough room for signature (CONFIG_ID + VERNUM)
+    if (statbuf.st_size < (4+4)) {
+        close(fd);
+        DieError(0, "Bad .CNF signature");
+        ENDFUNCRC(-1);
+    }
+
+    buffer = (char *) malloc((size_t)statbuf.st_size);
     if (buffer == 0) {
         close(fd);
-        return -1;
+        ENDFUNCRC(-1);
     }
-    if (read(fd, buffer, statbuf.st_size) != statbuf.st_size) {
+    if (read(fd, buffer, (size_t)statbuf.st_size) != statbuf.st_size) {
         close(fd);
-        return -1;
+        free(buffer);
+        ENDFUNCRC(-1);
     }
     close(fd);
 
@@ -1058,16 +1276,19 @@ int LoadConfig(int /*argc*/, char ** /*argv*/, char *CfgFileName) {
     ln = (l[3] << 24) + (l[2] << 16) + (l[1] << 8) + l[0];
 
     if (ln != CONFIG_ID) {
+        free(buffer);
         DieError(0, "Bad .CNF signature");
-        return -1;
+        ENDFUNCRC(-1);
     }
 
     memcpy(l, buffer + 4, 4);
     ln = (l[3] << 24) + (l[2] << 16) + (l[1] << 8) + l[0];
 
     if (ln != VERNUM) {
+        LOG << std::hex << ln << " != " << VERNUM << ENDLINE;
+        free(buffer);
         DieError(0, "Bad .CNF version.");
-        return -1;
+        ENDFUNCRC(-1);
     }
 
     cp.name = CfgFileName;
@@ -1084,7 +1305,7 @@ int LoadConfig(int /*argc*/, char ** /*argv*/, char *CfgFileName) {
     if (rc == -1) {
         DieError(1, "Error %s offset %d\n", CfgFileName, cp.c - cp.a);
     }
-    return rc;
+    ENDFUNCRC(rc);
 }
 
 #include "defcfg.h"
