@@ -7,8 +7,19 @@
  *
  */
 
+#include "c_fconfig.h"
+#include "c_hilit.h"
+#include "c_mode.h"
+#include "console.h"
+#include "ftever.h"
+#include "s_files.h"
+#include "s_string.h"
+#include "sysdep.h"
+
+#include "c_commands.h"
+#include "c_cmdtab.h"
+
 #include <stdio.h>
-#include <string.h>
 #include <stdlib.h>
 #include <errno.h>
 #include <stdarg.h>
@@ -17,23 +28,15 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
-#include "feature.h"
-#include "ftever.h"
-#include "sysdep.h"
-#include "c_fconfig.h"
-#include "s_files.h"
-#include "c_mode.h"
-#include "console.h"
-#include "c_hilit.h"
 
 #define slen(s) ((s) ? (strlen(s) + 1) : 0)
 
-typedef struct {
+struct ExMacroCFTE {
     char *Name;
-} ExMacro;
+};
 
-static unsigned int CMacros = 0;
-static ExMacro *Macros = 0;
+static unsigned CMacrosCFTE = 0;
+static ExMacroCFTE *MacrosCFTE = 0;
 
 static FILE *output = 0;
 static int lntotal = 0;
@@ -43,17 +46,15 @@ static char XTarget[MAXPATH] = "";
 static char StartDir[MAXPATH] = "";
 static bool preprocess_only = false;
 
-#include "c_commands.h"
-#include "c_cmdtab.h"
 
-typedef struct _CurPos {
-    int sz;
+struct CurPos {
+    off_t sz;
     char *a;
     char *c;
     char *z;
     int line;
     const char *name; // filename
-} CurPos;
+};
 
 static void cleanup(int xerrno) {
     if (output)
@@ -78,7 +79,7 @@ static void Fail(CurPos &cp, const char *s, ...) {
 static int LoadFile(const char *WhereName, const char *CfgName, int Level = 1);
 static void DefineWord(const char *w);
 
-static void PutObject(CurPos &cp, int xtag, int xlen, void *obj) {
+static void PutObject(CurPos &cp, int xtag, size_t xlen, void *obj) {
     unsigned char tag = (unsigned char)xtag;
     unsigned short len = (unsigned short)xlen;
     unsigned char l[2];
@@ -86,8 +87,8 @@ static void PutObject(CurPos &cp, int xtag, int xlen, void *obj) {
     if (preprocess_only == false)
     {
 
-	l[0] = len & 0xFF;
-	l[1] = (len >> 8) & 0xFF;
+	l[0] = (char)len;
+	l[1] = (char)(len >> 8);
 
 	if (fwrite(&tag, 1, 1, output) != 1 ||
 	    fwrite(l, 2, 1, output) != 1 ||
@@ -182,11 +183,11 @@ int main(int argc, char **argv) {
 	    switch(n)
 	    {
 	    case 0:
-		strcpy(Source, argv[i]);
+		strlcpy(Source, argv[i], sizeof(Source));
 		break;
 
 	    case 1:
-                strcpy(Target, argv[i]);
+                strlcpy(Target, argv[i], sizeof(Target));
 		break;
 
 	    default:
@@ -202,22 +203,8 @@ int main(int argc, char **argv) {
 	fprintf(stderr, "No configuration file specified\n");
 	exit(1);
     }
-/*    if (strncmp(p, "-o", 2) == 0) {
-        p += 2;
-        offset = atol(p);
-        n++;
-    }
-    if (n == 1 && argc == 4) {
-        fprintf(stderr, "Invalid option '%s'\n", argv[1]);
-        exit(1);
-    }
-    strcpy(Source, argv[n++]);
-    strcpy(Target, "fte-new.cnf");
-    if (n < argc)
-        strcpy(Target, argv[n++]);
-	*/
 
-    JustDirectory(Target, XTarget);
+    JustDirectory(Target, XTarget, sizeof(XTarget));
     Slash(XTarget, 1);
 
     if (preprocess_only == false)
@@ -265,7 +252,7 @@ int main(int argc, char **argv) {
 #ifdef UNIX
                "."
 #endif
-               , StartDir);
+               , StartDir, sizeof(StartDir));
     Slash(StartDir, 1);
 
     if (preprocess_only == false)
@@ -273,7 +260,7 @@ int main(int argc, char **argv) {
         CurPos cp;
         char FSource[MAXPATH];
 
-        if (ExpandPath(Source, FSource) != 0) {
+        if (ExpandPath(Source, FSource, sizeof(FSource)) != 0) {
             fprintf(stderr, "Could not expand path %s\n", Source);
             exit(1);
         }
@@ -330,10 +317,10 @@ int main(int argc, char **argv) {
 #define COLORIZE_FLG(x) { #x, COL_##x }
 #define HILIT_CLR(x) { #x, CLR_##x }
 
-typedef struct _OrdLookup {
+struct OrdLookup {
     const char *Name;
     int num;
-} OrdLookup;
+};
 
 static const OrdLookup mode_num[] = {
 MODE_BFI(AutoIndent),
@@ -382,6 +369,7 @@ MODE_BFI(TrimOnSave),
 MODE_BFI(SaveBookmarks),
 MODE_BFI(HilitTags),
 MODE_BFI(ShowBookmarks),
+MODE_BFI(MakeBackups),
 { 0, 0 },
 };
 
@@ -453,6 +441,7 @@ MODE_FLG(LoadDesktopMode),
 MODE_FLG(IgnoreBufferList),
 MODE_FLG(ReassignModelIds),
 MODE_FLG(RecheckReadOnly),
+MODE_FLG(CursorBlink),
 { 0, 0 },
 };
 
@@ -466,7 +455,10 @@ MODE_FLG(HelpCommand),
 MODE_FLG(GUICharacters),
 MODE_FLG(CvsCommand),
 MODE_FLG(CvsLogMode),
+MODE_FLG(SvnCommand),
+MODE_FLG(SvnLogMode),
 MODE_FLG(XShellCommand),
+MODE_FLG(RGBColor),
 { 0, 0 },
 };
 
@@ -563,6 +555,7 @@ static int Lookup(const OrdLookup *where, char *what) {
 #define K_HWTYPE     20
 #define K_COLPALETTE 21
 #define K_CVSIGNRX   22
+#define K_SVNIGNRX   23
 
 typedef char Word[64];
 
@@ -589,6 +582,7 @@ static const OrdLookup CfgKW[] = {
 { "h_wtype", K_HWTYPE },
 { "submenucond", K_SUBMENUCOND },
 { "CvsIgnoreRx", K_CVSIGNRX },
+{ "SvnIgnoreRx", K_SVNIGNRX },
 { 0, 0 },
 };
 
@@ -675,8 +669,8 @@ static char *GetColor(CurPos &cp, char *name) {
         clr[2] = GetColor(cp, c)[2];
         clr[3] = 0;
 
-        memcpy((void *)color, (void *)clr, sizeof(color));
-        name = (char *)color;
+        memcpy(color, clr, sizeof(color));
+        name = color;
     } else {
         char *p = DefinedColor(name);
         if (!p)
@@ -811,7 +805,7 @@ static char *GetString(CurPos &cp) {
                     else if (*cp.c >= 'a' && *cp.c <= 'f') n = n * 16 + *cp.c - 'a' + 10;
                     else if (*cp.c >= 'A' && *cp.c <= 'F') n = n * 16 + *cp.c - 'A' + 10;
                     else cp.c--;
-                    *cp.c = n;
+                    *cp.c = (char)n;
                     break;
                 }
             }
@@ -835,7 +829,7 @@ static int GetNumber(CurPos &cp) {
     int value = 0;
     int neg = 0;
 
-    if (cp.c < cp.z && *cp.c == '-' || *cp.c == '+') {
+    if (cp.c < cp.z && (*cp.c == '-' || *cp.c == '+')) {
         if (*cp.c == '-') neg = 1;
         cp.c++;
     }
@@ -846,27 +840,24 @@ static int GetNumber(CurPos &cp) {
     return neg ? -value : value;
 }
 
-static int CmdNum(const char *Cmd) {
-    unsigned int i;
+static int CmdNumCFTE(const char *Cmd) {
 
-    for (i = 0;
-         i < sizeof(Command_Table) / sizeof(Command_Table[0]);
-         i++)
+    for (size_t i = 0; i < FTE_ARRAY_SIZE(Command_Table); ++i)
         if (strcmp(Cmd, Command_Table[i].Name) == 0)
             return Command_Table[i].CmdId;
-    for (i = 0; i < CMacros; i++)
-        if (Macros[i].Name && (strcmp(Cmd, Macros[i].Name)) == 0)
+    for (unsigned i = 0; i < CMacrosCFTE; i++)
+        if (MacrosCFTE[i].Name && (strcmp(Cmd, MacrosCFTE[i].Name)) == 0)
             return i | CMD_EXT;
     return 0; // Nop
 }
 
-int NewCommand(const char *Name) {
+static int NewCommandCFTE(const char *Name) {
     if (Name == 0)
         Name = "";
-    Macros = (ExMacro *) realloc(Macros, sizeof(ExMacro) * (1 + CMacros));
-    Macros[CMacros].Name = strdup(Name);
-    CMacros++;
-    return CMacros - 1;
+    MacrosCFTE = (ExMacroCFTE *) realloc(MacrosCFTE, sizeof(ExMacroCFTE) * (1 + CMacrosCFTE));
+    MacrosCFTE[CMacrosCFTE].Name = strdup(Name);
+    CMacrosCFTE++;
+    return CMacrosCFTE - 1;
 }
 
 static int ParseCommands(CurPos &cp, char *Name) {
@@ -874,7 +865,7 @@ static int ParseCommands(CurPos &cp, char *Name) {
     //    return 0;
     Word cmd;
     int p;
-    long Cmd = NewCommand(Name) | CMD_EXT;
+    long Cmd = NewCommandCFTE(Name) | CMD_EXT;
 
     long cnt;
     long ign = 0;
@@ -902,7 +893,7 @@ static int ParseCommands(CurPos &cp, char *Name) {
             long Command;
 
             if (GetWord(cp, cmd) == -1) Fail(cp, "Syntax error");
-            Command = CmdNum(cmd);
+            Command = CmdNumCFTE(cmd);
             if (Command == 0)
                 Fail(cp, "Unrecognised command: %s", cmd);
             PutNumber(cp, CF_COMMAND, Command);
@@ -957,7 +948,7 @@ static int ParseConfigFile(CurPos &cp) {
                     PutString(cp, CF_SUB, Name);
                     if (Parse(cp) != P_OPENBRACE) Fail(cp, "'{' expected");
                     GetOp(cp, P_OPENBRACE);
-                    if (ParseCommands(cp, strdup(Name ? Name : "")) == -1)
+                    if (ParseCommands(cp, strdup(Name)) == -1)
                         Fail(cp, "Parse failed");
                     PutNull(cp, CF_END);
                 }
@@ -1621,6 +1612,20 @@ static int ParseConfigFile(CurPos &cp) {
                                 GetOp(cp, P_EOS);
                             }
                             break;
+                        case K_SVNIGNRX:
+                            {
+                                char *regexp;
+
+                                if (Parse(cp) != P_ASSIGN) Fail(cp, "'=' expected");
+                                GetOp(cp, P_ASSIGN);
+                                if (Parse(cp) != P_STRING) Fail(cp, "String expected");
+                                regexp = GetString(cp);
+                                PutNull(cp, CF_SVNIGNRX);
+                                PutString(cp, CF_REGEXP, regexp);
+                                if (Parse(cp) != P_EOS) Fail(cp, "';' expected");
+                                GetOp(cp, P_EOS);
+                            }
+                            break;
                         default:  // mode::
                             if (Parse(cp) != P_ASSIGN) Fail(cp, "'=' expected");
                             GetOp(cp, P_ASSIGN);
@@ -1646,7 +1651,7 @@ static int ParseConfigFile(CurPos &cp) {
                                     s = GetString(cp);
                                     if (s == 0) Fail(cp, "Parse failed");
                                     var = Lookup(global_string, w);
-                                    if (var == -1) Fail(cp, "Lookup of '%s' failed");
+                                    if (var == -1) Fail(cp, "Lookup of '%s' failed", w);
                                     PutNumber(cp, CF_SETVAR, var);
                                     PutString(cp, CF_STRING, s);
                                 }
@@ -1872,14 +1877,15 @@ static int LoadFile(const char *WhereName, const char *CfgName, int Level) {
 
     //fprintf(stderr, "Loading file %s %s\n", WhereName, CfgName);
 
-    JustDirectory(WhereName, last);
+    JustDirectory(WhereName, last, sizeof(last));
 
     if (IsFullPath(CfgName)) {
-        strcpy(Cfg, CfgName);
+        strlcpy(Cfg, CfgName, sizeof(Cfg));
     } else {
         // here we will try relative to a number of places.
         // 1. User's .fte directory.
         // 2. System's "local config" directory.
+        // 3. /usr/share/fte (FHS compliant - from Gentoo)
         // 3. Initial file's directory.
         // 4. Current directory.
         // This means that a user's directory will always win out,
@@ -1891,39 +1897,45 @@ static int LoadFile(const char *WhereName, const char *CfgName, int Level) {
         // 1. User's .fte directory.
         char tmp[MAXPATH];
         sprintf(tmp, "~/.fte/%s", CfgName);
-        ExpandPath(tmp, Cfg);
+        ExpandPath(tmp, Cfg, sizeof(Cfg));
         //fprintf(stderr, "Looking for %s\n", Cfg);
         if (!FileExists(Cfg))
         {
-            // Okay, now try "local config".
+            // 2. try "local config".
             sprintf(tmp, "%slocalconfig/%s", StartDir, CfgName);
-            ExpandPath(tmp, Cfg);
+            ExpandPath(tmp, Cfg, sizeof(Cfg));
             //fprintf(stderr, "Looking for %s\n", Cfg);
             if (!FileExists(Cfg))
             {
-                sprintf(tmp, "%sconfig/%s", StartDir, CfgName);
-                ExpandPath(tmp, Cfg);
-                //fprintf(stderr, "Looking for %s\n", Cfg);
+                // 3. /usr/share/fte
+                sprintf(tmp, "/usr/share/fte/%s", CfgName);
+                ExpandPath(tmp, Cfg, sizeof(Cfg));
                 if (!FileExists(Cfg))
                 {
-                    sprintf(tmp, "./%s", CfgName);
-                    ExpandPath(tmp, Cfg);
+                    sprintf(tmp, "%sconfig/%s", StartDir, CfgName);
+                    ExpandPath(tmp, Cfg, sizeof(Cfg));
                     //fprintf(stderr, "Looking for %s\n", Cfg);
                     if (!FileExists(Cfg))
                     {
-                        fprintf(stderr, "Cannot find '%s' in:\n"
-                                "\t~/.fte,\n""\t%slocalconfig,\n"
-                                "\t%sconfig, or\n"
-                                "\t.",
-                                CfgName, StartDir, StartDir);
+                        sprintf(tmp, "./%s", CfgName);
+                        ExpandPath(tmp, Cfg, sizeof(Cfg));
+                        //fprintf(stderr, "Looking for %s\n", Cfg);
+                        if (!FileExists(Cfg))
+                        {
+                            fprintf(stderr, "Cannot find '%s' in:\n"
+                                    "\t~/.fte,\n""\t%slocalconfig,\n\t/usr/share/fte,\n"
+                                    "\t%sconfig, or\n"
+                                    "\t.",
+                                    CfgName, StartDir, StartDir);
+                        }
                     }
                 }
             }
         }
 #else // UNIX
         SlashDir(last);
-        strcat(last, CfgName);
-        ExpandPath(last, Cfg);
+        strlcat(last, CfgName, sizeof(last));
+        ExpandPath(last, Cfg, sizeof(Cfg));
 #endif // UNIX
     }
     // puts(Cfg);
@@ -1938,7 +1950,7 @@ static int LoadFile(const char *WhereName, const char *CfgName, int Level) {
         fprintf(stderr, "Cannot stat '%s', errno=%d\n", Cfg, errno);
         return -1;
     }
-    buffer = (char *) malloc(statbuf.st_size+1);
+    buffer = (char *) malloc((size_t)statbuf.st_size+1);
     if (buffer == NULL) {
         close(fd);
         return -1;
@@ -1946,7 +1958,7 @@ static int LoadFile(const char *WhereName, const char *CfgName, int Level) {
 
     buffer[statbuf.st_size] = 0; // add null to end of buffer, NOTE: allocated statbuf.st_size + 1
 
-    if (read(fd, buffer, statbuf.st_size) != statbuf.st_size) {
+    if (read(fd, buffer, (size_t)statbuf.st_size) != statbuf.st_size) {
         close(fd);
         free(buffer);
         return -1;
