@@ -9,10 +9,10 @@
 
 #include "fte.h"
 
-int GetPMClip() {
+int GetPMClip(int clipboard) {
     return 0;
 }
 
-int PutPMClip() {
+int PutPMClip(int clipboard) {
     return 0;
 }
