@@ -10,19 +10,21 @@
 
 // include
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <assert.h>
-#include <signal.h>
-
 #include "sysdep.h"
 #include "console.h"
+#include "c_config.h"
 #include "gui.h"
+#include "s_string.h"
 
-#include <stdlib.h>
 #include <process.h>
-#include	"port.h"                // DOS portability calls,
+#include "port.h"                // DOS portability calls,
+
+#include <stdio.h>
+#include <stdlib.h>
+#include <signal.h>
+#ifdef DJGPP
+#include <sys/stat.h>
+#endif
 
 #define MAX_PIPES 4
 #define PIPE_BUFLEN 4096
@@ -30,31 +32,222 @@
 typedef struct {
 	int used;
 	int id;
-	int reading, stopped;
+//	int reading;
+	int stopped;
 //	TID tid;
 //	HMTX Access;
 //	HEV ResumeRead;
 //	HEV NewData;
-	char *buffer;
-	int buflen;
-	int bufused;
-	int bufpos;
+//	char *buffer;
+//	int buflen;
+//	int bufused;
+//	int bufpos;
 	EModel *notify;
-	char *Command;
-	int RetCode;
-	int DoTerm;
+//	char *Command;
+//	int RetCode;
+//	int DoTerm;
+        FILE *fp;
 } GPipe;
 
 static GPipe Pipes[MAX_PIPES] = {
 	{ 0 }, { 0 }, { 0 }, { 0 }
 };
 
+
+
+#if defined(__DJGPP__)
+
+// adapted from djgpp library source - redirects stderr as well as stdout
+// Original author: pacetti@fl-ngnet.army.mil
+
+#include <io.h>
+#include <errno.h>
+#include <unistd.h>
+
+/* hold file pointer, descriptor, command, mode, temporary file name,
+   and the status of the command  */
+struct pipe_list {
+  FILE *fp;
+  int fd;
+  int exit_status;
+  char *command, mode[10], temp_name[L_tmpnam];
+  struct pipe_list *next;
+};
+
+/* static, global list pointer */
+static struct pipe_list *pl = NULL;
+
+static FILE *
+xpopen (const char *cm, const char *md) /* program name, pipe mode */
+{
+  struct pipe_list *l1, *l2;
+
+  /* make new node */
+  if ((l1 = (struct pipe_list *) malloc (sizeof (struct pipe_list))) == NULL)
+    return NULL;
+
+  /* zero out elements to we'll get here */
+  l1->fd = 0;
+  l1->fp = NULL;
+  l1->next = NULL;
+
+  /* if empty list - just grab new node */
+  if (!pl)
+    pl = l1;
+  else
+  {
+    /* otherwise, find last node in list */
+    ++(l1->fd);
+    l2 = pl;
+    while (l2->next)
+    {
+      ++(l1->fd);
+      l2 = l2->next;
+    };
+    /* add new node to list */
+    l2->next = l1;
+  }
+
+  /* stick in elements we know already */
+  l1->exit_status = -1;
+  strcpy (l1->mode, md);
+  if (tmpnam (l1->temp_name) == NULL)
+    return NULL;
+
+  /* if can save the program name, build temp file */
+  if ((l1->command = (char *) malloc(strlen(cm)+1)))
+  {
+    strcpy(l1->command, cm);
+    /* if caller wants to read */
+    if (l1->mode[0] == 'r')
+    {
+#if 1
+      int fd2 = -1;
+#endif
+      /* dup stdout */
+      if ((l1->fd = dup (fileno (stdout))) == EOF)
+	l1->fp = NULL;
+      else if (!(l1->fp = freopen (l1->temp_name, "wb", stdout)))
+	l1->fp = NULL;
+#if 1
+      /* dup stderr */
+      else if ((fd2 = dup (STDERR_FILENO)) == EOF)
+	l1->fp = NULL;
+      /* redirect stderr to new stdout */
+      else if (dup2(fileno(l1->fp),STDERR_FILENO) == EOF)
+        l1->fp = NULL;
+#endif
+      else
+	/* exec cmd */
+	if ((l1->exit_status = system (cm)) == EOF)
+	  l1->fp = NULL;
+      /* reopen real stdout */
+      if (dup2 (l1->fd, fileno (stdout)) == EOF)
+	l1->fp = NULL;
+      else
+	/* open file for reader */
+	l1->fp = fopen (l1->temp_name, l1->mode);
+      close(l1->fd);
+#if 1
+      /* restore stderr */
+      if (fd2 >= 0) {
+        (void) dup2(fd2, STDERR_FILENO);
+	(void) close(fd2);
+      }
+#endif
+    }
+    else
+      /* if caller wants to write */
+      if (l1->mode[0] == 'w')
+        /* open temp file */
+        l1->fp = fopen (l1->temp_name, l1->mode);
+      else
+        /* unknown mode */
+        l1->fp = NULL;
+  }
+  return l1->fp;              /* return == NULL ? ERROR : OK */
+}
+
+static int
+xpclose (FILE *pp)
+{
+  struct pipe_list *l1, *l2;    /* list pointers */
+  int retval=0;			/* function return value */
+
+  /* if pointer is first node */
+  if (pl->fp == pp)
+  {
+    /* save node and take it out the list */
+    l1 = pl;
+    pl = l1->next;
+  }
+  else
+    /* if more than one node in list */
+    if (pl->next)
+    {
+      /* find right node */
+      for (l2 = pl, l1 = pl->next; l1; l2 = l1, l1 = l2->next)
+        if (l1->fp == pp)
+          break;
+
+      /* take node out of list */
+      l2->next = l1->next;
+    }
+    else
+      return -1;
+
+  /* if FILE not in list - return error */
+  if (l1->fp == pp)
+  {
+    /* close the (hopefully) popen()ed file */
+    fclose (l1->fp);
+
+    /* if pipe was opened to write */
+    if (l1->mode[0] == 'w')
+    {
+      /* dup stdin */
+      if ((l1->fd = dup (fileno (stdin))) == EOF)
+	retval = -1;
+      else
+	/* open temp stdin */
+	if (!(l1->fp = freopen (l1->temp_name, "rb", stdin)))
+	  retval = -1;
+	else
+	  /* exec cmd */
+          if ((retval = system (l1->command)) != EOF)
+	  {
+            /* reopen stdin */
+	    if (dup2 (l1->fd, fileno (stdin)) == EOF)
+	      retval = -1;
+	  }
+      close(l1->fd);
+    }
+    else
+      /* if pipe was opened to read, return the exit status we saved */
+      if (l1->mode[0] == 'r')
+        retval = l1->exit_status;
+      else
+        /* invalid mode */
+        retval = -1;
+  }
+  remove (l1->temp_name);       /* remove temporary file */
+  free (l1->command);           /* dealloc memory */
+  free (l1);                    /* dealloc memory */
+  l1 = NULL;                    /* make pointer bogus */
+
+  return retval;              /* retval==0 ? OK : ERROR */
+}
+
+#endif /* defined(__DJGPP__) */
+
+
+
 static long MouseAutoDelay = 400;
 static long MouseAutoRepeat = 5;
 static long MouseMultiClick = 300;
 
-static Initialized = 0;
-static MousePresent = 0;
+static int Initialized = 0;
+static int MousePresent = 0;
 static int CursorVisible = 1; /* 1 means visible */
 static int MouseVisible = 0; /* 0 means hidden */
 static TEvent MouseEv = { evNone };
@@ -76,81 +269,80 @@ static void DrawMouse(int Show)
 		MOUSCursen(FALSE);
 }
 
+// *INDENT-OFF*
 static struct { // TransCharScan
-	unsigned short CharScan;
-	TKeyCode KeyCode;
+    unsigned short CharScan;
+    TKeyCode KeyCode;
 } TransCharScan[] = {
-	{ 0x0100, kbEsc },					   { 0x011B, kbEsc },
-	{ 0x1C0D, kbEnter },				   { 0x1C0A, kbEnter },
-	{ 0x1C00, kbEnter },				   { 0xE00D, kbEnter | kfGray },
-	{ 0xA600, kbEnter | kfGray },		   { 0xE00A, kbEnter | kfGray },
-	{ 0x0E08, kbBackSp },				   { 0x0E7F, kbBackSp },
-	{ 0x0E00, kbBackSp },				   { 0x0F09, kbTab },
-	{ 0x9400, kbTab },					   { 0xA500, kbTab },
-	{ 0x0F00, kbTab },					   { 0x4E00, '+' | kfGray },
-	{ 0x9000, '+' | kfGray },              { 0x4E2B, '+' | kfGray },
-	{ 0x4A00, '-' | kfGray },              { 0x8E00, '-' | kfGray },
-	{ 0x4A2D, '-' | kfGray },              { 0x3700, '*' | kfGray },
-	{ 0x9600, '*' | kfGray },              { 0x372A, '*' | kfGray },
-	{ 0xE02F, '/' | kfGray },              { 0xA400, '/' | kfGray },
-	{ 0x9500, '/' | kfGray },              { 0x0300, 0 }
+    { 0x0100, kbEsc },           { 0x011B, kbEsc },
+    { 0x1C0D, kbEnter },         { 0x1C0A, kbEnter },
+    { 0x1C00, kbEnter },         { 0xE00D, kbEnter | kfGray },
+    { 0xA600, kbEnter | kfGray },{ 0xE00A, kbEnter | kfGray },
+    { 0x0E08, kbBackSp },        { 0x0E7F, kbBackSp },
+    { 0x0E00, kbBackSp },        { 0x0F09, kbTab },
+    { 0x9400, kbTab },           { 0xA500, kbTab },
+    { 0x0F00, kbTab },           { 0x4E00, '+' | kfGray },
+    { 0x9000, '+' | kfGray },    { 0x4E2B, '+' | kfGray },
+    { 0x4A00, '-' | kfGray },    { 0x8E00, '-' | kfGray },
+    { 0x4A2D, '-' | kfGray },    { 0x3700, '*' | kfGray },
+    { 0x9600, '*' | kfGray },    { 0x372A, '*' | kfGray },
+    { 0xE02F, '/' | kfGray },    { 0xA400, '/' | kfGray },
+    { 0x9500, '/' | kfGray },    { 0x0300, 0 }
 };
 
 static struct { // TransScan
-	int ScanCode;
-	TKeyCode KeyCode;
+    int ScanCode;
+    TKeyCode KeyCode;
 } TransScan[] = {
-	{ 0x78, '1' }, { 0x79, '2' }, { 0x7A, '3' }, { 0x7B, '4' }, { 0x7C, '5' },
-	{ 0x7D, '6' }, { 0x7E, '7' }, { 0x7F, '8' }, { 0x80, '9' }, { 0x81, '0' },
+    { 0x78, '1' }, { 0x79, '2' }, { 0x7A, '3' }, { 0x7B, '4' }, { 0x7C, '5' },
+    { 0x7D, '6' }, { 0x7E, '7' }, { 0x7F, '8' }, { 0x80, '9' }, { 0x81, '0' },
 
-	{ 0x10, 'Q' }, { 0x11, 'W' }, { 0x12, 'E' }, { 0x13, 'R' }, { 0x14, 'T' },
-	{ 0x15, 'Y' }, { 0x16, 'U' }, { 0x17, 'I' }, { 0x18, 'O' }, { 0x19, 'P' },
+    { 0x10, 'Q' }, { 0x11, 'W' }, { 0x12, 'E' }, { 0x13, 'R' }, { 0x14, 'T' },
+    { 0x15, 'Y' }, { 0x16, 'U' }, { 0x17, 'I' }, { 0x18, 'O' }, { 0x19, 'P' },
 
-	{ 0x1E, 'A' }, { 0x1F, 'S' }, { 0x20, 'D' }, { 0x21, 'F' }, { 0x22, 'G' },
-	{ 0x23, 'H' }, { 0x24, 'J' }, { 0x25, 'K' }, { 0x26, 'L' },
+    { 0x1E, 'A' }, { 0x1F, 'S' }, { 0x20, 'D' }, { 0x21, 'F' }, { 0x22, 'G' },
+    { 0x23, 'H' }, { 0x24, 'J' }, { 0x25, 'K' }, { 0x26, 'L' }, { 0x2C, 'Z' },
+    { 0x2D, 'X' }, { 0x2E, 'C' }, { 0x2F, 'V' }, { 0x30, 'B' }, { 0x31, 'N' },
+    { 0x32, 'M' }, { 0x29, '`' }, { 0x82, '-' }, { 0x83, '=' }, { 0x2B, '\\' },
+    { 0x1A, '[' }, { 0x1B, ']' }, { 0x27, ';' }, { 0x28, '\'' }, { 0x33, ',' },
+    { 0x34, '.' }, { 0x35, '/' }, { 0x37, '*' }, { 0x4E, '+' }, { 0x4A, '-' },
 
-	{ 0x2C, 'Z' }, { 0x2D, 'X' }, { 0x2E, 'C' }, { 0x2F, 'V' }, { 0x30, 'B' },
-	{ 0x31, 'N' }, { 0x32, 'M' },
+    { 0x3B, kbF1    }, { 0x3C, kbF2     }, { 0x3D, kbF3    },
+    { 0x3E, kbF4    }, { 0x3F, kbF5     }, { 0x40, kbF6    },
+    { 0x41, kbF7    }, { 0x42, kbF8     }, { 0x43, kbF9    },
+    { 0x44, kbF10   }, { 0x85, kbF11    }, { 0x86, kbF12   },
 
-	{ 0x29, '`' }, { 0x82, '-' }, { 0x83, '=' }, { 0x2B, '\\' }, { 0x1A, '[' },
-	{ 0x1B, ']' }, { 0x27, ';' }, { 0x28, '\'' }, { 0x33, ',' }, { 0x34, '.' },
-	{ 0x35, '/' }, { 0x37, '*' }, { 0x4E, '+' }, { 0x4A, '-' },
+    { 0x54, kbF1    }, { 0x55, kbF2     }, { 0x56, kbF3    },
+    { 0x57, kbF4    }, { 0x58, kbF5     }, { 0x59, kbF6    },
+    { 0x5A, kbF7    }, { 0x5B, kbF8     }, { 0x5C, kbF9    },
+    { 0x5D, kbF10   }, { 0x87, kbF11    }, { 0x88, kbF12   },
 
-	{ 0x3B, kbF1	},	{ 0x3C, kbF2	},	{ 0x3D, kbF3	},
-	{ 0x3E, kbF4	},	{ 0x3F, kbF5	},	{ 0x40, kbF6	},
-	{ 0x41, kbF7	},	{ 0x42, kbF8	},	{ 0x43, kbF9	},
-	{ 0x44, kbF10	},	{ 0x85, kbF11	},	{ 0x86, kbF12	},
+    { 0x5E, kbF1    }, { 0x5F, kbF2     }, { 0x60, kbF3    },
+    { 0x61, kbF4    }, { 0x62, kbF5     }, { 0x63, kbF6    },
+    { 0x64, kbF7    }, { 0x65, kbF8     }, { 0x66, kbF9    },
+    { 0x67, kbF10   }, { 0x89, kbF11    }, { 0x8A, kbF12   },
 
-	{ 0x54, kbF1	},	{ 0x55, kbF2	},	{ 0x56, kbF3	},
-	{ 0x57, kbF4	},	{ 0x58, kbF5	},	{ 0x59, kbF6	},
-	{ 0x5A, kbF7	},	{ 0x5B, kbF8	},	{ 0x5C, kbF9	},
-	{ 0x5D, kbF10	},	{ 0x87, kbF11	},	{ 0x88, kbF12	},
+    { 0x68, kbF1    }, { 0x69, kbF2     }, { 0x6A, kbF3    },
+    { 0x6B, kbF4    }, { 0x6C, kbF5     }, { 0x6D, kbF6    },
+    { 0x6E, kbF7    }, { 0x6F, kbF8     }, { 0x70, kbF9    },
+    { 0x71, kbF10   }, { 0x8B, kbF11    }, { 0x8C, kbF12   },
 
-	{ 0x5E, kbF1	},	{ 0x5F, kbF2	},	{ 0x60, kbF3	},
-	{ 0x61, kbF4	},	{ 0x62, kbF5	},	{ 0x63, kbF6	},
-	{ 0x64, kbF7	},	{ 0x65, kbF8	},	{ 0x66, kbF9	},
-	{ 0x67, kbF10	},	{ 0x89, kbF11	},	{ 0x8A, kbF12	},
+    { 0x47, kbHome  }, { 0x48, kbUp     }, { 0x49, kbPgUp  },
+    { 0x4B, kbLeft  }, { 0x4C, kbCenter }, { 0x4D, kbRight },
+    { 0x4F, kbEnd   }, { 0x50, kbDown   }, { 0x51, kbPgDn  },
+    { 0x52, kbIns   }, { 0x53, kbDel    }, { 0x77, kbHome  },
 
-	{ 0x68, kbF1	},	{ 0x69, kbF2	},	{ 0x6A, kbF3	},
-	{ 0x6B, kbF4	},	{ 0x6C, kbF5	},	{ 0x6D, kbF6	},
-	{ 0x6E, kbF7	},	{ 0x6F, kbF8	},	{ 0x70, kbF9	},
-	{ 0x71, kbF10	},	{ 0x8B, kbF11	},	{ 0x8C, kbF12	},
+    { 0x8D, kbUp    }, { 0x84, kbPgUp   }, { 0x73, kbLeft  },
+    { 0x74, kbRight }, { 0x75, kbEnd    }, { 0x91, kbDown  },
+    { 0x76, kbPgDn  }, { 0x92, kbIns    }, { 0x93, kbDel   },
 
-	{ 0x47, kbHome	},	{ 0x48, kbUp	},	{ 0x49, kbPgUp	},
-	{ 0x4B, kbLeft	},	{ 0x4C, kbCenter},	{ 0x4D, kbRight },
-	{ 0x4F, kbEnd	},	{ 0x50, kbDown	},	{ 0x51, kbPgDn	},
-	{ 0x52, kbIns	},	{ 0x53, kbDel	},
-
-	{ 0x77, kbHome	},	{ 0x8D, kbUp	},	{ 0x84, kbPgUp	},
-	{ 0x73, kbLeft	},						{ 0x74, kbRight },
-	{ 0x75, kbEnd	},	{ 0x91, kbDown	},	{ 0x76, kbPgDn	},
-	{ 0x92, kbIns	},	{ 0x93, kbDel	},
-
-	{ 0x97, kbHome	| kfGray },  { 0x98, kbUp	 | kfGray },  { 0x99, kbPgUp  | kfGray },
-	{ 0x9B, kbLeft	| kfGray }, 							  { 0x9D, kbRight | kfGray },
-	{ 0x9F, kbEnd	| kfGray },  { 0xA0, kbDown  | kfGray },  { 0xA1, kbPgDn  | kfGray },
-	{ 0xA2, kbIns	| kfGray },  { 0xA3, kbDel	 | kfGray }
+    { 0x97, kbHome  | kfGray }, { 0x98, kbUp    | kfGray },
+    { 0x99, kbPgUp  | kfGray }, { 0xA1, kbPgDn  | kfGray },
+    { 0x9B, kbLeft  | kfGray }, { 0x9D, kbRight | kfGray },
+    { 0x9F, kbEnd   | kfGray }, { 0xA0, kbDown  | kfGray },
+    { 0xA2, kbIns   | kfGray }, { 0xA3, kbDel   | kfGray }
 };
+// *INDENT-ON*
 
 int ReadKbdEvent(TEvent *Event, int Wait)
 {
@@ -161,6 +353,8 @@ int ReadKbdEvent(TEvent *Event, int Wait)
 	UWORD CharScan, Flags;
 	static ULONG PrevFlags = 0;
 	int I;
+	const int Tcs = (int) (sizeof(TransCharScan)/sizeof(TransCharScan[0]));
+	const int Ts = (int) (sizeof(TransScan)/sizeof(TransScan[0]));
 
 	Event->What = evNone;
 	if(! plKbdReadF(&ki)) return 0; 		// No data-> no read..
@@ -181,7 +375,7 @@ int ReadKbdEvent(TEvent *Event, int Wait)
 	/*	  cpCount = sizeof(cpList);*/
 	/*	  rc = DosQueryCp(sizeof(cpList), cpList, &cpCount);  // get active code page*/
 	if (CharCode != 0) {
-		if ((Flags & PLKF_ALT) != 0) KeyFlags |= kfAlt;
+		// if ((Flags & PLKF_ALT) != 0) KeyFlags |= kfAlt;
 	} else {
 		if((Flags & PLKF_ALT) != 0) KeyFlags |= kfAlt;
 	}
@@ -193,7 +387,7 @@ int ReadKbdEvent(TEvent *Event, int Wait)
 		KeyCode = CharCode;
 		KeyFlags |= kfAltXXX;
 	} else { /* now check special combinations */
-		for (I = 0; I < sizeof(TransCharScan)/sizeof(TransCharScan[0]); I++)
+		for (I = 0; I < Tcs; I++)
 			if (TransCharScan[I].CharScan == CharScan) {
 				KeyCode = TransCharScan[I].KeyCode;
 				break;
@@ -202,7 +396,7 @@ int ReadKbdEvent(TEvent *Event, int Wait)
 			if ((CharCode == 0) || (CharCode == 0xE0)) {
 				if (CharCode == 0xE0)
 					KeyFlags |= kfGray;
-				for (I = 0; I < sizeof(TransScan)/sizeof(TransScan[0]); I++)
+				for (I = 0; I < Ts; I++)
 					if (TransScan[I].ScanCode == ScanCode) {
 						KeyCode = TransScan[I].KeyCode;
 						break;
@@ -641,14 +835,18 @@ int ConSuspend()
 {
 	plScnSetFlash(TRUE);					// Set "flash" mode, not bright mode
 	ConHideMouse();
-	signal(SIGBREAK, SIG_DFL);
+#if defined(SIGBREAK)
+  	signal(SIGBREAK, SIG_DFL);
+#endif
 	signal(SIGINT, SIG_DFL);
 	return 0;
 }
 
 int ConContinue()
 {
-	signal(SIGBREAK, SIG_IGN);
+#if defined(SIGBREAK)
+  	signal(SIGBREAK, SIG_IGN);
+#endif
 	signal(SIGINT, SIG_IGN);
 	plScnSetFlash(FALSE);					// Set "bright" mode, not flashing
 	ConShowMouse();
@@ -656,7 +854,25 @@ int ConContinue()
 }
 
 int GetPipeEvent(TEvent *Event) {
-	return 0;
+#ifdef DJGPP
+    int i;
+
+    Event->What = evNone;
+    for (i = 0; i < MAX_PIPES; i++) {
+        if (Pipes[i].used == 0) continue;
+        if (Pipes[i].notify == 0) continue;
+        if (1) {
+            //fprintf(stderr, "Pipe New Data: %d\n", i);
+            Event->What = evNotify;
+            Event->Msg.View = 0;
+            Event->Msg.Model = Pipes[i].notify;
+            Event->Msg.Command = cmPipeRead;
+            Event->Msg.Param1 = i;
+            return 1;
+        }
+    }
+#endif
+    return 0;
 }
 
 int ConGetEvent(TEventMask EventMask, TEvent *Event, int WaitTime, int Delete)
@@ -735,25 +951,43 @@ int RestoreScreen() {
 }
 
 
-GUI::GUI(int&, char**, int XSize, int YSize) {
+GUI::GUI(int &argc, char **argv, int XSize, int YSize) {
+	fArgc = argc;
+	fArgv = argv;
 	::ConInit(-1, -1);
 	SaveScreen();
 	::ConSetSize(XSize, YSize);
 	gui = this;
+#ifdef DJGPP
+	// speed up directory access by turning off unused stat functionality
+	_djstat_flags |= _STAT_INODE;
+	_djstat_flags |= _STAT_EXEC_EXT;
+	_djstat_flags |= _STAT_EXEC_MAGIC;
+	_djstat_flags |= _STAT_DIRSIZE;
+	_djstat_flags |= _STAT_ROOT_TIME;
+	_djstat_flags |= _STAT_WRITEBIT;
+#endif
 }
 
 GUI::~GUI() {
 	RestoreScreen();
 	::ConDone();
-	gui = 0;
+
+        if(SavedScreen)
+        {
+            free(SavedScreen);
+            SavedScreen = 0;
+        }
+
+        gui = 0;
 }
 
-int GUI::ConSuspend(void) {
+int GUI::ConSuspend() {
 	RestoreScreen();
 	return ::ConSuspend();
 }
 
-int GUI::ConContinue(void) {
+int GUI::ConContinue() {
 	SaveScreen();
 	return ::ConContinue();
 }
@@ -777,23 +1011,78 @@ int GUI::ShowEntryScreen() {
 static void PipeThread(void *p) {
 }
 
-int GUI::OpenPipe(char *Command, EModel *notify) {
-	return 0;
+int GUI::OpenPipe(const char *Command, EModel *notify) {
+#ifdef DJGPP
+    int i;
+
+    for (i = 0; i < MAX_PIPES; i++) {
+        if (Pipes[i].used == 0) {
+            Pipes[i].id = i;
+            Pipes[i].notify = notify;
+            Pipes[i].stopped = 1;
+
+            Pipes[i].fp = xpopen(Command,"r");
+            if (Pipes[i].fp == NULL)
+                return -1;
+            Pipes[i].used = 1;
+            //fprintf(stderr, "Pipe Open: %d\n", i);
+            return i;
+        }
+    }
+    return -1;
+#else
+    return 0;
+#endif
 }
 
 int GUI::SetPipeView(int id, EModel *notify) {
-	return 0;
+#ifdef DJGPP
+    if (id < 0 || id >= MAX_PIPES)
+        return -1;
+    if (Pipes[id].used == 0)
+        return -1;
+    //fprintf(stderr, "Pipe View: %d %08X\n", id, notify);
+    Pipes[id].notify = notify;
+#endif
+    return 0;
 }
 
-int GUI::ReadPipe(int id, void *buffer, int len) {
-	return 0;
+ssize_t GUI::ReadPipe(int id, void *buffer, size_t len) {
+#ifdef DJGPP
+    ssize_t rc;
+    if (id < 0 || id >= MAX_PIPES)
+        return -1;
+    if (Pipes[id].used == 0)
+        return -1;
+    //fprintf(stderr, "Pipe Read: Get %d %d\n", id, len);
+    
+    rc = fread(buffer,1,len,Pipes[id].fp);
+    //fprintf(stderr, "Pipe Read: Got %d %d\n", id, rc);
+    if (ferror(Pipes[id].fp)) {
+        Pipes[id].stopped = 1;
+        return 0;
+    }
+    return rc == 0 ? -1 : rc;
+#else
+    return 0;
+#endif
 }
 
 int GUI::ClosePipe(int id) {
-	return 0;
+#ifdef DJGPP
+    if (id < 0 || id >= MAX_PIPES)
+        return -1;
+    if (Pipes[id].used == 0)
+        return -1;
+    Pipes[id].used = 0;
+    //fprintf(stderr, "Pipe Close: %d\n", id);
+    return xpclose(Pipes[id].fp);
+#else
+    return 0;
+#endif
 }
 
-int GUI::RunProgram(char *Command) {
+int GUI::RunProgram(int mode, char *Command) {
 	int rc, W, H, W1, H1;
 
 	ConQuerySize(&W, &H);
@@ -818,13 +1107,13 @@ int GUI::RunProgram(char *Command) {
 	return rc;
 }
 
-int ConSetTitle(char *Title, char *STitle) {
+int ConSetTitle(const char *Title, const char *STitle) {
 	return 0;
 }
 
-int ConGetTitle(char *Title, int MaxLen, char *STitle, int SMaxLen) {
-	strcpy(Title, "FTE");
-	strcpy(STitle, "FTE");
+int ConGetTitle(char *Title, size_t MaxLen, char *STitle, size_t SMaxLen) {
+	strlcpy(Title, "FTE", MaxLen);
+	strlcpy(STitle, "FTE", SMaxLen);
 	return 0;
 }
 
@@ -832,16 +1121,19 @@ int ConCursorVisible() {
 	return 0;
 }
 
-int ConPutEvent(TEvent Event)
+int ConPutEvent(const TEvent& Event)
 {
 	EventBuf = Event;
 	return 0;
 }
 
-char ConGetDrawChar(int index) {
-	static char tab[] = "ڿĳô\x1A";
+char ConGetDrawChar(unsigned int index) {
+    static const char *tab = NULL;
 
-	assert(index >= 0 && index < strlen(tab));
+    if (!tab)
+        tab = GetGUICharacters ("DOS","\xDA\xBF\xC0\xD9\xC4\xB3\xC2\xC3\xB4\xC1\xC5\x1A\xFA\x04\xC4\x18\x19\B1\xB0\x1B\x1A");
 
-	return tab[index];
+    assert(index < strlen(tab) && strlen(tab) > 20);
+
+    return tab[index];
 }
