@@ -23,58 +23,59 @@
 
 // FIX: caps lock is ignored (I haven't pressed it in years, except by mistake).
 // FIX: access GPM clipboard. how?
+// Also: num lock is ignored
 
 // ... some more comments below
 
+#include "c_config.h"
+#include "console.h"
+#include "gui.h"
+#include "s_string.h"
+#include "sysdep.h"
 
-#include <signal.h>
-#include <assert.h>
-#include <termios.h>
-#include <stdio.h>
-#include <fcntl.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <unistd.h>
-#include <string.h>
-#include <sys/types.h>
-#include <sys/ioctl.h>
-#include <sys/vt.h>
-#include <sys/kd.h>
-#include <sys/time.h>
-#include <sys/wait.h>
-#include <sys/stat.h>
-#include <ctype.h>
-#include <linux/tty.h>
-#include <linux/major.h>
-#include <linux/kdev_t.h>
-#include <linux/kd.h>
-#include <linux/keyboard.h>
 #ifdef USE_GPM
 extern "C" {
 #include <gpm.h>
 }
 #endif
 
-#include "console.h"
-#include "gui.h"
+#include <ctype.h>
+#include <errno.h>
+#include <fcntl.h>
+#include <signal.h>
+#include <stdarg.h>
+#include <stdio.h>
+#include <sys/ioctl.h>
+#include <sys/kd.h>
+#include <sys/stat.h>
+#include <sys/time.h>
+#include <sys/vt.h>
+#include <sys/wait.h>
+#include <termios.h>
+#include <unistd.h>
+
+#include <linux/tty.h>
+#include <linux/major.h>
+#include <linux/kdev_t.h>
+#include <linux/kd.h>
+#include <linux/keyboard.h>
+
 
 #define MAX_PIPES 4
 //#define PIPE_BUFLEN 4096
 
-typedef struct {
+struct GPipe {
     int used;
     int id;
     int fd;
     int pid;
     int stopped;
     EModel *notify;
-} GPipe;
-
-static GPipe Pipes[MAX_PIPES] = {
-    { 0 }, { 0 }, { 0 }, { 0 }
 };
 
-#define die(s) { printf("%s\n", s); exit(1); } while(0);
+static GPipe Pipes[MAX_PIPES];
+
+#define die(s) do { printf("%s\n", s); exit(1); } while(0)
 
 unsigned int VideoCols = 80;
 unsigned int VideoRows = 25;
@@ -108,32 +109,68 @@ static unsigned char fromScreen[256];
 int GetKeyEvent(TEvent *Event);
 int GetMouseEvent(TEvent *Event);
 
-void mouseShow() {
+static ssize_t conread(int fd, TCell *p, size_t len, off_t off) {
+    char buf[len * 2];
+    char *s = buf;
+
+    lseek(fd, off, SEEK_SET);
+    ssize_t rlen = read(fd, buf, len * 2);
+    for (unsigned n = 0; n < rlen; ++p, n += 2) {
+        char ch;
+#ifdef USE_SCRNMAP
+	if (!noCharTrans)
+	    ch = fromScreen[(unsigned char)s[n + (BYTE_ORDER == BIG_ENDIAN)]];
+	else
+#endif
+	    ch = s[n + (BYTE_ORDER == BIG_ENDIAN)];
+
+	p->Set(ch, s[n + (BYTE_ORDER != BIG_ENDIAN)]);
+    }
+    return rlen;
+}
+
+static ssize_t conwrite(int fd, TCell *p, size_t len, off_t off) {
+    char buf[len * 2];
+    char *c = buf;
+
+    lseek(fd, off, SEEK_SET);
+    for (unsigned n = 0; n < len; ++n) {
+#ifdef USE_SCRNMAP
+	if (!noCharTrans)
+	    c[BYTE_ORDER == BIG_ENDIAN] = toScreen[(unsigned char)p[n].GetChar()];
+	else
+#endif
+	    c[BYTE_ORDER == BIG_ENDIAN] = p[n].GetChar();
+	c[BYTE_ORDER != BIG_ENDIAN] = p[n].GetAttr();
+	c += 2;
+    }
+    return write(fd, buf, c - buf);
+}
+
+static void mouseShow() {
 #ifdef USE_GPM
     if (GpmFd != -1 && VcsFd != -1 && drawPointer && mouseDrawn == 0) {
         int pos = (LastMouseX + LastMouseY * VideoCols) * 2 + 4;
-        lseek(VcsFd, pos, SEEK_SET);
-        read(VcsFd, &MousePosCell, 2);
-        TCell newCell = MousePosCell ^ 0x7700;  // correct ?
-        lseek(VcsFd, pos, SEEK_SET);
-        write(VcsFd, &newCell, 2);
+        conread(VcsFd, &MousePosCell, 1, pos);
+        TCell newCell(MousePosCell.GetChar(),
+                      MousePosCell.GetAttr() ^ 0x77);  // correct ?
+        conwrite(VcsFd, &newCell, 1, pos);
         mouseDrawn = 1;
     }
 #endif
 }
 
-void mouseHide() {
+static void mouseHide() {
 #ifdef USE_GPM
     if (GpmFd != -1 && VcsFd != -1 && drawPointer && mouseDrawn == 1) {
         int pos = (LastMouseX + LastMouseY * VideoCols) * 2 + 4;
-        lseek(VcsFd, pos, SEEK_SET);
-        write(VcsFd, &MousePosCell, 2);
+        conwrite(VcsFd, &MousePosCell, 1, pos);
         mouseDrawn = 0;
     }
 #endif
 }
 
-void SigWindowChanged(int arg) {
+void SigWindowChanged(int /*arg*/) {
     signal(SIGWINCH, SigWindowChanged);
     WindowChanged = 1;
 }
@@ -147,7 +184,7 @@ static void Die(int) {
     exit(66);
 }
 
-int ConInit(int XSize, int YSize) {
+int ConInit(int /*XSize*/, int /*YSize*/) {
     int tmp;
     int mode;
     struct termios newt;
@@ -225,10 +262,8 @@ int ConInit(int XSize, int YSize) {
         die("could not get screen character mapping!");
 
     {
-        int c;
-        for (c = 0; c < 256; c++)
-            fromScreen[c] = 0;
-        for (c = 0; c < 256; c++)
+        memset(fromScreen, 0, sizeof(fromScreen));
+        for (unsigned c = 0; c < 256; c++)
             fromScreen[toScreen[c]] = c;
     }
 #endif
@@ -319,7 +354,7 @@ int ConContinue() {
     struct termios newt;
     int tmp;
 
-    newt.c_lflag &= ~ (ICANON | ECHO | ISIG);
+    newt.c_lflag = ~ (ICANON | ECHO | ISIG);
     newt.c_iflag = 0;
     newt.c_cc[VMIN] = 16;
     newt.c_cc[VTIME] = 1;
@@ -343,114 +378,64 @@ int ConContinue() {
     return 0;
 }
 
-#ifdef USE_SCRNMAP
-int conread(int fd, void *p, int len) {   // len should be a multiple of 2
-    char buf[512];
-    char *c = (char *)p;
-    char *s = buf;
-
-    if (noCharTrans || (len > 512)) {
-        return read(fd, p, len);
-    } else {
-        int rlen = read(fd, buf, len);
-        for (int n = 0; n < rlen; n += 2) {
-            *c++ = fromScreen[(unsigned char)*s++];
-            *c++ = *s++;
-        }
-        return rlen;
-    }
-}
-
-int conwrite(int fd, void *p, int len) {  // len should be a multiple of 2
-    char buf[512];
-    char *s = (char *)p;
-    char *c = buf;
-
-    if (noCharTrans || (len > 512)) {
-        return write(fd, p, len);
-    } else {
-        for (int n = 0; n < len; n += 2) {
-            *c++ = toScreen[(unsigned char)*s++];
-            *c++ = *s++;
-        }
-        return write(fd, buf, len);
-    }
-}
-#else
-#define conread(fd,b,len) read(fd,b,len)
-#define conwrite(fd,b,len) write(fd,b,len)
-#endif
-
 int ConClear() {
     int X, Y;
-    TCell Cell = ' ' | (0x07 << 8);
+    TCell Cell(' ', 0x07);
     ConQuerySize(&X, &Y);
     ConSetBox(0, 0, X, Y, Cell);
     ConSetCursorPos(0, 0);
     return 0;
 }
 
-int ConSetTitle(char *Title, char *STitle) {
+int ConSetTitle(const char */*Title*/, const char */*STitle*/) {
     return 0;
 }
-int ConGetTitle(char *Title, int MaxLen, char *STitle, int SMaxLen) {
-    *Title = 0;
+int ConGetTitle(char *Title, size_t MaxLen, char *STitle, size_t SMaxLen) {
+    strlcpy(Title, "", MaxLen);
+    strlcpy(STitle, "", SMaxLen);
     return 0;
 }
 
 int ConPutBox(int X, int Y, int W, int H, PCell Cell) {
-    int i, hidden = 0;
-
-    for (i = 0; i < H; i++) {
-        if (LastMouseY == Y + i) { mouseHide(); hidden = 1; }
-        lseek(VcsFd, 4 + ((Y + i) * VideoCols + X) * 2, SEEK_SET);
-        conwrite(VcsFd, Cell, 2 * W);
-        Cell += W;
-        if (hidden) mouseShow();
+    for (int i = 0; i < H; Cell += W, ++i) {
+        if (LastMouseY == Y + i) mouseHide();
+        conwrite(VcsFd, Cell, W, 4 + ((Y + i) * VideoCols + X) * 2);
+        if (LastMouseY == Y + i) mouseShow();
     }
     return 0;
 }
 
 int ConGetBox(int X, int Y, int W, int H, PCell Cell) {
-    int i, hidden = 0;
-
-    for (i = 0; i < H; i++) {
-        if (LastMouseY == Y + i) { mouseHide(); hidden = 1; }
-        lseek(VcsFd, 4 + ((Y + i) * VideoCols + X) * 2, SEEK_SET);
-        conread(VcsFd, Cell, 2 * W);
-        Cell += W;
-        if (hidden) mouseShow();
+    for (int i = 0; i < H; Cell += W, ++i) {
+        if (LastMouseY == Y + i) mouseHide();
+        conread(VcsFd, Cell, W, 4 + ((Y + i) * VideoCols + X) * 2);
+        if (LastMouseY == Y + i) mouseShow();
     }
     return 0;
 }
 
 int ConPutLine(int X, int Y, int W, int H, PCell Cell) {
-    int i, hidden = 0;
 
-    for (i = 0; i < H; i++) {
-        if (LastMouseY == Y + i) { mouseHide(); hidden = 1; }
-        lseek(VcsFd, 4 + ((Y + i) * VideoCols + X) * 2, SEEK_SET);
-        conwrite(VcsFd, Cell, 2 * W);
-        if (hidden) mouseShow();
-    }
+    for (int i = 0; i < H; ++i)
+	ConPutBox(X, Y + i, W, 1, Cell);
+
     return 0;
 }
 
 int ConSetBox(int X, int Y, int W, int H, TCell Cell) {
     TDrawBuffer B;
-    MoveCh(B, Cell & 0xFF, Cell >> 8, W);
+    MoveCh(B, Cell.GetChar(), Cell.GetAttr(), W);
     ConPutLine(X, Y, W, H, B);
     return 0;
 }
 
 int ConScroll(int Way, int X, int Y, int W, int H, TAttr Fill, int Count) {
-    PCell C;
-    TDrawBuffer B;
 
-    if (Count == 0 || Count > H) return 0;
-    C = (PCell)malloc(2 * W * H);
-    if (C == 0)
-        return -1;
+    if (Count == 0 || Count > H)
+	return 0;
+
+    TDrawBuffer B;
+    TCell C[W * H];
 #ifdef USE_SCRNMAP
     noCharTrans = 1;
 #endif
@@ -466,11 +451,10 @@ int ConScroll(int Way, int X, int Y, int W, int H, TAttr Fill, int Count) {
 #ifdef USE_SCRNMAP
     noCharTrans = 0;
 #endif
-    free((void *)C);
     return 0;
 }
 
-int ConSetSize(int X, int Y) {
+int ConSetSize(int /*X*/, int /*Y*/) {
     return -1;
 }
 
@@ -485,8 +469,8 @@ int ConSetCursorPos(int X, int Y) {
 
     if (X >= 0 && X < int(VideoCols)) CursorX = X;
     if (Y >= 0 && Y < int(VideoRows)) CursorY = Y;
-    pos[0] = CursorX;
-    pos[1] = CursorY;
+    pos[0] = (char)CursorX;
+    pos[1] = (char)CursorY;
     lseek(VcsFd, 2, SEEK_SET);
     write(VcsFd, pos, 2);
     return 0;
@@ -498,19 +482,19 @@ int ConQueryCursorPos(int *X, int *Y) {
     return 0;
 }
 
-int ConShowCursor(void) {
+int ConShowCursor() {
     return 0;
 }
 
-int ConHideCursor(void) {
+int ConHideCursor() {
     return 0;
 }
 
-int ConCursorVisible(void) {
+int ConCursorVisible() {
     return 1;
 }
 
-int ConSetMousePos(int X, int Y) {
+int ConSetMousePos(int /*X*/, int /*Y*/) {
     return -1;
 }
 
@@ -520,15 +504,15 @@ int ConQueryMousePos(int *X, int *Y) {
     return -1;
 }
 
-int ConShowMouse(void) {
+int ConShowMouse() {
     return -1;
 }
 
-int ConHideMouse(void) {
+int ConHideMouse() {
     return -1;
 }
 
-int ConMouseVisible(void) {
+int ConMouseVisible() {
     return 0;
 }
 
@@ -537,10 +521,9 @@ int ConQueryMouseButtons(int *ButtonCount) {
     return 0;
 }
 
-int GetEvent(TEvent *Event);
 static TEvent Prev = { evNone };
 
-int ConGetEvent(TEventMask EventMask, TEvent *Event, int WaitTime, int Delete) {
+int ConGetEvent(TEventMask /*EventMask*/, TEvent *Event, int WaitTime, int Delete) {
     fd_set readfds;
     struct timeval timeout;
 
@@ -602,90 +585,92 @@ int ConGetEvent(TEventMask EventMask, TEvent *Event, int WaitTime, int Delete) {
     return 0;
 }
 
-int ConPutEvent(TEvent Event) {
+int ConPutEvent(const TEvent& Event) {
     Prev = Event;
     return 0;
 }
 
-int ConFlush(void) {return 0;  }
-int ConGrabEvents(TEventMask EventMask) { return 0; }
+int ConFlush() {return 0;  }
+int ConGrabEvents(TEventMask /*EventMask*/) { return 0; }
 
-int shift_state = 0;
-int lock_state = 0;
-int slock_state = 0;
-char dead_key = 0;
+static int shift_state = 0;
+static int lock_state = 0;
+static int slock_state = 0;
+static char dead_key = 0;
+
+// *INDENT-OFF*
+static const struct {
+    unsigned int KeySym;
+    unsigned int KeyCode;
+} KeyTrans[] = {
+    { K(KT_FN, K_F1),         kbF1 },
+    { K(KT_FN, K_F2),         kbF2 },
+    { K(KT_FN, K_F3),         kbF3 },
+    { K(KT_FN, K_F4),         kbF4 },
+    { K(KT_FN, K_F5),         kbF5 },
+    { K(KT_FN, K_F6),         kbF6 },
+    { K(KT_FN, K_F7),         kbF7 },
+    { K(KT_FN, K_F8),         kbF8 },
+    { K(KT_FN, K_F9),         kbF9 },
+    { K(KT_FN, K_F10),        kbF10 },
+    { K(KT_FN, K_F11),        kbF11 },
+    { K(KT_FN, K_F12),        kbF12 },
+    { K(KT_FN, K_INSERT),     kbIns | kfGray },
+    { K(KT_FN, K_REMOVE),     kbDel | kfGray },
+    { K(KT_FN, K_FIND),       kbHome | kfGray },
+    { K(KT_FN, K_SELECT),     kbEnd | kfGray },
+    { K(KT_FN, K_PGUP),       kbPgUp | kfGray },
+    { K(KT_FN, K_PGDN),       kbPgDn | kfGray },
+    { K(KT_CUR, K_UP),        kbUp | kfGray },
+    { K(KT_CUR, K_DOWN),      kbDown | kfGray },
+    { K(KT_CUR, K_LEFT),      kbLeft | kfGray },
+    { K(KT_CUR, K_RIGHT),     kbRight | kfGray },
+    { K(KT_SPEC, K_ENTER),    kbEnter },
+    { K(KT_PAD, K_PENTER),    kbEnter | kfGray },
+    { K(KT_PAD, K_PPLUS),     '+' | kfGray },
+    { K(KT_PAD, K_PMINUS),    '-' | kfGray },
+    { K(KT_PAD, K_PSTAR),     '*' | kfGray },
+    { K(KT_PAD, K_PSLASH),    '/' | kfGray },
+    { K(KT_PAD, K_P0),        kbIns },
+    { K(KT_PAD, K_PDOT),      kbDel },
+    { K(KT_PAD, K_P1),        kbEnd },
+    { K(KT_PAD, K_P2),        kbDown },
+    { K(KT_PAD, K_P3),        kbPgDn },
+    { K(KT_PAD, K_P4),        kbLeft },
+    { K(KT_PAD, K_P5),        kbCenter },
+    { K(KT_PAD, K_P6),        kbRight },
+    { K(KT_PAD, K_P7),        kbHome },
+    { K(KT_PAD, K_P8),        kbUp },
+    { K(KT_PAD, K_P9),        kbPgUp },
+    { K(KT_FN, K_PAUSE),      kbPause },
+    { K(KT_LATIN, 27),        kbEsc },
+    { K(KT_LATIN, 13),        kbEnter },
+    { K(KT_LATIN, 8),         kbBackSp },
+    { K(KT_LATIN, 127),       kbDel },
+    { K(KT_LATIN, 9),         kbTab },
+    { K(KT_SHIFT, KG_SHIFT),  kbShift | kfSpecial | kfModifier },
+    { K(KT_SHIFT, KG_SHIFTL), kbShift | kfSpecial | kfModifier },
+    { K(KT_SHIFT, KG_SHIFTR), kbShift | kfSpecial | kfModifier | kfGray },
+    { K(KT_SHIFT, KG_CTRL),   kbCtrl  | kfSpecial | kfModifier },
+    { K(KT_SHIFT, KG_CTRLL),  kbCtrl  | kfSpecial | kfModifier },
+    { K(KT_SHIFT, KG_CTRLR),  kbCtrl  | kfSpecial | kfModifier | kfGray },
+    { K(KT_SHIFT, KG_ALT),    kbAlt   | kfSpecial | kfModifier },
+    { K(KT_SHIFT, KG_ALTGR),  kbAlt   | kfSpecial | kfModifier | kfGray },
+    { 0, 0 }
+};
 
 static const struct {
-    unsigned long KeySym;
-    unsigned long KeyCode;
-} KeyTrans[] = {
-{ K(KT_FN, K_F1),               kbF1 },
-{ K(KT_FN, K_F2),               kbF2 },
-{ K(KT_FN, K_F3),               kbF3 },
-{ K(KT_FN, K_F4),               kbF4 },
-{ K(KT_FN, K_F5),               kbF5 },
-{ K(KT_FN, K_F6),               kbF6 },
-{ K(KT_FN, K_F7),               kbF7 },
-{ K(KT_FN, K_F8),               kbF8 },
-{ K(KT_FN, K_F9),               kbF9 },
-{ K(KT_FN, K_F10),              kbF10 },
-{ K(KT_FN, K_F11),              kbF11 },
-{ K(KT_FN, K_F12),              kbF12 },
-{ K(KT_FN, K_INSERT),           kbIns | kfGray },
-{ K(KT_FN, K_REMOVE),           kbDel | kfGray },
-{ K(KT_FN, K_FIND),             kbHome | kfGray },
-{ K(KT_FN, K_SELECT),           kbEnd | kfGray },
-{ K(KT_FN, K_PGUP),             kbPgUp | kfGray },
-{ K(KT_FN, K_PGDN),             kbPgDn | kfGray },
-{ K(KT_CUR, K_UP),              kbUp | kfGray },
-{ K(KT_CUR, K_DOWN),            kbDown | kfGray },
-{ K(KT_CUR, K_LEFT),            kbLeft | kfGray },
-{ K(KT_CUR, K_RIGHT),           kbRight | kfGray },
-{ K(KT_SPEC, K_ENTER),          kbEnter },
-{ K(KT_PAD, K_PENTER),          kbEnter | kfGray },
-{ K(KT_PAD, K_PPLUS),           '+' | kfGray },
-{ K(KT_PAD, K_PMINUS),          '-' | kfGray },
-{ K(KT_PAD, K_PSTAR),           '*' | kfGray },
-{ K(KT_PAD, K_PSLASH),          '/' | kfGray },
-{ K(KT_PAD, K_P0),              kbIns },
-{ K(KT_PAD, K_PDOT),            kbDel },
-{ K(KT_PAD, K_P1),              kbEnd },
-{ K(KT_PAD, K_P2),              kbDown },
-{ K(KT_PAD, K_P3),              kbPgDn },
-{ K(KT_PAD, K_P4),              kbLeft },
-{ K(KT_PAD, K_P5),              kbCenter },
-{ K(KT_PAD, K_P6),              kbRight },
-{ K(KT_PAD, K_P7),              kbHome },
-{ K(KT_PAD, K_P8),              kbUp },
-{ K(KT_PAD, K_P9),              kbPgUp },
-{ K(KT_FN, K_PAUSE),            kbPause },
-{ K(KT_LATIN, 27),              kbEsc },
-{ K(KT_LATIN, 13),              kbEnter },
-{ K(KT_LATIN, 8),               kbBackSp },
-{ K(KT_LATIN, 127),             kbDel },
-{ K(KT_LATIN, 9),               kbTab },
-{ K(KT_SHIFT, KG_SHIFT),        kbShift | kfSpecial | kfModifier },
-{ K(KT_SHIFT, KG_SHIFTL),       kbShift | kfSpecial | kfModifier },
-{ K(KT_SHIFT, KG_SHIFTR),       kbShift | kfSpecial | kfModifier | kfGray },
-{ K(KT_SHIFT, KG_CTRL),         kbCtrl | kfSpecial | kfModifier },
-{ K(KT_SHIFT, KG_CTRLL),        kbCtrl | kfSpecial | kfModifier },
-{ K(KT_SHIFT, KG_CTRLR),        kbCtrl | kfSpecial | kfModifier | kfGray },
-{ K(KT_SHIFT, KG_ALT),          kbAlt | kfSpecial | kfModifier },
-{ K(KT_SHIFT, KG_ALTGR),        kbAlt | kfSpecial | kfModifier | kfGray },
-{ 0, 0 }
-};
-
-struct {
-    unsigned long KeySym;
+    unsigned int KeySym;
     char Diacr;
 } DeadTrans[] = {
-{ K_DGRAVE, '`' },
-{ K_DACUTE, '\'' },
-{ K_DCIRCM, '^' },
-{ K_DTILDE, '~' },
-{ K_DDIERE, '"' },
-{ K_DCEDIL, ',' }
+    { K_DGRAVE, '`' },
+    { K_DACUTE, '\'' },
+    { K_DCIRCM, '^' },
+    { K_DTILDE, '~' },
+    { K_DDIERE, '"' },
+    { K_DCEDIL, ',' }
 };
+// *INDENT-ON*
 
 int GetKeyEvent(TEvent *Event) {
     char keycode;
@@ -789,12 +774,13 @@ int GetKeyEvent(TEvent *Event) {
     if (key == 14 && keysym == K(KT_LATIN,127)) {
         // we are running on a system with broken backspace key
         KeyCode = kbBackSp;
-    } else for(unsigned int i = 0; i < sizeof(KeyTrans) / sizeof(KeyTrans[0]); i++) {
-        if (KeyTrans[i].KeySym == keysym) {
-            KeyCode = KeyTrans[i].KeyCode;
-            break;
-        }
-    }
+    } else
+        for(size_t i = 0; i < FTE_ARRAY_SIZE(KeyTrans); ++i)
+            if (KeyTrans[i].KeySym == keysym) {
+                KeyCode = KeyTrans[i].KeyCode;
+                break;
+            }
+
     if (KeyCode == 0) {
         switch (KTYP(keysym)) {
         case KT_CONS:
@@ -846,23 +832,22 @@ int GetKeyEvent(TEvent *Event) {
                 KeyCode = toupper(KeyCode);
 
             if (KTYP(keysym) == KT_DEAD) {
-                for (int i = 0;i < sizeof(DeadTrans) / sizeof(DeadTrans[0]); i++) {
+                for (size_t i = 0; i < FTE_ARRAY_SIZE(DeadTrans); ++i)
                     if (DeadTrans[i].KeySym == keysym) {
                         dead_key = DeadTrans[i].Diacr;
                         return -1;
                     }
-                }
+
                 dead_key = 0;
                 return -1;
             }
             if (! (ShiftFlags & (kfAlt | kfCtrl)) && dead_key) {
-                for (int i = 0; i < diacr_table.kb_cnt; i++) {
+                for (unsigned i = 0; i < diacr_table.kb_cnt; ++i)
                     if (diacr_table.kbdiacr[i].base == KeyCode &&
                         diacr_table.kbdiacr[i].diacr == dead_key) {
                         KeyCode=diacr_table.kbdiacr[i].result;
                         break;
                     }
-                }
             }
             dead_key = 0;
 
@@ -900,33 +885,32 @@ int GetKeyEvent(TEvent *Event) {
         // there is also a problem with shift-states (should update it
         // after getting back, but how to know when that happens, without
         // using VT_PROCESS?).
-        {
-            int vc = -1;
+	int vc = -1;
 
-//            switch (kbCode(Event->Key.Code) | kfCtrl) {
-            switch (kbCode(Event->Key.Code)) {
-            case kfAlt | kfCtrl | kbF1: vc = 1; break;
-            case kfAlt | kfCtrl | kbF2: vc = 2; break;
-            case kfAlt | kfCtrl | kbF3: vc = 3; break;
-            case kfAlt | kfCtrl | kbF4: vc = 4; break;
-            case kfAlt | kfCtrl | kbF5: vc = 5; break;
-            case kfAlt | kfCtrl | kbF6: vc = 6; break;
-            case kfAlt | kfCtrl | kbF7: vc = 7; break;
-            case kfAlt | kfCtrl | kbF8: vc = 8; break;
-            case kfAlt | kfCtrl | kbF9: vc = 9; break;
-            case kfAlt | kfCtrl | kbF10: vc = 10; break;
-            case kfAlt | kfCtrl | kbF11: vc = 11; break;
-            case kfAlt | kfCtrl | kbF12: vc = 12; break;
-            }
-            if (vc != -1) {
-                /* perform the console switch */
-                ioctl(VtFd, VT_ACTIVATE, vc);
-                Event->What = evNone;
-//                shift_state = lock_state = slock_state = 0; // bad
-                shift_state = slock_state = 0; // bad
-                return -1;
-            }
-        }
+	//            switch (kbCode(Event->Key.Code) | kfCtrl) {
+	switch (kbCode(Event->Key.Code)) {
+	case kfAlt | kfCtrl | kbF1: vc = 1; break;
+	case kfAlt | kfCtrl | kbF2: vc = 2; break;
+	case kfAlt | kfCtrl | kbF3: vc = 3; break;
+	case kfAlt | kfCtrl | kbF4: vc = 4; break;
+	case kfAlt | kfCtrl | kbF5: vc = 5; break;
+	case kfAlt | kfCtrl | kbF6: vc = 6; break;
+	case kfAlt | kfCtrl | kbF7: vc = 7; break;
+	case kfAlt | kfCtrl | kbF8: vc = 8; break;
+	case kfAlt | kfCtrl | kbF9: vc = 9; break;
+	case kfAlt | kfCtrl | kbF10: vc = 10; break;
+	case kfAlt | kfCtrl | kbF11: vc = 11; break;
+	case kfAlt | kfCtrl | kbF12: vc = 12; break;
+	}
+
+	if (vc != -1) {
+	    /* perform the console switch */
+	    ioctl(VtFd, VT_ACTIVATE, vc);
+	    Event->What = evNone;
+	    //                shift_state = lock_state = slock_state = 0; // bad
+	    shift_state = slock_state = 0; // bad
+	    return -1;
+	}
 
         return 0;
     }
@@ -939,11 +923,10 @@ int GetMouseEvent(TEvent *Event) {
 
     Event->What = evNone;
     int rc = Gpm_GetEvent(&e);
-    if (rc == 0) { 
+    if (rc == 0) {
         Gpm_Close();
         GpmFd = -1;
-    } else if (rc == -1) {
-    } else {
+    } else if (rc != -1) {
         //Gpm_FitEvent(&e);
         if ((e.type & GPM_MOVE) || (e.type & GPM_DRAG))
             Event->What = evMouseMove;
@@ -993,7 +976,7 @@ int GetMouseEvent(TEvent *Event) {
     return 0;
 }
 
-int ConSetCursorSize(int Start, int End) {
+int ConSetCursorSize(int /*Start*/, int /*End*/) {
     return 0;
 }
 
@@ -1001,9 +984,7 @@ static PCell SavedScreen = 0;
 static int SavedX, SavedY, SaveCursorPosX, SaveCursorPosY;
 
 int SaveScreen() {
-    if (SavedScreen)
-        free(SavedScreen);
-
+    free(SavedScreen);
     ConQuerySize(&SavedX, &SavedY);
 
     SavedScreen = (PCell) malloc(SavedX * SavedY * sizeof(PCell));
@@ -1022,9 +1003,10 @@ int RestoreScreen() {
     return 1;
 }
 
-GUI::GUI(int &argc, char **argv, int XSize, int YSize) {
-    fArgc = argc;
-    fArgv = argv;
+GUI::GUI(int &argc, char **argv, int XSize, int YSize) :
+    fArgc(argc),
+    fArgv(argv)
+{
     ::ConInit(-1, -1);
     SaveScreen();
     ::ConSetSize(XSize, YSize);
@@ -1034,15 +1016,18 @@ GUI::GUI(int &argc, char **argv, int XSize, int YSize) {
 GUI::~GUI() {
     RestoreScreen();
     ::ConDone();
+
+    free(SavedScreen);
+    SavedScreen = 0;
     gui = 0;
 }
 
-int GUI::ConSuspend(void) {
+int GUI::ConSuspend() {
     RestoreScreen();
     return ::ConSuspend();
 }
 
-int GUI::ConContinue(void) {
+int GUI::ConContinue() {
     SaveScreen();
     return ::ConContinue();
 }
@@ -1052,14 +1037,16 @@ int GUI::ShowEntryScreen() {
 
     ConHideMouse();
     RestoreScreen();
-    do { gui->ConGetEvent(evKeyDown, &E, -1, 1, 0); } while (E.What != evKeyDown);
+    do {
+	gui->ConGetEvent(evKeyDown, &E, -1, 1, 0);
+    } while (E.What != evKeyDown);
     ConShowMouse();
     if (frames)
         frames->Repaint();
     return 1;
 }
 
-int GUI::OpenPipe(char *Command, EModel *notify) {
+int GUI::OpenPipe(const char *Command, EModel *notify) {
     int i;
 
     for (i = 0; i < MAX_PIPES; i++) {
@@ -1109,8 +1096,8 @@ int GUI::SetPipeView(int id, EModel *notify) {
     return 0;
 }
 
-int GUI::ReadPipe(int id, void *buffer, int len) {
-    int rc;
+ssize_t GUI::ReadPipe(int id, void *buffer, size_t len) {
+    ssize_t rc;
 
     if (id < 0 || id > MAX_PIPES)
         return -1;
@@ -1150,7 +1137,7 @@ int GUI::ClosePipe(int id) {
     return WEXITSTATUS(status);
 }
 
-int GUI::RunProgram(int mode, char *Command) {
+int GUI::RunProgram(int /*mode*/, char *Command) {
     int rc, W, H, W1, H1;
 
     ConQuerySize(&W, &H);
@@ -1173,16 +1160,26 @@ int GUI::RunProgram(int mode, char *Command) {
     return rc;
 }
 
-char ConGetDrawChar(int index) {
-    static const unsigned char tab[] = "ڿĳô\x1A\x1B\x1A";
-    //static char tab[] = "++++-|+++++>.*-^v :[>";
-    //static char tab[] = "\x0D\x0C\x0E\x0B\x12\x19____+>\x1F\x01\x12 ";
+char ConGetDrawChar(unsigned int idx) {
+    static const char *tab = NULL;
+    static size_t tablen = 0;
 
-    assert(index >= 0 && index < int(strlen(tab)));
+    if (!tab) {
+        if (getenv("ISOCONSOLE")) {
+            tab = GetGUICharacters("Linux", "++++-|+++++>.*-^v :[>");
+        } else {
+            /* it's hard to pick usable chars between way to many fonts */
+            tab = GetGUICharacters("Linux", "\xDA\xBF\xC0\xD9\xC4\xB3\xC2\xC3\xB4\xC1\xC5\x1A.\x0A\xC4\x18\x19\xB1\xB0\x1B\x1A");
+            //tab = GetGUICharacters("Linux", "\xDA\xBF\xC0\xD9\xC4\xB3\xC2\xC3\xB4\xC1\xC5\x1A\xFA\x04\xC4\x18\x19\xB1\xB0\x1B\x1A");
+            //tab = GetGUICharacters("Linux", "\x0D\x0C\x0E\x0B\x12\x19____+>\x1F\x01\x12\x01\x01 \x02\x01\x01");
+        }
+        tablen = strlen(tab);
+    }
+    assert(idx < tablen);
 
 #ifdef USE_SCRNMAP
-    return fromScreen[tab[index]];
+    return fromScreen[(unsigned char)tab[idx]];
 #else
-    return tab[index];
+    return tab[idx];
 #endif
 }
