@@ -44,6 +44,7 @@
 #include "console.h"
 #include "gui.h"
 #include "s_files.h"
+#include "s_string.h"
 #include "c_config.h"
 
 #define True    1
@@ -228,8 +229,8 @@ struct {
     { 0, 0 }
 };
 
-char shftwrng[]  = "~!@#$%^&*()_+{}|:\"<>?";
-char shftright[] = "`1234567890-=[]\\;',./";
+static const char shftwrng[]  = "~!@#$%^&*()_+{}|:\"<>?";
+static const char shftright[] = "`1234567890-=[]\\;',./";
 
 int ReadConsoleEvent(TEvent *E) /*FOLD00*/
 {
@@ -384,6 +385,37 @@ int ReadConsoleEvent(TEvent *E) /*FOLD00*/
         E->Mouse.Count = 1;
         if (inp.Event.MouseEvent.dwEventFlags & DOUBLE_CLICK)
             E->Mouse.Count = 2;
+
+        if ((inp.Event.MouseEvent.dwEventFlags & MOUSE_WHEELED) == MOUSE_WHEELED)
+        {
+            E->What        = evCommand;
+            E->Msg.View    = 0;
+            E->Msg.Model   = 0;
+            E->Msg.Param2  = 0;
+
+            // Scroll:
+            // (The SDK does not tell how to determine whether the wheel
+            // was scrolled up or down. Found an example on:
+            // http://www.adrianxw.dk/SoftwareSite/Consoles/Consoles5.html
+            if (inp.Event.MouseEvent.dwButtonState & 0xFF000000) {  // Wheel down
+                if (flg & kfShift) { // Translate to horizontal scroll.
+                    E->Msg.Command = (flg & kfCtrl) ? cmHScrollPgRt : cmHScrollRight;
+                } else { // Translate to vertical scroll.
+                    E->Msg.Command = (flg & kfCtrl) ? cmVScrollPgDn : cmVScrollDown;
+                }
+            } else { // Wheel up
+                if (flg & kfShift) { // Translate to horizontal scroll.
+                    E->Msg.Command = (flg & kfCtrl) ? cmHScrollPgLt : cmHScrollLeft;
+                } else { // Translate to vertical scroll.
+                    E->Msg.Command = (flg & kfCtrl) ? cmVScrollPgUp : cmVScrollUp;
+                }
+            }
+
+            E->Msg.Param1 = (flg & kfCtrl) ? 1 : 3; // 1 page / 3 lines
+
+            return True;
+        }
+
         if (inp.Event.MouseEvent.dwEventFlags == MOUSE_MOVED)
         {
             E->What = evMouseMove;
@@ -442,20 +474,20 @@ int ConInit(int /*XSize*/, int /*YSize*/) { /*FOLD00*/
     return 0;
 }
 
-int ConDone(void) { /*FOLD00*/
+int ConDone() { /*FOLD00*/
     ConSuspend();
     CloseHandle(OurConOut);
     return 0;
 }
 
-int ConSuspend(void) { /*FOLD00*/
+int ConSuspend() { /*FOLD00*/
     SetConsoleActiveScreenBuffer(ConOut);
     SetConsoleMode(ConIn, OldConsoleMode);
 
     return 0;
 }
 
-int ConContinue(void) { /*FOLD00*/
+int ConContinue() { /*FOLD00*/
     SetConsoleActiveScreenBuffer(OurConOut);
     GetConsoleMode(ConIn, &OldConsoleMode);
     SetConsoleMode(ConIn, ENABLE_WINDOW_INPUT | ENABLE_MOUSE_INPUT);
@@ -466,7 +498,7 @@ int ConContinue(void) { /*FOLD00*/
     return 0;
 }
 
-int ConClear(void) { /*FOLD00*/
+int ConClear() { /*FOLD00*/
     int W, H;
     TDrawBuffer B;
 
@@ -684,13 +716,13 @@ int ConQueryCursorPos(int *X, int *Y) { /*FOLD00*/
     return 0;
 }
 
-int ConShowCursor(void) { /*FOLD00*/
+int ConShowCursor() { /*FOLD00*/
     CursorVisible = 1;
     DrawCursor(1);
     return 0;
 }
 
-int ConHideCursor(void) { /*FOLD00*/
+int ConHideCursor() { /*FOLD00*/
     CursorVisible = 0;
     DrawCursor(0);
     return 0;
@@ -716,13 +748,13 @@ int ConQueryMousePos(int *X, int *Y) { /*FOLD00*/
     return 0;
 }
 
-int ConShowMouse(void) { /*FOLD00*/
+int ConShowMouse() { /*FOLD00*/
     MouseVisible = 1;
     if (!MousePresent) return -1;
     return 0;
 }
 
-int ConHideMouse(void) { /*FOLD00*/
+int ConHideMouse() { /*FOLD00*/
     MouseVisible = 0;
     if (!MousePresent) return -1;
     return 0;
@@ -736,12 +768,12 @@ int ConQueryMouseButtons(int *ButtonCount) { /*FOLD00*/
     return 0;
 }
 
-int ConPutEvent(TEvent Event) { /*FOLD00*/
+int ConPutEvent(const TEvent& Event) { /*FOLD00*/
     EventBuf = Event;
     return 0;
 }
 
-int ConFlush(void) { /*FOLD00*/
+int ConFlush() { /*FOLD00*/
     return 0;
 }
 
@@ -795,12 +827,12 @@ GUI::~GUI() { /*FOLD00*/
     gui = 0;
 }
 
-int GUI::ConSuspend(void) { /*FOLD00*/
+int GUI::ConSuspend() { /*FOLD00*/
     RestoreScreen();
     return ::ConSuspend();
 }
 
-int GUI::ConContinue(void) { /*FOLD00*/
+int GUI::ConContinue() { /*FOLD00*/
     SaveScreen();
     return ::ConContinue();
 }
@@ -819,13 +851,13 @@ int GUI::ShowEntryScreen() { /*FOLD00*/
     return 1;
 }
 
-char ConGetDrawChar(int index) { /*FOLD00*/
+char ConGetDrawChar(unsigned int index) { /*FOLD00*/
     static const char *tab=NULL;
 
     if (!tab) {
         tab=GetGUICharacters ("WindowsNT","ڿĳô\x1A\x1B\x1A");
     }
-    assert(index >= 0 && index < (int)strlen(tab));
+    assert(index < strlen(tab));
 
     return tab[index];
 }
@@ -864,32 +896,28 @@ int GUI::RunProgram(int mode, char *Command) { /*FOLD00*/
     return rc;
 }
 
-int ConSetTitle(char *Title, char *STitle) { /*FOLD00*/
+int ConSetTitle(const char *Title, const char *STitle) { /*FOLD00*/
     char buf[sizeof(winTitle)] = {0};
-    JustFileName(Title, buf);
+    JustFileName(Title, buf, sizeof(buf));
     if (buf[0] == '\0') // if there is no filename, try the directory name.
-        JustLastDirectory(Title, buf);
+        JustLastDirectory(Title, buf, sizeof(buf));
 
-    strncpy(winTitle, "FTE - ", sizeof(winTitle) - 1);
+    strlcpy(winTitle, "FTE - ", sizeof(winTitle));
     if (buf[0] != 0) // if there is a file/dir name, stick it in here.
     {
-        strncat(winTitle, buf, sizeof(winTitle) - 1 - strlen(winTitle));
-        strncat(winTitle, " - ", sizeof(winTitle) - 1 - strlen(winTitle));
+        strlcat(winTitle, buf, sizeof(winTitle));
+        strlcat(winTitle, " - ", sizeof(winTitle));
     }
-    strncat(winTitle, Title, sizeof(winTitle) - 1 - strlen(winTitle));
-    winTitle[sizeof(winTitle) - 1] = 0;
-    strncpy(winSTitle, STitle, sizeof(winSTitle) - 1);
-    winSTitle[sizeof(winSTitle) - 1] = 0;
+    strlcat(winTitle, Title, sizeof(winTitle));
+    strlcpy(winSTitle, STitle, sizeof(winSTitle));
     SetConsoleTitle (winTitle);
 
     return 0;
 }
 
-int ConGetTitle(char *Title, int MaxLen, char *STitle, int SMaxLen) { /*FOLD00*/
-    strncpy(Title, winTitle, MaxLen);
-    Title[MaxLen - 1] = 0;
-    strncpy(STitle, winSTitle, SMaxLen);
-    STitle[SMaxLen - 1] = 0;
+int ConGetTitle(char *Title, size_t MaxLen, char *STitle, size_t SMaxLen) { /*FOLD00*/
+    strlcpy(Title, winTitle, MaxLen);
+    strlcpy(STitle, winSTitle, SMaxLen);
     return 0;
 }
 
@@ -1152,20 +1180,21 @@ int GPipe::runCommand() /*FOLD00*/
         //** It's 4DOS all right..
         args = getenv("SystemRoot");
         if(args== 0) return -1;
-        strcpy(tbuf, args);                 // Get to c:\winnt
-        strcat(tbuf, "\\system32\\cmd.exe");
+        strlcpy(tbuf, args, sizeof(tbuf));                 // Get to c:\winnt
+        strlcat(tbuf, "\\system32\\cmd.exe", sizeof(tbuf));
         comspec = tbuf;
     }
 
-    args    = (char *)malloc(strlen(comspec) + strlen(p_command) + 120);
+    int argslen = strlen(comspec) + strlen(p_command) + 120;
+    args    = (char *)malloc(argslen);
     if(args == 0)
         dbm("malloc() failed for command line..");
     else
     {
         //** Form a command line for the process;
-        strcpy(args, comspec);
-        strcat(args, " /c ");
-        strcat(args, p_command);
+        strlcpy(args, comspec, argslen);
+        strlcat(args, " /c ", argslen);
+        strlcat(args, p_command, argslen);
 
         //** Dup the child handle to get separate handles for stdout and err,
         if (DuplicateHandle(GetCurrentProcess(), p_child_ph, // Source,
@@ -1466,7 +1495,7 @@ int GPipe::getEvent(TEvent* event) /*FOLD00*/
  *
  *
  */
-int GUI::OpenPipe(char *Command, EModel *notify) /*FOLD00*/
+int GUI::OpenPipe(const char *Command, EModel *notify) /*FOLD00*/
 {
     GPipe*  gp;
 
@@ -1488,7 +1517,7 @@ int GUI::SetPipeView(int id, EModel *notify) /*FOLD00*/
 }
 
 
-int GUI::ReadPipe(int id, void *buffer, int len) /*FOLD00*/
+ssize_t GUI::ReadPipe(int id, void *buffer, size_t len) /*FOLD00*/
 {
     //int     l;
     GPipe*  p;
@@ -1582,20 +1611,21 @@ static int CreatePipeChild(HANDLE &child, HANDLE &hPipe, char *Command) {
         //** It's 4DOS all right..
         args = getenv("SystemRoot");
         if(args== 0) return -1;
-        strcpy(tbuf, args);                 // Get to c:\winnt
-        strcat(tbuf, "\\system32\\cmd.exe");
+        strlcpy(tbuf, args, sizeof(tbuf));                 // Get to c:\winnt
+        strlcat(tbuf, "\\system32\\cmd.exe", sizeof(tbuf));
         comspec = tbuf;
     }
 
-    args    = (char *)malloc(strlen(comspec) + strlen(Command) + 120);
+    int argslen = strlen(comspec) + strlen(Command) + 120;
+    args    = (char *)malloc(argslen);
     if(args == 0)
         dbm("malloc() failed for command line..");
     else
     {
         //** Form a command line for the process;
-        strcpy(args, comspec);
-        strcat(args, " /c ");
-        strcat(args, Command);
+        strlcpy(args, comspec, argslen);
+        strlcat(args, " /c ", argslen);
+        strlcat(args, Command, argslen);
 
         //** Dup the child handle to get separate handles for stdout and err,
         /*if (DuplicateHandle(GetCurrentProcess(), hChildPipe,
@@ -1708,7 +1738,7 @@ static DWORD __stdcall PipeThread(void *p) {
     return 0;
 }
 
-int GUI::OpenPipe(char *Command, EModel *notify) {
+int GUI::OpenPipe(const char *Command, EModel *notify) {
     int i;
 
     for (i = 0; i < MAX_PIPES; i++) {
@@ -1779,8 +1809,8 @@ int GUI::SetPipeView(int id, EModel *notify) {
     return 0;
 }
 
-int GUI::ReadPipe(int id, void *buffer, int len) {
-    int l;
+ssize_t GUI::ReadPipe(int id, void *buffer, size_t len) {
+    ssize_t l;
     //ULONG ulPostCount;
 
     if (id < 0 || id > MAX_PIPES)
