@@ -9,15 +9,19 @@
 
 // include
 
+#include "console.h"
+#include "gui.h"
+#include "sysdep.h"
+
+#include <process.h>
+#include <signal.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <assert.h>
-#include <signal.h>
 
-#include "sysdep.h"
-#include "console.h"
-#include "gui.h"
+#ifndef OS2_INCLUDED
+#include <os2.h>
+#endif
 
 extern int ShowVScroll;
 
@@ -32,12 +36,6 @@ extern int ShowVScroll;
 #define INCL_DOS
 #define INCL_DOSDEVIOCTL
 
-#include <stdlib.h>
-#include <process.h>
-
-#ifndef OS2_INCLUDED
-#include <os2.h>
-#endif
 
 #define MAX_PIPES 4
 #define PIPE_BUFLEN 4096
@@ -106,6 +104,7 @@ static void DrawMouse(int Show) {
     }
 }
 
+// *INDENT-OFF*
 static struct { // TransCharScan
     USHORT CharScan;
     TKeyCode KeyCode;
@@ -181,6 +180,7 @@ static struct { // TransScan
     { 0x9F, kbEnd   | kfGray },  { 0xA0, kbDown  | kfGray },  { 0xA1, kbPgDn  | kfGray },
     { 0xA2, kbIns   | kfGray },  { 0xA3, kbDel   | kfGray }
 };
+// *INDENT-ON*
 
 int ReadKbdEvent(TEvent *Event, int Wait) {
     KBDKEYINFO ki;
@@ -802,12 +802,12 @@ GUI::~GUI() {
     gui = 0;
 }
 
-int GUI::ConSuspend(void) {
+int GUI::ConSuspend() {
     RestoreScreen();
     return ::ConSuspend();
 }
 
-int GUI::ConContinue(void) {
+int GUI::ConContinue() {
     SaveScreen();
     return ::ConContinue();
 }
@@ -991,7 +991,7 @@ static void _LNK_CONV PipeThread(void *p) {
     return;
 }
 
-int GUI::OpenPipe(char *Command, EModel *notify) {
+int GUI::OpenPipe(const char *Command, EModel *notify) {
     int i;
 
     for (i = 0; i < MAX_PIPES; i++) {
@@ -1063,8 +1063,8 @@ int GUI::SetPipeView(int id, EModel *notify) {
     return 0;
 }
 
-int GUI::ReadPipe(int id, void *buffer, int len) {
-    int l;
+ssize_t GUI::ReadPipe(int id, void *buffer, size_t len) {
+    ssize_t l;
     //ULONG ulPostCount;
 
     if (id < 0 || id > MAX_PIPES)
@@ -1150,7 +1150,7 @@ int GUI::RunProgram(int mode, char *Command) {
     return rc;
 }
 
-int ConSetTitle(char *Title, char *STitle) {
+int ConSetTitle(const char *Title, const char *STitle) {
 /*    HSWITCH hsw;
     SWCNTRL sw;
     HAB hab;
@@ -1196,9 +1196,9 @@ int ConSetTitle(char *Title, char *STitle) {
     return 0;
 }
 
-int ConGetTitle(char *Title, int MaxLen, char *STitle, int SMaxLen) {
-    strcpy(Title, "FTE");
-    strcpy(STitle, "FTE");
+int ConGetTitle(char *Title, size_t MaxLen, char *STitle, size_t SMaxLen) {
+    strlcpy(Title, "FTE",  MaxLen);
+    strlcpy(STitle, "FTE", SMaxLen);
     return 0;
 }
 
@@ -1206,21 +1206,18 @@ int ConCursorVisible() {
     return (CursorVisible == 1);
 }
 
-int ConPutEvent(TEvent Event) {
+int ConPutEvent(const TEvent& Event) {
     EventBuf = Event;
     return 0;
 }
 
 extern int SevenBit;
 
-char ConGetDrawChar(int index) {
-    static char tab[] =  "ڿĳô\x1A\x1B\x1A";
-    static char tab7[] = "++++-|+++++\x1A.-++#+\x1B\x1A";
+char ConGetDrawChar(unsigned int index) {
+    static const char tab[] =  "ڿĳô\x1A\x1B\x1A";
+    static const char tab7[] = "++++-|+++++\x1A.-++#+\x1B\x1A";
 
-    assert(index >= 0 && index < (int)strlen(tab));
+    assert(index < strlen(tab));
 
-    if (SevenBit)
-        return tab7[index];
-    else
-        return tab[index];
+    return (SevenBit) ? tab7[index] : tab[index];
 }
