@@ -7,33 +7,56 @@
  *
  */
 
-#include "fte.h"
+#include "e_buffer.h"
+
+#include "c_bind.h"
+#include "c_config.h"
 #include "c_history.h"
+#include "e_mark.h"
+#include "e_undo.h"
+#include "i_modelview.h"
+#include "i_view.h"
+#include "o_routine.h"
+#include "s_util.h"
 
 EBuffer *SSBuffer = 0; // scrap buffer (clipboard)
 
 ///////////////////////////////////////////////////////////////////////////////
 
-EBuffer::EBuffer(int createFlags, EModel **ARoot, const char * /*AName*/)
-    :EModel(createFlags, ARoot), TP(0,0), CP(0,0), BB(-1,-1), BE(-1,-1),
-     PrevPos(-1, -1), SavedPos(-1, -1), Match(-1, -1)
+EBuffer::EBuffer(int createFlags, EModel **ARoot, const char * /*AName*/) :
+    EModel(createFlags, ARoot),
+    FileName(0),
+    Modified(0),
+    TP(0,0),
+    CP(0,0),
+    BB(-1,-1),
+    BE(-1,-1),
+    PrevPos(-1, -1),
+    SavedPos(-1, -1),
+    BlockMode(bmStream),
+    ExtendGrab(0),
+    AutoExtend(0),
+    Loaded(0),
+    Loading(0),
+    RAllocated(0),
+    RGap(0),
+    RCount(0),
+    LL(0),
+    VAllocated(0),
+    VGap(0),
+    VCount(0),
+    VV(0),
+#ifdef CONFIG_FOLDS
+    FCount(0),
+    FF(0),
+#endif
+    Match(-1, -1),
+    MatchLen(0),
+    MatchCount(0),
+    MinRedraw(-1),
+    MaxRedraw(-1),
+    RedrawToEos(0)
 {
-    Modified = 0;
-    Loaded = 0;
-    Loading = 0;
-
-    FileName = 0;
-    LL = 0;
-    VV = 0;
-    FF = 0;
-    RGap = RCount = RAllocated = 0;
-    VGap = VCount = VAllocated = 0;
-    FCount = 0;
-    Modified = 0;
-    BlockMode = bmStream;
-    ExtendGrab = 0;
-    AutoExtend = 0;
-    MatchLen = MatchCount = 0;
 #ifdef CONFIG_UNDOREDO
     US.Num = 0;
     US.Data = 0;
@@ -43,18 +66,10 @@ EBuffer::EBuffer(int createFlags, EModel **ARoot, const char * /*AName*/)
     US.Record = 1;
     US.Undo = 0;
 #endif
-#ifdef CONFIG_BOOKMARKS
-    BMCount = 0;
-    BMarks = 0;
-#endif
 #ifdef CONFIG_OBJ_ROUTINE
     rlst.Count = 0;
     rlst.Lines = 0;
     Routines = 0;
-#endif
-#ifdef CONFIG_WORD_HILIT
-    WordList = 0;
-    WordCount = 0;
 #endif
     //Name = strdup(AName);
     Allocate(0);
@@ -63,10 +78,6 @@ EBuffer::EBuffer(int createFlags, EModel **ARoot, const char * /*AName*/)
     Flags = (Mode->Flags);
     BFI(this, BFI_Undo) = 0;
     BFI(this, BFI_ReadOnly) = 0;
-    Modified = 0;
-    MinRedraw = -1;
-    MaxRedraw = -1;
-    RedrawToEos = 0;
 #ifdef CONFIG_SYNTAX_HILIT
     StartHilit = 0;
     EndHilit = -1;
@@ -98,13 +109,8 @@ EBuffer::~EBuffer() {
     if (FileName)
 	free(FileName);
 #ifdef CONFIG_BOOKMARKS
-    if (BMCount != 0) {
-        for (int i = 0; i < BMCount; i++)
-            free(BMarks[i].Name);
-        free(BMarks);
-        BMarks = 0;
-        BMCount = 0;
-    }
+    vector_iterate(EBookmark*, BMarks, it)
+	delete *it;
 #endif
 #ifdef CONFIG_OBJ_ROUTINE
     if (rlst.Lines) {
@@ -129,15 +135,7 @@ int EBuffer::Clear() {
 #ifdef CONFIG_SYNTAX_HILIT
     EndHilit = -1;
     StartHilit = 0;
-
-    while (WordCount--)
-    {
-        free(WordList[WordCount]);
-    }
-    free(WordList);
-
-    WordCount = 0;
-    WordList = 0;
+    WordList.clear();
 #endif
 #ifdef CONFIG_OBJ_ROUTINE
     rlst.Count = 0;
@@ -163,11 +161,15 @@ int EBuffer::Clear() {
 #ifdef CONFIG_UNDOREDO
     FreeUndo();
 #endif
+
+#ifdef CONFIG_FOLDS
     if (FCount != 0) {
         free(FF);
         FCount = 0;
         FF = 0;
     }
+#endif
+
     return 0;
 }
 
@@ -252,7 +254,7 @@ int EBuffer::LoadRegion(EPoint * /*A*/, int /*FH*/, int /*StripChar*/, int /*Lin
     return 0;
 }
 
-int EBuffer::InsertLine(EPoint Pos, int ACount, const char *AChars) {
+int EBuffer::InsertLine(const EPoint& Pos, size_t ACount, const char *AChars) {
     if (InsLine(Pos.Row, 0) == 0) return 0;
     if (InsText(Pos.Row, Pos.Col, ACount, AChars) == 0) return 0;
     return 1;
@@ -285,27 +287,30 @@ int EBuffer::UpdateMark(EPoint &M, int Type, int Row, int Col, int Rows, int Col
         switch (BlockMode) {
         case bmLine:
         case bmColumn:
-            if (M.Row >= Row)
+            if (M.Row >= Row) {
                 if (InRange(Row, M.Row, Row + Rows))
                     M.Row = Row;
                 else
                     M.Row -= Rows;
+            }
             break;
         case bmStream:
             if (Cols) {
                 if (M.Row == Row)
-                    if (M.Col >= Col)
+                    if (M.Col >= Col) {
                         if (M.Col < Col + Cols)
                             M.Col = Col;
-                else
-                    M.Col -= Cols;
+                        else
+                            M.Col -= Cols;
+                    }
             }
             if (Rows) {
-                if (M.Row >= Row)
+                if (M.Row >= Row) {
                     if (M.Row < Row + Rows) {
                         M.Row = Row;
                         M.Col = 0;
                     } else M.Row -= Rows;
+                }
             }
         }
         break;
@@ -385,7 +390,7 @@ int EBuffer::UpdateMarker(int Type, int Row, int Col, int Rows, int Cols) {
         rlst.Lines[i] = M.Row;
     }
 #endif
-    
+#ifdef CONFIG_FOLDS
     for (int f = 0; f < FCount; f++) {
         EPoint M;
         
@@ -394,10 +399,10 @@ int EBuffer::UpdateMarker(int Type, int Row, int Col, int Rows, int Cols) {
         UpdateMark(M, Type, Row, Col, Rows, Cols);
         FF[f].line = M.Row;
     }
-    
+#endif
 #ifdef CONFIG_BOOKMARKS
-    for (int b = 0; b < BMCount; b++)
-        UpdateMark(BMarks[b].BM, Type, Row, Col, Rows, Cols);
+    vector_iterate(EBookmark*, BMarks, it)
+        UpdateMark((*it)->GetPoint(), Type, Row, Col, Rows, Cols);
 #endif
     
     if (OldBB.Row != BB.Row) {
@@ -444,7 +449,15 @@ int EBuffer::SetFileName(const char *AFileName, const char *AMode) {
     FileName = strdup(AFileName);
     Mode = 0;
     if (AMode)
-        Mode = FindMode(AMode);
+    {
+	Mode = FindMode(AMode);
+	if (Mode == 0)
+	{
+	    StlString AMODE(AMode);
+	    AMODE.toupper();
+	    Mode = FindMode(AMODE.c_str());
+	}
+    }
     if (Mode == 0)
         Mode = GetModeForName(AFileName);
     assert(Mode != 0);
@@ -459,6 +472,8 @@ int EBuffer::SetFileName(const char *AFileName, const char *AMode) {
 }
 
 int EBuffer::SetPos(int Col, int Row, int tabMode) {
+    if (!VCount)
+        return 1;
     assert (Col >= 0 && Row >= 0 && Row < VCount);
 
 #ifdef CONFIG_UNDOREDO
@@ -590,11 +605,13 @@ int EBuffer::DelLine(int Row, int DoMark) {
         if (ExposeRow(Row) == 0) return 0;
     VLine = RToV(Row);
     assert(VLine != -1);
-    
+
+#ifdef CONFIG_FOLDS
     if (FindFold(Row) != -1) {
         if (FoldDestroy(Row) == 0) return 0;
     }
-    
+#endif
+
     VLine = RToV(Row);
     assert(VLine != -1);
     
@@ -716,7 +733,7 @@ int EBuffer::InsLine(int Row, int DoAppend, int DoMark) {
     return 1;
 }
 
-int EBuffer::DelChars(int Row, int Ofs, int ACount) {
+int EBuffer::DelChars(int Row, int Ofs, size_t ACount) {
     PELine L;
     
 //    printf("DelChars: %d:%d %d\n", Row, Ofs, ACount);
@@ -752,7 +769,7 @@ int EBuffer::DelChars(int Row, int Ofs, int ACount) {
     return 1;
 }
 
-int EBuffer::InsChars(int Row, int Ofs, int ACount, const char *Buffer) {
+int EBuffer::InsChars(int Row, int Ofs, size_t ACount, const char *Buffer) {
     PELine L;
     
 //    printf("InsChars: %d:%d %d\n", Row, Ofs, ACount);
@@ -810,7 +827,7 @@ int EBuffer::UnTabPoint(int Row, int Col) {
     return 1;
 }
 
-int EBuffer::ChgChars(int Row, int Ofs, int ACount, const char * /*Buffer*/) {
+int EBuffer::ChgChars(int Row, int Ofs, size_t ACount, const char * /*Buffer*/) {
     PELine L;
     
     assert(Row >= 0 && Row < RCount && Ofs >= 0);
@@ -840,7 +857,7 @@ int EBuffer::ChgChars(int Row, int Ofs, int ACount, const char * /*Buffer*/) {
     return 1;
 }
 
-int EBuffer::DelText(int Row, int Col, int ACount, int DoMark) {
+int EBuffer::DelText(int Row, int Col, size_t ACount, int DoMark) {
     int L, B, C;
 
 //    printf("DelTExt: %d:%d %d\n", Row, Col, ACount);
@@ -868,11 +885,11 @@ int EBuffer::DelText(int Row, int Col, int ACount, int DoMark) {
     return 1;
 }
 
-int EBuffer::InsText(int Row, int Col, int ACount, const char *ABuffer, int DoMark) {
+int EBuffer::InsText(int Row, int Col, size_t ACount, const char *ABuffer, int DoMark) {
     int B, L;
     
 //    printf("InsText: %d:%d %d\n", Row, Col, ACount);
-    assert(Row >= 0 && Row < RCount && Col >= 0 && ACount >= 0);
+    assert(Row >= 0 && Row < RCount && Col >= 0);
     if (ACount == 0) return 1;
     if (Modify() == 0) return 0;
     
@@ -889,8 +906,8 @@ int EBuffer::InsText(int Row, int Col, int ACount, const char *ABuffer, int DoMa
     return 1;
 }
 
-int EBuffer::PadLine(int Row, int Length) {
-    int L;
+int EBuffer::PadLine(int Row, size_t Length) {
+    size_t L;
     
     L = LineLen(Row);
     if (L < Length)
@@ -899,7 +916,7 @@ int EBuffer::PadLine(int Row, int Length) {
     return 1;
 }
   
-int EBuffer::InsLineText(int Row, int Col, int ACount, int LCol, PELine Line) {
+int EBuffer::InsLineText(int Row, int Col, size_t ACount, int LCol, PELine Line) {
     int Ofs, Pos, TPos, C, B, L;
     
     //fprintf(stderr, "\n\nInsLineText: %d:%d %d %d", Row, Col, ACount, LCol);
@@ -965,7 +982,9 @@ int EBuffer::SplitLine(int Row, int Col) {
     } else {
         UpdateMarker(umSplitLine, Row, Col, 0, 0);
         if (InsLine(Row, 1, 0) == 0) return 0;
-        RLine(Row)->StateE = short((Row > 0) ? RLine(Row - 1)->StateE : 0);
+#ifdef CONFIG_SYNTAX_HILIT
+	RLine(Row)->StateE = short((Row > 0) ? RLine(Row - 1)->StateE : 0);
+#endif
         if (Col < LineLen(Row)) {
             int P, L;
             //if (RLine(Row)->ExpandTabs(Col, -2, &Flags) == 0) return 0;
