@@ -6,9 +6,17 @@
  *
  */
 
-#include "fte.h"
+#include "e_tags.h"
 
 #ifdef CONFIG_TAGS
+
+#include "o_buflist.h"
+#include "s_files.h"
+#include "s_util.h"
+#include "sysdep.h"
+
+#include <fcntl.h>
+
 struct TagData {
     int Tag;        // tag name pos
     int FileName;
@@ -18,13 +26,29 @@ struct TagData {
 };
 
 struct TagStack {
-    char *FileName;
+    static TagStack* TStack;
+
+    StlString CurrentTag;
+    StlString FileName;
     int Line, Col;
-    TagStack *Next;
     int TagPos;
-    char *CurrentTag;
+
+    TagStack(const char* tag, const char* file, int line, int col, int pos) :
+        CurrentTag(tag), FileName(file), Line(line), Col(col), TagPos(pos), Next(TStack)
+    {
+        TStack = this;
+    }
+    ~TagStack()
+    {
+        TStack = Next;
+    }
+
+private:
+    TagStack *Next;
 };
 
+TagStack* TagStack::TStack = 0;
+
 static char *TagMem = 0;
 static int TagLen = 0;
 static int ATagMem  = 0;
@@ -37,11 +61,10 @@ static int TagFileCount = 0;
 static int *TagFiles = 0;
 
 static int TagFilesLoaded = 0;   // tag files are loaded at first lookup
-static char *CurrentTag = 0;
+static char *CurrentTag;
 static int TagPosition = -1;
-static TagStack *TStack;
 
-static int AllocMem(char *Mem, int Len) { /*FOLD00*/
+static int AllocMem(const char *Mem, size_t Len) {
     int N = 1024;
     char *NM;
     int TagPos = TagLen;
@@ -59,7 +82,7 @@ static int AllocMem(char *Mem, int Len) { /*FOLD00*/
     return TagPos;
 }
 
-static int AddTag(int Tag, int FileName, int TagBase, int Line, int StrFind) { /*FOLD00*/
+static int AddTag(int Tag, int FileName, int TagBase, int Line, int StrFind) {
     int N;
 
     N = 1024;
@@ -82,19 +105,37 @@ static int AddTag(int Tag, int FileName, int TagBase, int Line, int StrFind) { /
     return 0;
 }
 
-int cmptags(const void *p1, const void *p2) { /*FOLD00*/
-    return strcmp(TagMem + TagD[*(int *)p1].Tag,
-                  TagMem + TagD[*(int *)p2].Tag);
+#if defined(__IBMCPP__)
+int _LNK_CONV cmptags(const void *p1, const void *p2) {
+#else
+int cmptags(const void *p1, const void *p2) {
+#endif
+    //return strcmp(TagMem + TagD[*(int *)p1].Tag,
+    //              TagMem + TagD[*(int *)p2].Tag);
+    int r = strcmp(TagMem + TagD[*(int *)p1].Tag,
+		   TagMem + TagD[*(int *)p2].Tag);
+    if (r != 0)
+	return r;
+
+    r = strcmp(TagMem + TagD[*(int *)p1].FileName,
+	       TagMem + TagD[*(int *)p2].FileName);
+    if (r != 0)
+	return r;
+
+    if (TagD[*(int *)p1].Line != TagD[*(int *)p2].Line)
+	r = TagD[*(int *)p1].Line < TagD[*(int *)p2].Line ? -1 : 1;
+
+    return r;
 }
 
-int SortTags() { /*FOLD00*/
+int SortTags() {
     int *NI;
     int i;
 
     if (CTags == 0)
         return 0;
 
-    NI = (int *)realloc((void *)TagI, CTags * sizeof(int));
+    NI = (int *)realloc(TagI, CTags * sizeof(int));
     if (NI == 0)
         return -1;
     TagI = NI;
@@ -106,7 +147,7 @@ int SortTags() { /*FOLD00*/
     return 0;
 }
 
-int TagsLoad(int id) { /*FOLD00*/
+int TagsLoad(int id) {
     //char line[2048];
     char *tags;
     int fd;
@@ -122,12 +163,12 @@ int TagsLoad(int id) { /*FOLD00*/
     if (fstat(fd, &sb) == -1)
         return -1;
 
-    if ((tags = (char *)malloc(sb.st_size)) == 0) {
+    if ((tags = (char *)malloc((size_t)sb.st_size)) == 0) {
         close(fd);
         return -1;
     }
 
-    size = read(fd, tags, sb.st_size);
+    size = read(fd, tags, (size_t)sb.st_size);
     close(fd);
     if (size != sb.st_size)
         return -1;
@@ -135,10 +176,10 @@ int TagsLoad(int id) { /*FOLD00*/
     if (TagMem == 0) { // preallocate (useful when big file)
         char *NM;
 
-        NM = (char *)realloc((void *)TagMem, TagLen + sb.st_size);
+        NM = (char *)realloc((void *)TagMem, TagLen + (size_t)sb.st_size);
         if (NM != 0) {
             TagMem = NM;
-            ATagMem = TagLen + sb.st_size;
+            ATagMem = TagLen + (int)sb.st_size;
         }
     }
 
@@ -174,6 +215,8 @@ int TagsLoad(int id) { /*FOLD00*/
 
         MFile = MTag + TagL;
 
+	//printf("TSTR  %s\n", LTag);
+	//printf("FSTR  %s\n", LTag + TagL);
         if (LLine[0] == '/') {
             char *AStr = LLine;
             char *p = AStr + 1;
@@ -194,9 +237,9 @@ int TagsLoad(int id) { /*FOLD00*/
             *d = 0;
             if (stricmp(d - 10, "/*FOLD00*/") == 0)
                 d[-11] = 0; /* remove our internal folds */
-
+	    //printf("MSTR  %s\n", AStr);
             MStr = AllocMem(AStr, strlen(AStr) + 1);
-            if (MStr == -1)
+	    if (MStr == -1)
                 break;
 
             if (AddTag(MTag, MFile, TagFiles[id], -1, MStr) == -1)
@@ -210,7 +253,7 @@ int TagsLoad(int id) { /*FOLD00*/
     return 0;
 }
 
-int TagsAdd(char *FileName) { /*FOLD00*/
+int TagsAdd(const char *FileName) {
     int *NewT;
     int NewF;
 
@@ -226,31 +269,23 @@ int TagsAdd(char *FileName) { /*FOLD00*/
     return 1;
 }
 
-int TagsSave(FILE *fp) { /*FOLD00*/
+int TagsSave(FILE *fp) {
     for (int i = 0; i < TagFileCount; i++)
-        fprintf(fp, "T|%s\n", TagMem + TagFiles[i]);
-    return 1;
+        if (fprintf(fp, "T|%s\n", TagMem + TagFiles[i]) < 0)
+            return ErFAIL;
+
+    return ErOK;
 }
 
-static void ClearTagStack() { /*FOLD00*/
-    TagStack *T;
-
-    if (CurrentTag) {
-        free(CurrentTag);
-        CurrentTag = 0;
-    }
+static void ClearTagStack() {
+    free(CurrentTag);
+    CurrentTag = 0;
     TagPosition = -1;
-    while (TStack) {
-        T = TStack;
-        TStack = TStack->Next;
-
-        free(T->CurrentTag);
-        free(T->FileName);
-        free(T);
-    }
+    while (TagStack::TStack)
+        delete TagStack::TStack;
 }
 
-int TagLoad(char *FileName) { /*FOLD00*/
+int TagLoad(const char *FileName) {
     if (TagsAdd(FileName) == 0)
         return 0;
     ClearTagStack();
@@ -266,7 +301,7 @@ int TagLoad(char *FileName) { /*FOLD00*/
     return 1;
 }
 
-static int LoadTagFiles() { /*FOLD00*/
+static int LoadTagFiles() {
     int i;
 
     assert(TagFilesLoaded == 0);
@@ -283,7 +318,7 @@ static int LoadTagFiles() { /*FOLD00*/
     return 1;
 }
 
-void TagClear() { /*FOLD00*/
+void TagClear() {
     free(TagD);
     free(TagI);
     TagD = 0;
@@ -304,7 +339,7 @@ void TagClear() { /*FOLD00*/
     ClearTagStack();
 }
 
-static int GotoFilePos(EView *View, char *FileName, int Line, int Col) { /*FOLD00*/
+static int GotoFilePos(EView *View, const char *FileName, int Line, int Col) {
     if (FileLoad(0, FileName, 0, View) == 0)
         return 0;
     if (((EBuffer *)ActiveModel)->Loaded == 0)
@@ -313,12 +348,12 @@ static int GotoFilePos(EView *View, char *FileName, int Line, int Col) { /*FOLD0
     return 1;
 }
 
-static int GotoTag(int M, EView *View) { /*FOLD00*/
+static int GotoTag(int M, EView *View) {
     char path[MAXPATH];
     char Dir[MAXPATH];
     TagData *TT = &TagD[TagI[M]];
 
-    JustDirectory(TagMem + TT->TagBase, Dir);
+    JustDirectory(TagMem + TT->TagBase, Dir, sizeof(Dir));
 
     if (IsFullPath(TagMem + TT->FileName)) {
         strcpy(path, TagMem + TT->FileName);
@@ -340,29 +375,15 @@ static int GotoTag(int M, EView *View) { /*FOLD00*/
     return 1;
 }
 
-static int PushPos(EBuffer *B) { /*FOLD00*/
-    TagStack *T;
-
-    T = (TagStack *)malloc(sizeof(TagStack));
-    if (T == 0)
-        return 0;
-    T->FileName = strdup(B->FileName);
-    if (T->FileName == 0) {
-        free(T);
-        return 0;
-    }
-    T->Line = B->VToR(B->CP.Row);
-    T->Col = B->CP.Col;
-    T->Next = TStack;
-    T->CurrentTag = CurrentTag;
-    CurrentTag = 0;
-    T->TagPos = TagPosition;
-    TagPosition = -1;
-    TStack = T;
+static int PushPos(EBuffer *B)
+{
+    (void) new TagStack(CurrentTag, B->FileName,
+			B->VToR(B->CP.Row), B->CP.Col,
+			TagPosition);
     return 1;
 }
 
-int TagGoto(EView *View, char *Tag) {
+int TagGoto(EView *View, const char *Tag) {
     assert(Tag != 0);
 
     if (TagFilesLoaded == 0)
@@ -384,6 +405,7 @@ int TagGoto(EView *View, char *Tag) {
             if (GotoTag(M, View) == 0)
                 return 0;
 
+            free(CurrentTag);
             CurrentTag = strdup(Tag);
             TagPosition = M;
             return 1;
@@ -396,7 +418,7 @@ int TagGoto(EView *View, char *Tag) {
     return 0; // tag not found
 }
 
-int TagFind(EBuffer *B, EView *View, char *Tag) { /*FOLD00*/
+int TagFind(EBuffer *B, EView *View, const char *Tag) {
     assert(View != 0 && Tag != 0 && B != 0);
 
     if (TagFilesLoaded == 0)
@@ -404,16 +426,14 @@ int TagFind(EBuffer *B, EView *View, char *Tag) { /*FOLD00*/
             return 0;
 
     int L = 0, R = CTags, M, cmp;
-
+    //printf("TAGFIND %s  %s   tp:%d\n", Tag,  CurrentTag, TagPosition);
     if (CurrentTag) {
         if (strcmp(CurrentTag, Tag) == 0) {
-            if (PushPos(B) == 0)
-                return 0;
+            if (!TagStack::TStack || TagPosition != TagStack::TStack->TagPos)
+                if (PushPos(B) == 0)
+                    return 0;
 
-            CurrentTag = strdup(Tag);
-            if (CurrentTag == 0)
-                return 0;
-            TagPosition = TStack->TagPos;
+            TagPosition = TagStack::TStack->TagPos;
 
             return TagNext(View);
         }
@@ -435,6 +455,7 @@ int TagFind(EBuffer *B, EView *View, char *Tag) { /*FOLD00*/
             if (GotoTag(M, View) == 0)
                 return 0;
 
+            free(CurrentTag); // in case it already exists.
             CurrentTag = strdup(Tag);
             TagPosition = M;
 
@@ -448,7 +469,7 @@ int TagFind(EBuffer *B, EView *View, char *Tag) { /*FOLD00*/
     return 0; // tag not found
 }
 
-int TagDefined(char *Tag) {
+int TagDefined(const char *Tag) {
     int L = 0, R = CTags, M, cmp;
 
     if (TagFilesLoaded == 0) // !!! always?
@@ -471,7 +492,7 @@ int TagDefined(char *Tag) {
     return 0; // tag not found
 }
 
-int TagComplete(char **Words, int *WordsPos, int WordsMax, char *Tag) {
+int TagComplete(char **Words, int *WordsPos, int WordsMax, const char *Tag) {
     if ((Tag == NULL) || (Words == NULL) || (*WordsPos >= WordsMax))
         return 0;
 
@@ -523,12 +544,11 @@ int TagComplete(char **Words, int *WordsPos, int WordsMax, char *Tag) {
     return 0; // tag not found
 }
 
-int TagNext(EView *View) { /*FOLD00*/
+int TagNext(EView *View) {
     assert(View != 0);
 
-    if (CurrentTag == 0 || TagPosition == -1) {
+    if (CurrentTag == 0 || TagPosition == -1)
         return 0;
-    }
 
     if (TagPosition < CTags - 1 && strcmp(CurrentTag, TagMem + TagD[TagI[TagPosition + 1]].Tag) == 0) {
         TagPosition++;
@@ -540,7 +560,7 @@ int TagNext(EView *View) { /*FOLD00*/
     return 0;
 }
 
-int TagPrev(EView *View) { /*FOLD00*/
+int TagPrev(EView *View) {
     assert(View != 0);
 
     if (CurrentTag == 0 || TagPosition == -1) {
@@ -558,25 +578,18 @@ int TagPrev(EView *View) { /*FOLD00*/
     return 0;
 }
 
-int TagPop(EView *View) { /*FOLD00*/
-    TagStack *T = TStack;
+int TagPop(EView *View) {
 
     assert(View != 0);
 
+    delete TagStack::TStack;
+
+    TagStack* T = TagStack::TStack;
     if (T) {
-        TStack = T->Next;
-
-        if (CurrentTag)
-            free(CurrentTag);
-        CurrentTag = T->CurrentTag;
+        free(CurrentTag);
+        CurrentTag = strdup(T->CurrentTag.c_str());
         TagPosition = T->TagPos;
-
-        if (GotoFilePos(View, T->FileName, T->Line, T->Col) == 0) {
-            free(T);
-            return 0;
-        }
-        free(T);
-        return 1;
+        return GotoFilePos(View, T->FileName.c_str(), T->Line, T->Col);
     }
     View->Msg(S_INFO, "Tag stack empty.");
     return 0;
