@@ -7,16 +7,25 @@
  *
  */
 
+// UNIX specific routines
+
+#include "c_bind.h"
+#include "c_config.h"
+#include "c_history.h"
+#include "i_modelview.h"
+#include "i_view.h"
+#include "o_buflist.h"
+#include "s_util.h"
+
 #ifdef WINNT
 #include "e_win32.cpp"
 #else
 
-// UNIX specific routines
-
-#include "fte.h"
-
-#include <sys/wait.h>
+#include <errno.h>
+#include <fcntl.h>
 #include <signal.h>
+#include <stdio.h>
+#include <sys/wait.h>
 
 int EView::SysShowHelp(ExState &State, const char *word) {
     char options[128] = "";
@@ -35,8 +44,8 @@ int EView::SysShowHelp(ExState &State, const char *word) {
         word = wordAsk;
     }
 
-    sprintf(file, "/tmp/fte%d-man-%s", getpid(), word);
-    sprintf(command, "%s %s %s >'%s' 2>&1", HelpCommand, options, word, file);
+    snprintf(file, sizeof(file)-1, "/tmp/fte%d-man-%s", getpid(), word);
+    snprintf(command, sizeof(command)-1, "%s %s %s >'%s' 2>&1", HelpCommand, options, word, file);
 
     /// !!! why is this needed ???
 #define SYSCALL(call) while(((call) == -1) && (errno == EINTR))
@@ -48,8 +57,8 @@ int EView::SysShowHelp(ExState &State, const char *word) {
     if ((pid = fork()) == 0) {
         close(1);
         SYSCALL(err = open(file, O_CREAT | O_WRONLY | O_APPEND, S_IRWXU));
-	if (err != -1) {
-	    close(2);
+        if (err != -1) {
+            close(2);
             //dup(1); // ignore error output
             close(0);
             assert(open("/dev/null", O_RDONLY) == 0);
@@ -59,8 +68,9 @@ int EView::SysShowHelp(ExState &State, const char *word) {
 #endif
                    word, NULL);
             // execlp("/bin/sh", "sh", "-c", command, NULL);
-        }
-        perror("Can't Exec Command\n");
+            perror("Can't exec command");
+        } else
+            perror("Can't open file");
         exit(-1);
     } else if (pid < 0) {
         perror("Can't fork");
@@ -68,13 +78,15 @@ int EView::SysShowHelp(ExState &State, const char *word) {
     }
     SYSCALL(err = waitpid(pid, &status, 0));
     if (err == -1) {
-        perror("Waitpid failed\n");
+        perror("waitpid failed");
         return 0;
     }
 
     // int rc = system(command);
 
     err = FileLoad(0, file, "CATBS", this);
+    BFI((EBuffer*)(this->Model), BFI_ReadOnly) = 1;
+
     unlink(file);
 
     if (err == 0){
