@@ -7,29 +7,46 @@
  *
  */
 
-#include "fte.h"
+#include "egui.h"
+
+#include "c_bind.h"
+#include "c_config.h"
+#include "c_desktop.h"
+#include "c_history.h"
+#include "e_buffer.h"
+#include "e_tags.h"
+#include "i_modelview.h"
+#include "i_view.h"
+#include "o_cvsbase.h"
+#include "o_directory.h"
+#include "o_messages.h"
+#include "o_svnbase.h"
+#include "s_files.h"
+#include "s_string.h"
+#include "s_util.h"
+
+#include <stdio.h>
 
 int LastEventChar = -1;
 
-EFrame::EFrame(int XSize, int YSize): GFrame(XSize, YSize) {
-    CMap = 0;
-    CModel = 0;
-    frames = this;
+EFrame::EFrame(int XSize, int YSize) :
+    GFrame(XSize, YSize),
+    CMap(0),
+    CModel(0)
+{
 }
 
 EFrame::~EFrame() {
 }
 
 void EFrame::Update() {
-    GxView *V = (GxView *)Active;
-
-    if (V) {
+    if (Active) {
         if (CModel != ActiveModel && ActiveModel) {
             char Title[256] = ""; //fte: ";
             char STitle[256] = ""; //"fte: ";
 
-            ActiveModel->GetTitle((char *)(Title + 0), sizeof(Title) - 0,
-                                  (char *)(STitle + 0), sizeof(STitle) - 0);
+            ActiveModel->GetTitle(Title, sizeof(Title),
+                                  STitle, sizeof(STitle));
             ConSetTitle(Title, STitle);
             CModel = ActiveModel;
         }
@@ -69,16 +86,20 @@ void EFrame::UpdateMenu() {
     GFrame::UpdateMenu();
 }
 
-EGUI::EGUI(int &argc, char **argv, int XSize, int YSize): GUI(argc, argv, XSize, YSize) {
-    ActiveMap = 0;
-    OverrideMap = 0;
-    strcpy(CharMap, "");
+EGUI::EGUI(int &argc, char **argv, int XSize, int YSize) :
+    GUI(argc, argv, XSize, YSize),
+    ActiveMap(0),
+    OverrideMap(0)
+{
+    CharMap[0] = 0;
 }
 
-EGUI::~EGUI() {
+EGUI::~EGUI()
+{
+    assert(frames == 0);
 }
 
-int EGUI::ExecCommand(GxView *view, int Command, ExState &State) {
+int EGUI::ExecCommand(GxView *view, ExCommands Command, ExState &State) {
     if (Command & CMD_EXT)
         return ExecMacro(view, Command & ~CMD_EXT);
 
@@ -87,7 +108,7 @@ int EGUI::ExecCommand(GxView *view, int Command, ExState &State) {
 
     if (view->IsModelView()) {
         ExModelView *V = (ExModelView *)view->Top;
-        EView *View = V->View;
+        EView *View = V->GetView();
 
         switch (Command) {
         case ExFileClose:               return FileClose(View, State);
@@ -99,10 +120,12 @@ int EGUI::ExecCommand(GxView *view, int Command, ExState &State) {
 #else
             return ErFAIL;
 #endif
+        default:
+            ;
         }
     }
     switch (Command) {
-    case ExWinRefresh:              view->Repaint(); return 1;
+    case ExWinRefresh:              view->Repaint(); return ErOK;
     case ExWinNext:                 return WinNext(view);
     case ExWinPrev:                 return WinPrev(view);
     case ExShowEntryScreen:         return ShowEntryScreen();
@@ -123,7 +146,7 @@ int EGUI::ExecCommand(GxView *view, int Command, ExState &State) {
     case ExDesktopSave:
         if (DesktopFileName[0] != 0)
             return SaveDesktop(DesktopFileName);
-        return 0;
+        return ErOK;
     case ExChangeKeys:
         {
             char kmaps[64] = "";
@@ -131,14 +154,15 @@ int EGUI::ExecCommand(GxView *view, int Command, ExState &State) {
 
             if (State.GetStrParam(0, kmaps, sizeof(kmaps)) == 0) {
                 SetOverrideMap(0, 0);
-                return 0;
+                return ErFAIL;
             }
-            m = FindEventMap(kmaps);
-            if (m == 0)
-                return 0;
+            if ((m = FindEventMap(kmaps)) == 0)
+                return ErFAIL;
             SetOverrideMap(m->KeyMap, m->Name);
-            return 1;
+            return ErOK;
         }
+    default:
+        ;
     }
     return view->ExecCommand(Command, State);
 }
@@ -170,31 +194,31 @@ int EGUI::ExecMacro(GxView *view, int Macro) {
 
         for (j = 0; j < m->cmds[i].repeat; j++) {
             State.Pos = i + 1;
-            if (ExecCommand(view, m->cmds[i].u.num, State) == 0 && !m->cmds[i].ign)
-            {
+            if (ExecCommand(view, (ExCommands)m->cmds[i].u.num, State) == 0 && !m->cmds[i].ign)
                 return ErFAIL;
-            }
         }
         State.Pos = i;
     }
     return ErOK;
 }
 
-void EGUI::SetMsg(char *Msg) {
-    static char CharMap[128] = "";
+void EGUI::SetMsg(const char *Msg) {
+    size_t len = Msg ? strlen(Msg) + 3 : 1;
+    char CharMap[len];
+
+    if (Msg) {
+	CharMap[0] = '[';
+	memcpy(CharMap + 1, Msg, len);
+	CharMap[len + 1] = ']';
+	CharMap[len + 2] = 0;
+    } else
+	CharMap[0] = 0;
 
-    if (Msg == 0) {
-        strcpy(CharMap, "");
-    } else {
-        strcat(CharMap, "[");
-        strcat(CharMap, Msg);
-        strcat(CharMap, "]");
-    }
     if (ActiveModel)
-        ActiveModel->Msg(S_INFO, CharMap);
+	ActiveModel->Msg(S_INFO, CharMap);
 }
 
-void EGUI::SetOverrideMap(EKeyMap *aMap, char *ModeName) {
+void EGUI::SetOverrideMap(EKeyMap *aMap, const char *ModeName) {
     OverrideMap = aMap;
     if (aMap == 0)
         SetMsg(0);
@@ -210,7 +234,7 @@ void EGUI::SetMap(EKeyMap *aMap, KeySel *ks) {
         SetMsg(0);
     } else {
         if (ks != 0) {
-            GetKeyName(key, *ks);
+            GetKeyName(key, sizeof(key), *ks);
             SetMsg(key);
         }
     }
@@ -291,11 +315,11 @@ void EGUI::DispatchCommand(GxView *view, TEvent &Event) {
         ExState State;
         State.Macro = 0;
         State.Pos = 0;
-        ExecCommand(view, Event.Msg.Command, State);
+        ExecCommand(view, (ExCommands)Event.Msg.Command, State);
         Event.What = evNone;
     } else if (Event.Msg.Command >= 65536) {
         Event.Msg.Command -= 65536;
-        ExecMacro(view, Event.Msg.Command);
+        ExecMacro(view, (int)Event.Msg.Command);
         Event.What = evNone;
     }
 }
@@ -305,13 +329,13 @@ void EGUI::DispatchEvent(GFrame *frame, GView *view, TEvent &Event) {
 
     if (Event.What == evNone ||
         (Event.What == evMouseMove && Event.Mouse.Buttons == 0))
-        return ;
+        return;
 
     if (Event.What == evNotify && Event.Msg.Command == cmPipeRead) {
-        Event.Msg.Model->NotifyPipe(Event.Msg.Param1);
+        Event.Msg.Model->NotifyPipe((int)Event.Msg.Param1);
         return;
     }
-    if (xview->GetEventMap() != 0) {
+    if (xview && xview->GetEventMap() != 0) {
         switch (Event.What) {
         case evKeyDown:
             DispatchKey(xview, Event);
@@ -319,7 +343,7 @@ void EGUI::DispatchEvent(GFrame *frame, GView *view, TEvent &Event) {
         case evCommand:
             if (Event.Msg.Command >= 65536) {
                 DispatchCommand(xview, Event);
-            } else {
+            } else
                 switch (Event.Msg.Command) {
                 case cmClose:
                     {
@@ -328,7 +352,6 @@ void EGUI::DispatchEvent(GFrame *frame, GView *view, TEvent &Event) {
                         return;
                     }
                 }
-            }
         }
     }
     GUI::DispatchEvent(frame, view, Event);
@@ -383,10 +406,21 @@ int EGUI::FileCloseX(EView *View, int CreateNew, int XClose) {
             return 0;
         }
 #endif
-
-        if (ActiveModel == 0) {
-            StopLoop();
+        if (ActiveView) {
+            /*
+             * FIXME: hack - fixing problem when closing window
+             * i.e.: F1 - Split - Close
+             * was leaving cursor in wrong window
+             * so going to next view and back for now
+             * fixes problem
+             */
+            ActiveView->MView->Win->Parent->SelectNext(0);
+            ActiveView->MView->Win->Parent->SelectNext(1);
         }
+
+        if (ActiveModel == 0)
+            StopLoop();
+
         return 1;
     }
     return 0;
@@ -414,26 +448,20 @@ int EGUI::FileCloseAll(EView *View, ExState &State) {
 }
 
 int EGUI::WinHSplit(GxView *View) {
-    GxView *view;
-    ExModelView *edit;
-    EView *win;
     int W, H;
 
     View->ConQuerySize(&W, &H);
 
     if (H < 8)
         return 0;
-    view = new GxView(View->Parent);
-    if (view == 0)
-        return 0;
-    win = new EView(ActiveModel);
-    if (win == 0)
-        return 0;
-    edit = new ExModelView(win);
-    if (edit == 0)
-        return 0;
+
+    GxView *view = new GxView(View->Parent);
+    EView *win = new EView(ActiveModel);
+    ExModelView *edit = new ExModelView(win);
+
     view->PushView(edit);
     view->Parent->SelectNext(0);
+    //fprintf(stderr, "WinHSplit %p   gv:%p\n", frames, view);
     return 1;
 }
 
@@ -476,24 +504,20 @@ int EGUI::WinResize(ExState &State, GxView *View) {
 }
 
 int EGUI::ExitEditor(EView *View) {
-    EModel *B = ActiveModel;
-
     // check/save modified files
-    while (ActiveModel) {
-        if (ActiveModel->CanQuit()) ;
-        else {
+    for (EModel *B = ActiveModel; ActiveModel; ActiveModel = ActiveModel->Next) {
+	if (!ActiveModel->CanQuit()) {
             View->SelectModel(ActiveModel);
             int rc = ActiveModel->ConfQuit(View->MView->Win, 1);
+            if (rc == 0)
+                return 0;
             if (rc == -2) {
                 View->FileSaveAll();
                 break;
             }
-            if (rc == 0)
-                return 0;
         }
 
-        ActiveModel = ActiveModel->Next;
-        if (ActiveModel == B)
+        if (ActiveModel->Next == B)
             break;
     }
 
@@ -506,9 +530,7 @@ int EGUI::ExitEditor(EView *View) {
         if (gx->GetStr("Save desktop As",
                        sizeof(DesktopFileName), DesktopFileName,
                        HIST_DEFAULT) != 0)
-        {
             SaveDesktop(DesktopFileName);
-        }
     }
 #endif
 
@@ -521,9 +543,9 @@ int EGUI::ExitEditor(EView *View) {
 
         View->Model->DeleteRelated();  // delete related views first
 
-        while (View->Model->Next != View->Model &&
-               View->Model->Next->CanQuit())
-            delete View->Model->Next;
+//        while (View->Model->Next != View->Model &&
+//               View->Model->Next->CanQuit())
+//            delete View->Model->Next;
 
         View->DeleteModel(View->Model);
     }
@@ -597,33 +619,28 @@ int EGUI::DesktopSaveAs(ExState &State, GxView *view) {
         if (view->GetFile("Save Desktop", sizeof(DesktopFileName), DesktopFileName, HIST_PATH, GF_SAVEAS) == 0)
             return 0;
 
-    if (DesktopFileName[0] != 0)
-        return SaveDesktop(DesktopFileName);
-    return 0;
+    if (!DesktopFileName[0])
+	return 0;
+
+    return SaveDesktop(DesktopFileName);
 }
 
 int EGUI::FrameNew() {
-    GxView *view;
-    ExModelView *edit;
 
     if (!multiFrame() && frames)
         return 0;
 
-    (void)new EFrame(ScreenSizeX, ScreenSizeY);
+    (void) new EFrame(ScreenSizeX, ScreenSizeY); // -> frames
     assert(frames != 0);
 
-    //frames->SetMenu("Main"); //??
+    GxView *view = new GxView(frames);
 
-    view = new GxView(frames);
-    assert(view != 0);
+    (void)new EView(ActiveModel); // ->ActiveView
 
-    (void)new EView(ActiveModel);
-    assert(ActiveView != 0);
-
-    edit = new ExModelView(ActiveView);
-    assert(edit != 0);
+    ExModelView *edit = new ExModelView(ActiveView);
     view->PushView(edit);
     frames->Show();
+    //fprintf(stderr, "FrameNew  %p\n", frames);
     return 1;
 }
 
@@ -631,30 +648,30 @@ int EGUI::FrameClose(GxView *View) {
     assert(frames != 0);
     assert(View != 0);
 
+    //fprintf(stderr, "FrameClose  %p  v:%p\n", frames, View);
     if (!frames->isLastFrame()) {
-        deleteFrame(frames);
+        delete frames;
     } else {
         if (ExitEditor(ActiveView) == 0)
             return 0;
-        deleteFrame(frames);
     }
     return 1;
 }
 
 int EGUI::FrameNext(GxView * /*View*/) {
-    if (!frames->isLastFrame()) {
-        frames->Next->Activate();
-        return 1;
-    }
-    return 0;
+    if (frames->isLastFrame())
+	return 0;
+
+    frames->Next->Activate();
+    return 1;
 }
 
 int EGUI::FramePrev(GxView * /*View*/) {
-    if (!frames->isLastFrame()) {
-        frames->Prev->Activate();
-        return 1;
-    }
-    return 0;
+    if (frames->isLastFrame())
+	return 0;
+
+    frames->Prev->Activate();
+    return 1;
 }
 
 #ifdef CONFIG_DESKTOP
@@ -674,29 +691,29 @@ int EGUI::findDesktop(char *argv[]) {
         //** 0: try curdir then "homedir"..
         //         fprintf(stderr, "ld: Mode 0\n");
         if (FileExists(DESKTOP_NAME))
-            ExpandPath(DESKTOP_NAME, DesktopFileName);
+            ExpandPath(DESKTOP_NAME, DesktopFileName, sizeof(DesktopFileName));
         else {
             //** Use homedir,
 #ifdef UNIX
-            ExpandPath("~/" DESKTOP_NAME, DesktopFileName);
+            ExpandPath("~/" DESKTOP_NAME, DesktopFileName, sizeof(DesktopFileName));
 #else
-            JustDirectory(argv[0], DesktopFileName);
-            strcat(DesktopFileName, DESKTOP_NAME);
-#endif
+            JustDirectory(argv[0], DesktopFileName, sizeof(DesktopFileName));
+            strlcat(DesktopFileName, DESKTOP_NAME, sizeof(DesktopFileName));
+#endif // UNIX
         }
         return FileExists(DesktopFileName);
 
     case 1:
     case 2:
         //** Try curdir, then it's owner(s)..
-        ExpandPath(".", DesktopFileName);
+        ExpandPath(".", DesktopFileName, sizeof(DesktopFileName));
         //fprintf(stderr, "ld: Mode 1 (start at %s)\n", DesktopFileName);
 
         for (;;) {
             //** Try current location,
             char *pe = DesktopFileName + strlen(DesktopFileName);
             Slash(DesktopFileName, 1);      // Add appropriate slash
-            strcat(DesktopFileName, DESKTOP_NAME);
+            strlcat(DesktopFileName, DESKTOP_NAME, sizeof(DesktopFileName));
 
             //fprintf(stderr, "ld: Mode 1 (trying %s)\n", DesktopFileName);
             if (FileExists(DesktopFileName)) {
@@ -712,9 +729,9 @@ int EGUI::findDesktop(char *argv[]) {
 
             if (p == NULL) {
                 //** No desktop! Set default name in current directory,
-                ExpandPath(".", DesktopFileName);
+                ExpandPath(".", DesktopFileName, sizeof(DesktopFileName));
                 Slash(DesktopFileName, 1);
-                strcat(DesktopFileName, DESKTOP_NAME);
+                strlcat(DesktopFileName, DESKTOP_NAME, sizeof(DesktopFileName));
 
                 SaveDesktopOnExit = 0;      // Don't save,
                 return 0;                   // NOT found!!
@@ -731,18 +748,17 @@ void EGUI::DoLoadDesktopOnEntry(int &/*argc*/, char **argv) {
     if (DesktopFileName[0] != 0) {
         if (IsDirectory(DesktopFileName)) {
             Slash(DesktopFileName, 1);
-            strcat(DesktopFileName, DESKTOP_NAME);
+            strlcat(DesktopFileName, DESKTOP_NAME, sizeof(DesktopFileName));
         }
 
         if (LoadDesktopOnEntry && FileExists(DesktopFileName))
             LoadDesktop(DesktopFileName);
     }
 }
-#endif
+#endif // CONFIG_DESKTOP
 
 void EGUI::EditorInit() {
     SSBuffer = new EBuffer(0, (EModel **)&SSBuffer, "Scrap");
-    assert(SSBuffer != 0);
     BFI(SSBuffer, BFI_Undo) = 0; // disable undo for clipboard
     ActiveModel = 0;
 }
@@ -757,20 +773,20 @@ int EGUI::InterfaceInit(int &/*argc*/, char ** /*argv*/) {
 void EGUI::DoLoadHistoryOnEntry(int &/*argc*/, char **argv) {
     if (HistoryFileName[0] == 0) {
 #ifdef UNIX
-        ExpandPath("~/.fte-history", HistoryFileName);
+        ExpandPath("~/.fte-history", HistoryFileName, sizeof(HistoryFileName));
 #else
-        JustDirectory(argv[0], HistoryFileName);
-        strcat(HistoryFileName, "fte.his");
-#endif
+        JustDirectory(argv[0], HistoryFileName, sizeof(HistoryFileName));
+        strlcat(HistoryFileName, "fte.his", sizeof(HistoryFileName));
+#endif // UNIX
     } else {
         char p[256];
 
-        ExpandPath(HistoryFileName, p);
+        ExpandPath(HistoryFileName, p, sizeof(p));
         if (IsDirectory(p)) {
             Slash(p, 1);
-            strcat(p, HISTORY_NAME);
+            strlcat(p, HISTORY_NAME, sizeof(p));
         }
-        strcpy(HistoryFileName, p);
+        strlcpy(HistoryFileName, p, sizeof(HistoryFileName));
     }
 
     if (KeepHistory && FileExists(HistoryFileName))
@@ -784,7 +800,7 @@ void EGUI::DoSaveHistoryOnExit() {
     // since we are exiting, free history
     ClearHistory();
 }
-#endif
+#endif // CONFIG_HISTORY
 
 int EGUI::CmdLoadFiles(int &argc, char **argv) {
     int QuoteNext = 0;
@@ -798,21 +814,24 @@ int EGUI::CmdLoadFiles(int &argc, char **argv) {
     int ReadOnly = 0;
 
     for (int Arg = 1; Arg < argc; Arg++) {
-        if (!QuoteAll && !QuoteNext && (argv[Arg][0] == '-')) {
-            if (argv[Arg][1] == '-') {
+	if (!QuoteAll && !QuoteNext && (argv[Arg][0] == '-')) {
+	    switch (argv[Arg][1]) {
+	    case '-':
                 if (strncmp(argv[Arg], "--debug", 7) != 0)
-                    QuoteAll = 1;
-            } else if (argv[Arg][1] == '!') {
+		    QuoteAll = 1;
+                Arg = argc;
+                break;
+	    case '!':
+	    case 'C':
+	    case 'c':
+	    case 'D':
+	    case 'd':
+	    case 'H':
                 // handled before
-            } else if (argv[Arg][1] == 'c' || argv[Arg][1] == 'C') {
-                // ^
-            } else if (argv[Arg][1] == 'D' || argv[Arg][1] == 'd') {
-                // ^
-            } else if (argv[Arg][1] == 'H') {
-                // ^
-            } else if (argv[Arg][1] == '+') {
-                QuoteNext = 1;
-            } else if (argv[Arg][1] == '#' || argv[Arg][1] == 'l') {
+                break;
+	    case '+': QuoteNext = 1; break;
+	    case '#':
+	    case 'l':
                 LineNum = 1;
                 ColNum = 1;
                 if (strchr(argv[Arg], ',')) {
@@ -820,32 +839,35 @@ int EGUI::CmdLoadFiles(int &argc, char **argv) {
                 } else {
                     GotoLine = (1 == sscanf(argv[Arg] + 2, "%d", &LineNum));
                 }
-                //                printf("Gotoline = %d, line = %d, col = %d\n", GotoLine, LineNum, ColNum);
-            } else if (argv[Arg][1] == 'r') {
-                ReadOnly = 1;
-            } else if (argv[Arg][1] == 'm') {
+		// printf("Gotoline = %d, line = %d, col = %d\n", GotoLine, LineNum, ColNum);
+                break;
+	    case 'r': ReadOnly = 1; break;
+	    case 'm':
                 if (argv[Arg][2] == 0) {
                     ModeOverride = 0;
                 } else {
                     ModeOverride = 1;
                     strcpy(Mode, argv[Arg] + 2);
-                }
-            } else if (argv[Arg][1] == 'T') {
-                TagsAdd(argv[Arg] + 2);
-            } else if (argv[Arg][1] == 't') {
-                TagGoto(ActiveView, argv[Arg] + 2);
-            } else {
-                DieError(2, "Invalid command line option %s", argv[Arg]);
+		}
+                break;
+#ifdef CONFIG_TAGS
+	    case 'T': TagsAdd(argv[Arg] + 2); break;
+	    case 't': TagGoto(ActiveView, argv[Arg] + 2); break;
+#endif
+	    default:
+		DieError(2, "Invalid command line option %s", argv[Arg]);
                 return 0;
-            }
+	    }
         } else {
-            char Path[MAXPATH];
-
             QuoteNext = 0;
-            if (ExpandPath(argv[Arg], Path) == 0 && IsDirectory(Path)) {
+#ifdef CONFIG_OBJ_DIRECTORY
+            char Path[MAXPATH];
+            if (ExpandPath(argv[Arg], Path, sizeof(Path)) == 0 && IsDirectory(Path)) {
                 EModel *m = new EDirectory(cfAppend, &ActiveModel, Path);
                 assert(ActiveModel != 0 && m != 0);
-            } else {
+            } else
+#endif
+            {
                 if (LCount != 0)
                     suspendLoads = 1;
                 if (MultiFileLoad(cfAppend, argv[Arg],
@@ -889,14 +911,10 @@ int EGUI::CmdLoadFiles(int &argc, char **argv) {
 }
 
 int EGUI::Start(int &argc, char **argv) {
-    {
-        int rc;
+    int rc;
 
-        rc = GUI::Start(argc, argv);
-
-        if (rc)
-            return rc;
-    }
+    if ((rc = GUI::Start(argc, argv)))
+        return rc;
 
     if (InterfaceInit(argc, argv) != 0)
         return 2;
@@ -923,7 +941,7 @@ int EGUI::Start(int &argc, char **argv) {
         assert(ActiveModel != 0 && m != 0);
         ActiveView->SwitchToModel(ActiveModel);
 #else
-        Usage();
+        // FIXME Usage();
         return 1;
 #endif
     }
@@ -933,7 +951,7 @@ int EGUI::Start(int &argc, char **argv) {
 void EGUI::EditorCleanup() {
     if (ActiveModel != NULL) {
         EModel *B, *N, *A;
-        
+
        	B = A = ActiveModel;
         do {
             N = B->Next;
@@ -946,29 +964,18 @@ void EGUI::EditorCleanup() {
     delete SSBuffer;
     SSBuffer = NULL;
 
-    if (ActiveView != NULL) {
-       	EView *BW, *NW;
-
-       	// If EView what is about to be deleted is currently ActiveView, ActiveView moves to next one
-       	// or if there is no next, it will be set as NULL.
-       	while ((BW = ActiveView) != NULL) {
-       		NW = BW->Next;
-       		delete BW;
-       	}
-        //EView *BW, *NW, *AW;
-        //BW = AW = ActiveView;
-        //do {
-        //    NW = BW->Next;
-        //    delete BW;
-        //    BW = NW;
-        //} while (BW != AW);
+#if 0
+    EView *BW;
+    // If EView what is about to be deleted is currently ActiveView, ActiveView moves to next one
+    // or if there is no next, it will be set as NULL.
+    while ((BW = ActiveView) != NULL) {
+	ActiveView = ActiveView->Next;
+	delete BW;
     }
-    ActiveView = NULL;
-}
+#endif
 
-void EGUI::InterfaceCleanup() {
-    while (frames)
-        delete frames;
+    while (frames && frames->Top)
+        delete frames->Top;
 }
 
 void EGUI::Stop() {
@@ -977,58 +984,42 @@ void EGUI::Stop() {
 #endif
 
     // free macros
-    if (Macros!=NULL)
+    if (Macros != 0)
     {
-        int i;
-
         while (CMacros--)
         {
             free(Macros[CMacros].Name);
 
-            for (i=0; i<Macros[CMacros].Count; i++)
-            {
+            for (unsigned i = 0; i < Macros[CMacros].Count; ++i)
                 if (Macros[CMacros].cmds[i].type == CT_STRING)
-                {
                     free(Macros[CMacros].cmds[i].u.string);
-                }
-            }
 
             free(Macros[CMacros].cmds);
         }
 
         free(Macros);
 
-        Macros = NULL;
+        Macros = 0;
     }
 
+#ifdef CONFIG_SYNTAX_HILIT
     // free colorizers
-    {
-        EColorize *p;
-
-        while ((p = Colorizers) != NULL) {
-            Colorizers = Colorizers->Next;
-            delete p;
-        }
+    while (EColorize *p = Colorizers) {
+	Colorizers = Colorizers->Next;
+	delete p;
     }
+#endif
 
     // free event maps
-    {
-        EEventMap *em;
-                  
-        while ((em = EventMaps) != NULL) {
-            EventMaps = EventMaps->Next;
-            delete em;
-        }
+    while (EEventMap *em = EventMaps) {
+	EventMaps = EventMaps->Next;
+	delete em;
     }
 
     // free modes
-    {
-        EMode *m;
-                  
-        while ((m = Modes) != NULL) {
-            Modes = Modes->fNext;
-            delete m;
-        }
+    while (EMode *m = Modes) {
+	Modes = Modes->fNext;
+	delete m;
     }
 
     // free menus
@@ -1057,33 +1048,35 @@ void EGUI::Stop() {
     }
 
     // free completion rexexp filter
-    {
-        extern RxNode *CompletionFilter;
-
-        RxFree(CompletionFilter);
-    }
+    extern RxNode *CompletionFilter;
+    RxFree(CompletionFilter);
 
+#ifdef CONFIG_OBJ_MESSAGES
     // free CRegexp array from o_messages.cpp
-    {
-        FreeCRegexp();
-    }
+    FreeCRegexp();
+#endif
 
 #ifdef CONFIG_OBJ_CVS
     // free CvsIgnoreRegexp array from o_messages.cpp
-    {
-        FreeCvsIgnoreRegexp();
-    }
+    FreeCvsIgnoreRegexp();
+#endif
+
+#ifdef CONFIG_OBJ_SVN
+    // free SvnIgnoreRegexp array from o_messages.cpp
+    FreeSvnIgnoreRegexp();
+#endif
+
+#ifdef CONFIG_TAGS
+    TagClear();
 #endif
 
     // free configuration file path
-    {
-        free(ConfigSourcePath);
-        ConfigSourcePath = NULL;
-    }
+    free(ConfigSourcePath);
+    ConfigSourcePath = NULL;
 
     EditorCleanup();
 
-    InterfaceCleanup();
+    assert(frames == 0);
 
     GUI::Stop();
 }
