@@ -9,8 +9,14 @@
 
 /* some stuff does not yet work */
 
-#ifndef __FEATURE_H__
-#define __FEATURE_H__
+#ifndef FEATURE_H
+#define FEATURE_H
+
+#ifndef FTE_H
+#warning Use header file fte.h
+#endif
+
+#define DEFAULT_INTERNAL_CONFIG
 
 #undef CONFIG_EMULATE_VI // todo
 
@@ -54,12 +60,14 @@
 #define CONFIG_OBJ_BUFFERS
 #define CONFIG_OBJ_MESSAGES
 #define CONFIG_OBJ_CVS
+#define CONFIG_OBJ_SVN
 #endif
 
 #define CONFIG_SYNTAX_HILIT
 #define CONFIG_WORD_HILIT
 
 #ifdef CONFIG_SYNTAX_HILIT
+#define CONFIG_INDENT
 #define CONFIG_HILIT_C
 #define CONFIG_HILIT_REXX
 #define CONFIG_HILIT_PERL
@@ -88,4 +96,24 @@
 #endif
 
 #define CONFIG_I_COMPLETE
-#endif
+
+
+#define CONFIG_USE_LOCALE
+
+/* Internal keyboard remapping in xfte - CZ support */
+//#define CONFIG_HARD_REMAP
+
+/* System X11R6 is compiled with X_LOCALE */
+//#define X_LOCALE
+
+/* Use XtInit */
+#define CONFIG_X11_XTINIT
+/* Drawing fonts with locale support */
+#define CONFIG_X11_XMB
+/* Supply icons in X, requires Xpm library */
+#define CONFIG_X11_XICON
+
+/* Dissable  internal logging mechanism */
+#define FTE_NO_LOGGING
+
+#endif // FEATURE_H
