@@ -1,5 +1,5 @@
-#ifndef __FTEVER_H
-#define __FTEVER_H
+#ifndef FTEVER_H
+#define FTEVER_H
 
 #define MAKE_VERSION(major,minor,release) ((major<<24L) | (minor << 16L) | release)
 
@@ -7,9 +7,9 @@
 #define PROG_CFTE     "cfte"
 #define PROGRAM       PROG_FTE
 #define EXTRA_VERSION "-cvs"
-#define VERSION       "0.49.13" EXTRA_VERSION
-#define VERNUM        MAKE_VERSION(0x00, 0x49, 0x0402)
+#define VERSION       "0.50.02" EXTRA_VERSION
+#define VERNUM        MAKE_VERSION(0x00, 0x50, 0x0002)
 #define COPYRIGHT     "Copyright (c) 1994-1998 Marko Macek\n" \
-                      "Copyright (c) 2000-2003 Others"
+                      "Copyright (c) 2000-2010 Others"
 
-#endif
+#endif // FTEVER_H
