@@ -7,17 +7,8 @@
  *
  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <assert.h>
-#include <signal.h>
-#include <stdarg.h>
-#if defined(MSVC)
-#include <malloc.h>
-#endif
-#include "console.h"
 #include "gui.h"
+#include "sysdep.h"
 
 int MenuCount = 0;
 mMenu *Menus = 0;
@@ -80,4 +71,3 @@ int GetMenuId(const char *Name) {
                 return i;
     return -1;
 }
-
