@@ -1,45 +1,51 @@
 /*    g_motif.cpp
  *
  *    Copyright (c) 1994-1996, Marko Macek
+ *    Copyright (c) 2010, Zdenek Kabelac
  *
  *    You may distribute under the terms of either the GNU General Public
  *    License or the Artistic License, as specified in the README file.
  *
+ *    Only demo code
  */
 
-#include <stdio.h>
-#include <string.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <assert.h>
-#include <stdarg.h>
-#include <fcntl.h>
-#include <sys/wait.h>
+#include "c_config.h"
+#include "console.h"
+#include "gui.h"
+#include "s_string.h"
+
 #include <X11/Xatom.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
+
 #ifdef HPUX
 #include </usr/include/X11R5/X11/HPkeysym.h>
 #endif
+
 #include <Xm/Xm.h>
-#include <Xm/Form.h>
-#include <Xm/PanedW.h>
-#include <Xm/MainW.h>
-#include <Xm/RowColumn.h>
-#include <Xm/MainW.h>
-#include <Xm/PushB.h>
+
 #include <Xm/BulletinB.h>
 #include <Xm/CascadeB.h>
-#include <Xm/Text.h>
-#include <Xm/Separator.h>
+#include <Xm/DrawingA.h>
+#include <Xm/Form.h>
+#include <Xm/Label.h>
+#include <Xm/MainW.h>
+#include <Xm/MainW.h>
+#include <Xm/PanedW.h>
+#include <Xm/PushB.h>
+#include <Xm/RowColumn.h>
 #include <Xm/ScrollBar.h>
 #include <Xm/ScrolledW.h>
-#include <Xm/DrawingA.h>
-#include <Xm/Label.h>
+#include <Xm/Separator.h>
+#include <Xm/Text.h>
 
-#include "console.h"
-#include "gui.h"
+#include <fcntl.h>
+#include <stdarg.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <sys/wait.h>
+#include <unistd.h>
 
 #define DEBUG(x)
 //printf x
@@ -67,7 +73,8 @@ static long MouseAutoDelay = 40;
 static long MouseAutoRepeat = 200;
 static long MouseMultiClick = 300;
 
-static Atom   WM_DELETE_WINDOW;
+static Atom WM_DELETE_WINDOW;
+static Atom XA_CLIPBOARD;
 
 class GViewPeer {
 public:
@@ -86,7 +93,7 @@ public:
     int sbVstart, sbVamount, sbVtotal;
     int sbHstart, sbHamount, sbHtotal;
     int VertPos, HorzPos;
-    unsigned char *ScreenBuffer;
+    char *ScreenBuffer;
 
     GViewPeer(GView *view, int XSize, int YSize);
     ~GViewPeer();
@@ -130,8 +137,8 @@ public:
     GFramePeer(GFrame *frame, int Width, int Height);
     ~GFramePeer();
 
-    int ConSetTitle(char *Title, char *STitle);
-    int ConGetTitle(char *Title, int MaxLen, char *STitle, int SMaxLen);
+    int ConSetTitle(const char *Title, const char *STitle);
+    int ConGetTitle(char *Title, size_t MaxLen, char *STitle, size_t SMaxLen);
 
     int ConSetSize(int X, int Y);
     int ConQuerySize(int *X, int *Y);
@@ -243,8 +250,8 @@ static int InitXColors() {
     return 0;
 }
 
-
-static struct {
+// *INDENT-OFF*
+static const struct {
     long keysym;
     long keycode;
 } key_table[] = {
@@ -312,10 +319,11 @@ static struct {
     { 0x1000FF75,        kbTab | kfShift },
     { 0, 0 }
 };
+// *INDENT-ON*
 
-void ConvertKeyToEvent(KeySym key, KeySym key1, char *keyname, int etype, int state, TEvent *Event) {
+static void ConvertKeyToEvent(KeySym key, KeySym key1, char *keyname, int etype, int state, TEvent *Event) {
     unsigned int myState = 0;
-    int i,k;
+    int k;
 
 
     DEBUG(("key: \n"));
@@ -349,7 +357,7 @@ void ConvertKeyToEvent(KeySym key, KeySym key1, char *keyname, int etype, int st
         Event->Key.Code = key | myState;
         return;
     } else {
-        for (i = 0; i < (sizeof(key_table) / sizeof(key_table[0])); i++) {
+        for (size_t i = 0; i < FTE_ARRAY_SIZE(key_table); ++i) {
             if (key1 == key_table[i].keysym) {
                 k = key_table[i].keycode;
                 if (k < 256)
@@ -368,7 +376,7 @@ static TEvent LastMouseEvent = { evNone };
 
 #define TM_DIFF(x,y) ((long)(((long)(x) < (long)(y)) ? ((long)(y) - (long)(x)) : ((long)(x) - (long)(y))))
 
-void ConvertClickToEvent(int type, int xx, int yy, int button, int state, TEvent *Event, Time time) {
+static void ConvertClickToEvent(int type, int xx, int yy, int button, int state, TEvent *Event, Time time) {
     unsigned int myState = 0;
     static unsigned long LastClickTime = 0;
     static unsigned long LastClickCount = 0;
@@ -440,7 +448,7 @@ void ConvertClickToEvent(int type, int xx, int yy, int button, int state, TEvent
     LastMouseEvent = *Event;
 }
 
-void ProcessXEvents(XEvent *event, TEvent *Event, GViewPeer *Peer) {
+static void ProcessXEvents(XEvent *event, TEvent *Event, GViewPeer *Peer) {
     XAnyEvent *anyEvent = (XAnyEvent *) event;
     XExposeEvent *exposeEvent = (XExposeEvent *) event;
     XButtonEvent *buttonEvent = (XButtonEvent *) event;
@@ -482,7 +490,9 @@ void ProcessXEvents(XEvent *event, TEvent *Event, GViewPeer *Peer) {
     }
 }
 
-static void CloseWindow(Widget w, GFramePeer *frame, XEvent *event, Boolean *cont) {
+static void CloseWindow(Widget w, void *framev, XEvent *event, Boolean *cont)
+{
+    GFramePeer *frame = (GFramePeer*)framev;
     if (event->type != ClientMessage ||
         ((XClientMessageEvent *)event)->data.l[0] != WM_DELETE_WINDOW)
     {
@@ -493,27 +503,36 @@ static void CloseWindow(Widget w, GFramePeer *frame, XEvent *event, Boolean *con
     *cont = False;
 }
 
-void MainCallback (Widget w, mItem *item, XtPointer callData) {
+static void MainCallback (Widget w, void* itemv, XtPointer callData)
+{
+    mItem *item = (mItem*) itemv;
     DEBUG(("Main: %d\n", item->Cmd));
     NextEvent.What = evCommand;
     NextEvent.Msg.Command = item->Cmd;
 }
 
-void PopupCallback (Widget w, mItem *item, XtPointer callData) {
+static void PopupCallback (Widget w, void* itemv, XtPointer callData)
+{
+    mItem *item = (mItem*) itemv;
     DEBUG(("Popup: %d\n", item->Cmd));
     NextEvent.What = evCommand;
     NextEvent.Msg.Command = item->Cmd;
 }
 
-void MenuPopdownCb (Widget w, mItem *item, XtPointer callData) {
+static void MenuPopdownCb (Widget w, void* itemv, XtPointer callData)
+{
+    mItem *item = (mItem*) itemv;
     DEBUG(("Popdown: %d\n", item->Cmd));
     if (LPopupMenu != 0) {
-        XtDestroyWidget(XtParent(LPopupMenu));
+        XtDestroyWidget(LPopupMenu);
         LPopupMenu = 0;
     }
 }
 
-void InputWindow(Widget w, GViewPeer *Peer, XEvent *event, Boolean *cont) {
+static void InputWindow(Widget w, void *PeerV, XEvent *event, Boolean *cont)
+{
+    GViewPeer *Peer = (GViewPeer*)PeerV;
+
     DEBUG(("Input\n"));
     if (!Peer->View->IsActive())
         Peer->View->Parent->SelectView(Peer->View);
@@ -522,7 +541,10 @@ void InputWindow(Widget w, GViewPeer *Peer, XEvent *event, Boolean *cont) {
     *cont = False;
 }
 
-static void VisibilityCb(Widget w, GViewPeer *peer, XEvent *event, Boolean *cont) {
+static void VisibilityCb(Widget w, void* peerv, XEvent *event, Boolean *cont)
+{
+    GViewPeer *peer = (GViewPeer*)peerv;
+
     if (event->type != VisibilityNotify)
         return;
 
@@ -538,19 +560,15 @@ static void VisibilityCb(Widget w, GViewPeer *peer, XEvent *event, Boolean *cont
     *cont = True;
 }
 
-void ConfigureWindow(Widget w, GViewPeer *Peer, XEvent *event, Boolean *cont) {
-    Dimension Xd, Yd;
+static void ConfigureWindow(Widget w, void *PeerV, XEvent *event, Boolean *cont)
+{
+    GViewPeer *Peer = (GViewPeer*)PeerV;
+    XConfigureEvent *confEvent = (XConfigureEvent*)event;
     int X, Y;
-    XConfigureEvent *confEvent = (XConfigureEvent *) event;
 
     DEBUG(("Configure\n"));
-    Xd = confEvent->width;
-    Yd = confEvent->height;
-    X = Xd;
-    Y = Yd;
-    DEBUG(("Resize %d, %d\n", X, Y));
-    X /= cxChar;
-    Y /= cyChar;
+    X = confEvent->width / cxChar;
+    Y = confEvent->height / cyChar;
     DEBUG(("!! Resize %d, %d\n", X, Y));
     if (X > 0 && Y > 0) {
         Peer->ConSetSize(X, Y);
@@ -561,7 +579,10 @@ void ConfigureWindow(Widget w, GViewPeer *Peer, XEvent *event, Boolean *cont) {
     *cont = True;
 }
 
-void ExposeWindow(Widget w, GViewPeer *Peer, XmDrawingAreaCallbackStruct *Call) {
+static void ExposeWindow(Widget w, void* PeerV, void* CallV)
+{
+    GViewPeer *Peer = (GViewPeer*)PeerV;
+    XmDrawingAreaCallbackStruct *Call = (XmDrawingAreaCallbackStruct*)CallV;
     XExposeEvent *exposeEvent = (XExposeEvent *) Call->event;
 
     DEBUG(("Expose\n"));
@@ -574,7 +595,11 @@ void ExposeWindow(Widget w, GViewPeer *Peer, XmDrawingAreaCallbackStruct *Call)
 }
 
 
-void VertValueChanged(Widget w, GViewPeer *Peer, XmScrollBarCallbackStruct *Call) {
+static void VertValueChanged(Widget w, void* PeerV, void* CallV)
+{
+    GViewPeer *Peer = (GViewPeer*)PeerV;
+    XmScrollBarCallbackStruct *Call = (XmScrollBarCallbackStruct*)CallV;
+
     if (!Peer->View->IsActive())
         Peer->View->Parent->SelectView(Peer->View);
     if (Peer->VertPos != Call->value) {
@@ -587,7 +612,12 @@ void VertValueChanged(Widget w, GViewPeer *Peer, XmScrollBarCallbackStruct *Call
     }
 }
 
-void HorzValueChanged(Widget w, GViewPeer *Peer, XmScrollBarCallbackStruct *Call) {
+//void HorzValueChanged(Widget w, GViewPeer *Peer, XmScrollBarCallbackStruct *Call)
+void HorzValueChanged(Widget w, void* PeerV, void* CallV)
+{
+    GViewPeer *Peer = (GViewPeer*)PeerV;
+    XmScrollBarCallbackStruct *Call = (XmScrollBarCallbackStruct*) CallV;
+
     if (!Peer->View->IsActive())
         Peer->View->Parent->SelectView(Peer->View);
     if (Call->value != Peer->HorzPos) {
@@ -602,40 +632,43 @@ void HorzValueChanged(Widget w, GViewPeer *Peer, XmScrollBarCallbackStruct *Call
 
 ///////////////////////////////////////////////////////////////////////////
 
-GViewPeer::GViewPeer(GView *view, int XSize, int YSize) {
-
-    View = view;
-    Visibility = VisibilityFullyObscured;
-
-    //    wX = 0;
-    //    wY = 0;
-    wW = XSize;
-    wH = YSize;
-    sbVtotal = 0;
-    sbVstart = 0;
-    sbVamount = 0;
-    sbHtotal = 0;
-    sbHstart = 0;
-    sbHamount = 0;
-    wState = 0;
-    cVisible = 1;
-    cStart = 0; // %
-    cEnd = 100;
-    wRefresh = 0;
-    ScreenBuffer = 0;
-    cX = -1;
-    cY = -1;
-    VertPos = HorzPos = -1;
-
+GViewPeer::GViewPeer(GView *view, int XSize, int YSize) :
+    Visibility(VisibilityFullyObscured),
+    View(view),
+    //wX(0),
+    //wY(0),
+    //wW(XSize),
+    //wH(YSize),
+    wW(80),
+    wH(40),
+    wState(0),
+    wRefresh(0),
+    cX(-1),
+    cY(-1),
+    cVisible(1),
+    cStart(0), // %
+    cEnd(100),
+    sbVstart(0),
+    sbVamount(0),
+    sbVtotal(0),
+    sbHstart(0),
+    sbHamount(0),
+    sbHtotal(0),
+    VertPos(-1),
+    HorzPos(-1),
+    ScreenBuffer(0)
+{
     for (int jj = 0; jj < 256; jj++)
         gc[jj] = 0;
 
+    AllocBuffer();
+
     ScrollWin = XtVaCreateWidget("ScrollWin",
                                  xmScrolledWindowWidgetClass, frames->Peer->PanedWin,
                                  XmNmarginHeight, 0,
                                  XmNmarginWidth, 0,
                                  XmNspacing, 0,
-                                 0);
+                                 NULL);
 
     TextWin = XtVaCreateManagedWidget ("TextWin",
                                        xmDrawingAreaWidgetClass, ScrollWin,
@@ -647,20 +680,20 @@ GViewPeer::GViewPeer(GView *view, int XSize, int YSize) {
                                        XmNheightInc, cyChar,
                                        XmNwidth, cxChar * 80,
                                        XmNheight, cyChar * 30,
-                                       0);
+                                       NULL);
 
-    /*    XSetWindowColormap(display, XtWindow(TextWin), colormap);*/
+    // XSetWindowColormap(display, XtWindow(TextWin), colormap);
 
     XtVaSetValues (ScrollWin,
                    XmNworkWindow, TextWin,
-                   0);
+                   NULL);
 
     SbVert = XtVaCreateManagedWidget("VScrollBar",
                                      xmScrollBarWidgetClass, ScrollWin,
                                      XmNmarginHeight, 0,
                                      XmNmarginWidth, 0,
                                      XmNwidth, 20,
-                                     0);
+                                     NULL);
 
     SbHorz = XtVaCreateManagedWidget("HScrollBar",
                                      xmScrollBarWidgetClass, ScrollWin,
@@ -672,7 +705,7 @@ GViewPeer::GViewPeer(GView *view, int XSize, int YSize) {
     XtVaSetValues(ScrollWin,
                   XmNhorizontalScrollBar, SbHorz,
                   XmNverticalScrollBar, SbVert,
-                  0);
+                  NULL);
 
     XtManageChild(ScrollWin);
 
@@ -717,18 +750,20 @@ GViewPeer::GViewPeer(GView *view, int XSize, int YSize) {
     XtAddCallback(SbVert, XmNpageDecrementCallback, VertValueChanged, this);
     XtAddCallback(SbVert, XmNtoTopCallback, VertValueChanged, this);
     XtAddCallback(SbVert, XmNtoBottomCallback, VertValueChanged, this);
-
 }
 
 GViewPeer::~GViewPeer() {
-    XtDestroyWidget(ScrollWin);
+    // Widget destroyed recursively
+    // from master shell widget
+    free(ScreenBuffer);
+    ScreenBuffer = 0;
 }
 
 int GViewPeer::AllocBuffer() {
     int i;
-    unsigned char *p;
-
-    ScreenBuffer = (unsigned char *)malloc(2 * wW * wH);
+    char *p;
+    /* FIXME */
+    ScreenBuffer = (char *)malloc(wW * wH * sizeof(TCell));
     if (ScreenBuffer == NULL) return -1;
     for (i = 0, p = ScreenBuffer; i < wW * wH; i++) {
         *p++ = 32;
@@ -760,7 +795,7 @@ void GViewPeer::DrawCursor(int Show) {
     //    if (!XtIsManaged(TextWin)) return ;
 
     if (cVisible && cX >= 0 && cY >= 0) {
-        unsigned char *p = CursorXYPos(cX, cY), attr;
+        char *p = CursorXYPos(cX, cY), attr;
         attr = p[1];
         /*if (Show) attr = ((((attr << 4) & 0xF0)) | (attr >> 4)) ^ 0x77;*/
         if (Show) attr = (attr ^ 0x77);
@@ -775,14 +810,14 @@ void GViewPeer::DrawCursor(int Show) {
         XDrawImageString(display, XtWindow(TextWin), gc[attr],
                          cX * cxChar,
                          fontStruct->max_bounds.ascent + cY * cyChar,
-                         p, 1);
+                         (const char*)p, 1);
     }
 }
 
 int GViewPeer::ConPutBox(int X, int Y, int W, int H, PCell Cell) {
     int i;
-    unsigned char temp[256], attr;
-    unsigned char *p, *ps, *c, *ops;
+    char temp[256], attr;
+    char *p, *ps, *c, *ops;
     int len, x, l, ox, olen, skip;
 
     if (!(View && View->Parent && gc))
@@ -798,12 +833,11 @@ int GViewPeer::ConPutBox(int X, int Y, int W, int H, PCell Cell) {
         return -1;
     }
 
-
     DEBUG(("PutBox %d | %d %d %d %d | %d %d\n", wRefresh, X, Y, W, H, wW, wH));
     for (i = 0; i < H; i++) {
         len = W;
         p = CursorXYPos(X, Y + i);
-        ps = (unsigned char *) Cell;
+        ps = (char *) Cell;
         x = X;
         while (len > 0) {
             if (!wRefresh) {
@@ -845,7 +879,7 @@ int GViewPeer::ConPutBox(int X, int Y, int W, int H, PCell Cell) {
             len -= l;
         }
         p = CursorXYPos(X, Y + i);
-        memcpy(p, Cell, W * 2);
+        memmove(p, Cell, W * sizeof(TCell));
         if (i + Y == cY)
             DrawCursor(1);
         Cell += W;
@@ -939,14 +973,14 @@ int GViewPeer::ConScroll(int Way, int X, int Y, int W, int H, TAttr Fill, int Co
 }
 
 int GViewPeer::ConSetSize(int X, int Y) {
-    unsigned char *NewBuffer;
-    unsigned char *p;
+    char *NewBuffer;
+    char *p;
     int i;
-    unsigned int MX, MY;
+    int MX, MY;
 
-    p = NewBuffer = (unsigned char *) malloc(X * Y * 2);
+    p = NewBuffer = (char *) malloc(X * Y * sizeof(TCell));
     if (NewBuffer == NULL) return -1;
-    for (i = 0; i < X * Y; i++) {
+    for (int i = 0; i < X * Y; i++) {
         *p++ = ' ';
         *p++ = 0x07;
     }
@@ -956,7 +990,7 @@ int GViewPeer::ConSetSize(int X, int Y) {
         if (X < MX) MX = X;
         p = NewBuffer;
         for (i = 0; i < MY; i++) {
-            memcpy(p, CursorXYPos(0, i), MX * 2);
+            memcpy(p, CursorXYPos(0, i), MX * sizeof(TCell));
             p += X * 2;
         }
         free(ScreenBuffer);
@@ -974,6 +1008,7 @@ int GViewPeer::ConSetSize(int X, int Y) {
 }
 
 int GViewPeer::ConQuerySize(int *X, int *Y) {
+    //printf("3CONQUERYSIZE  %d  %d\n", wW, wH);
     if (X) *X = wW;
     if (Y) *Y = wH;
     return 1;
@@ -981,9 +1016,9 @@ int GViewPeer::ConQuerySize(int *X, int *Y) {
 
 int GViewPeer::ConSetCursorPos(int X, int Y) {
     if (X < 0) X = 0;
-    if (X >= wW) X = wW - 1;
+    else if (X >= wW) X = wW - 1;
     if (Y < 0) Y = 0;
-    if (Y >= wH) Y = wH - 1;
+    else if (Y >= wH) Y = wH - 1;
     DrawCursor(0);
     cX = X;
     cY = Y;
@@ -1048,7 +1083,7 @@ int GViewPeer::SetSbVPos(int Start, int Amount, int Total) {
                           XmNpageIncrement, 1,
                           XmNsliderSize, 1,
                           XmNvalue, 0,
-                          0);
+                          NULL);
         } else {
             XtVaSetValues(SbVert,
                           XmNmaximum, Total,
@@ -1056,7 +1091,7 @@ int GViewPeer::SetSbVPos(int Start, int Amount, int Total) {
                           XmNpageIncrement, ((Amount > 1) ? Amount : 1),
                           XmNsliderSize, Amount,
                           XmNvalue, Start,
-                          0);
+                          NULL);
         }
     }
     return 1;
@@ -1081,7 +1116,7 @@ int GViewPeer::SetSbHPos(int Start, int Amount, int Total) {
                           XmNpageIncrement, 1,
                           XmNsliderSize, 1,
                           XmNvalue, 0,
-                          0);
+                          NULL);
         } else {
             XtVaSetValues(SbHorz,
                           XmNmaximum, Total,
@@ -1089,7 +1124,7 @@ int GViewPeer::SetSbHPos(int Start, int Amount, int Total) {
                           XmNpageIncrement, ((Amount > 1) ? Amount : 1),
                           XmNsliderSize, Amount,
                           XmNvalue, Start,
-                          0);
+                          NULL);
         }
     }
     return 1;
@@ -1131,10 +1166,12 @@ int GViewPeer::PMSetCursorPos() {
 
 ///////////////////////////////////////////////////////////////////////////
 
-GView::GView(GFrame *parent, int XSize, int YSize) {
-    Parent = parent;
-    Prev = Next = 0;
-    Peer = new GViewPeer(this, XSize, YSize);
+GView::GView(GFrame *parent, int XSize, int YSize) :
+    Parent(parent),
+    Next(0),
+    Prev(0),
+    Peer(new GViewPeer(this, XSize, YSize))
+{
     if (Parent)
         Parent->AddView(this);
 }
@@ -1142,8 +1179,7 @@ GView::GView(GFrame *parent, int XSize, int YSize) {
 GView::~GView() {
     if (Parent)
         Parent->RemoveView(this);
-    if (Peer)
-        delete Peer;
+    delete Peer;
 }
 
 int GView::ConClear() {
@@ -1177,6 +1213,7 @@ int GView::ConScroll(int Way, int X, int Y, int W, int H, TAttr Fill, int Count)
 }
 
 int GView::ConSetSize(int X, int Y) {
+    printf("ConSetSize %d  %d\n", X, Y);
     if (Peer->ConSetSize(X, Y)) ;
     //        Resize(X, Y);
     else
@@ -1288,7 +1325,9 @@ int GView::CaptureMouse(int grab) {
 
 ///////////////////////////////////////////////////////////////////////////
 
-GFramePeer::GFramePeer(GFrame *frame, int Width, int Height) {
+GFramePeer::GFramePeer(GFrame *frame, int Width, int Height) :
+    MenuBar(0)
+{
     ShellWin = XtCreatePopupShell("fte", topLevelShellWidgetClass,
                                   TopLevel, NULL, 0);
 
@@ -1304,43 +1343,46 @@ GFramePeer::GFramePeer(GFrame *frame, int Width, int Height) {
                                        xmPanedWindowWidgetClass, MainWin,
                                        XmNmarginHeight, 0,
                                        XmNmarginWidth, 0,
-                                       0 );
+                                       NULL);
 
     XtVaSetValues (MainWin,
                    XmNworkWindow, PanedWin,
-                   0);
+                   NULL);
 
     if (Width != -1 && Height != -1)
         ConSetSize(Width, Height);
 }
 
 GFramePeer::~GFramePeer() {
+    // Kill master window - deletes recursively all related widgets
+    XtDestroyWidget(MenuBar);
+    XtDestroyWidget(ShellWin);
 }
 
 int GFramePeer::ConSetSize(int X, int Y) {
+    printf("SetSIZE %d  %d\n", X, Y);
     //return ::ConSetSize(X, Y);
     return 0;
 }
 
 int GFramePeer::ConQuerySize(int *X, int *Y) {
-    //    ::ConQuerySize(&fW, &fH);
-    //    if (X) *X = fW;
-    //    if (Y) *Y = fH;
+    //::ConQuerySize(&fW, &fH);
+    //if (X) *X = fW;
+    //if (Y) *Y = fH;
+    if (X) *X = 80;
+    if (Y) *Y = 25;
+
+    printf("1CONQUERYSIZE %d  %d\n", *X, *Y);
     return 1;
 }
 
-//int GFrame::ConQuerySize(int *X, int *Y) {
-//    ::ConQuerySize(X, Y);
-//    if (ShowVScroll)
-//        --*X;
-//}
 
-int GFramePeer::ConSetTitle(char *Title, char *STitle) {
+int GFramePeer::ConSetTitle(const char *Title, const char *STitle) {
     XSetStandardProperties(display, XtWindow(ShellWin), Title, STitle, 0, NULL, 0, NULL);
     return 1;
 }
 
-int GFramePeer::ConGetTitle(char *Title, int MaxLen, char *STitle, int SMaxLen) {
+int GFramePeer::ConGetTitle(char *Title, size_t MaxLen, char *STitle, size_t SMaxLen) {
     *Title = 0;
     *STitle = 0;
     return 1;
@@ -1371,10 +1413,9 @@ GFrame::GFrame(int XSize, int YSize) {
 }
 
 GFrame::~GFrame() {
-    if (Peer) {
-        delete Peer;
-        Peer = 0;
-    }
+    delete Peer;
+    free(Menu);
+
     if (Next == this) {
         frames = 0;
         //        DEBUG(("No more frames\x7\x7\n"));
@@ -1383,14 +1424,13 @@ GFrame::~GFrame() {
         Prev->Next = Next;
         frames = Next;
     }
-    Next = Prev = 0;
 }
 
-int GFrame::ConSetTitle(char *Title, char *STitle) {
+int GFrame::ConSetTitle(const char *Title, const char *STitle) {
     return Peer->ConSetTitle(Title, STitle);
 }
 
-int GFrame::ConGetTitle(char *Title, int MaxLen, char *STitle, int SMaxLen) {
+int GFrame::ConGetTitle(char *Title, size_t MaxLen, char *STitle, size_t SMaxLen) {
     return Peer->ConGetTitle(Title, MaxLen, STitle, SMaxLen);
 }
 
@@ -1426,31 +1466,28 @@ int GFrame::ConResizeView(GView *view, int DeltaY) {
 }
 
 int GFrame::AddView(GView *view) {
-    if (Top != 0) {
+    if (Top != 0)
         return ConSplitView(Top, view);
-    } else {
-        //        int W, H;
 
-        view->Parent = this;
-        view->Prev = view->Next = 0;
+    //        int W, H;
 
-        //        view->Peer->wX = 0;
-        //        view->Peer->wY = 0;
-        //        ConQuerySize(&W, &H);
-        //        view->ConSetSize(W, H);
-        InsertView(Top, view);
-        return 0;
-    }
+    view->Parent = this;
+    view->Prev = view->Next = 0;
+
+    //        view->Peer->wX = 0;
+    //        view->Peer->wY = 0;
+    //        ConQuerySize(&W, &H);
+    //        view->ConSetSize(W, H);
+    InsertView(Top, view);
+    return 0;
 }
 
 void GFrame::Update() {
-    GView *v = Active;
 
     UpdateMenu();
-    while (v) {
+    for (GView *v = Active; v; v = v->Next) {
         v->Update();
-        v = v->Next;
-        if (v == Active)
+        if (v->Next == Active)
             break;
     }
 }
@@ -1459,12 +1496,9 @@ void GFrame::UpdateMenu() {
 }
 
 void GFrame::Repaint() {
-    GView *v = Active;
-
-    while (v) {
+    for (GView* v = Active; v; v = v->Next) {
         v->Repaint();
-        v = v->Next;
-        if (v == Active)
+        if (v->Next == Active)
             break;
     }
 }
@@ -1559,17 +1593,12 @@ void GFrame::Resize(int width, int height) {
     if (width < 8 || height < 2)
         return;
 
-    if (Top == Top->Next) {
+    if (Top == Top->Next)
         Top->ConSetSize(width, height);
-    } else {
-    }
 }
 
-Widget CreateMotifMenu(Widget parent, int menu, int main, XtCallbackProc MenuProc) {
+static Widget CreateMotifMenu(Widget parent, int menu, int main, XtCallbackProc MenuProc) {
     Widget hmenu;
-    int i;
-    char s[256];
-    char *p;
     Widget item;
 
     if (main == 1) {
@@ -1587,22 +1616,24 @@ Widget CreateMotifMenu(Widget parent, int menu, int main, XtCallbackProc MenuPro
                                      NULL, 0);
     }
 
-    for (i = 0; i < Menus[menu].Count; i++) {
+    for (int i = 0; i < Menus[menu].Count; ++i) {
         if (Menus[menu].Items[i].Name) {
+            char s[256];
             char *mn = 0;
 
             strcpy(s, Menus[menu].Items[i].Name);
-            p = strchr(s, '&');
+            char* p = strchr(s, '&');
             if (p) {
-                strcpy(p, p + 1);
                 mn = p;
+                // for strcpy src & dest may not overlap!
+                for (; p[0]; ++p)
+                    p[0] = p[1];
             }
             p = strchr(s, '\t');
             if (p) {
                 *p = 0;
                 p++;
             }
-
             if (Menus[menu].Items[i].SubMenu != -1) {
                 item = XtVaCreateManagedWidget(s,
                                                xmCascadeButtonWidgetClass,
@@ -1611,11 +1642,12 @@ Widget CreateMotifMenu(Widget parent, int menu, int main, XtCallbackProc MenuPro
                                                CreateMotifMenu(hmenu,
                                                                Menus[menu].Items[i].SubMenu,
                                                                0, MenuProc),
-                                               NULL );
+                                               NULL);
             } else {
                 item = XtVaCreateManagedWidget(s,
-                                               xmPushButtonWidgetClass, hmenu,
-                                               0);
+                                               xmPushButtonWidgetClass,
+                                               hmenu,
+                                               NULL);
                 XtAddCallback(item, XmNactivateCallback,
                               MenuProc, &(Menus[menu].Items[i]));
             }
@@ -1624,17 +1656,17 @@ Widget CreateMotifMenu(Widget parent, int menu, int main, XtCallbackProc MenuPro
                 XtVaSetValues(item,
                               XmNacceleratorText,
                               XmStringCreate(p, XmSTRING_DEFAULT_CHARSET),
-                              0);
+                              NULL);
             if (mn)
                 XtVaSetValues(item,
                               XmNmnemonic,
                               KeySym(*mn),
-                              0);
+                              NULL);
         } else {
             item = XtVaCreateManagedWidget("separator",
                                            xmSeparatorWidgetClass,
                                            hmenu,
-                                           0);
+                                           NULL);
             //XmCreateSeparator(parent, "xx", 0, 0);
         }
         //        item.id = Menus[menu].Items[i].Cmd & 0xFFFF; // ?
@@ -1642,22 +1674,24 @@ Widget CreateMotifMenu(Widget parent, int menu, int main, XtCallbackProc MenuPro
     return hmenu;
 }
 
-Widget CreateMotifMainMenu(Widget parent, char *Name) {
+static Widget CreateMotifMainMenu(Widget parent, char *Name) {
     int id = GetMenuId(Name);
 
     return CreateMotifMenu(parent, id, 1, MainCallback);
 }
 
 int GFrame::SetMenu(const char *Name) {
-    if (Menu) free(Menu);
+    free(Menu);
     Menu = strdup(Name);
 
-    Peer->MenuBar = CreateMotifMainMenu(Peer->MainWin, Name);
+    if (Peer->MenuBar)
+        XtDestroyWidget(Peer->MenuBar);
+
+    Peer->MenuBar = CreateMotifMainMenu(Peer->MainWin, Menu);
     XtManageChild (Peer->MenuBar);
     XtVaSetValues (Peer->MainWin,
                    XmNmenuBar, Peer->MenuBar,
-                   0);
-
+                   NULL);
     return 1;
 }
 
@@ -1668,18 +1702,37 @@ int GFrame::ExecMainMenu(char Sub) {
 int GFrame::PopupMenu(const char *Name) {
     int id = GetMenuId(Name);
 
-    LPopupMenu = CreateMotifMenu(Peer->MainWin, id, 2, (XtCallbackProc)PopupCallback);
+    if (LPopupMenu)
+        XtDestroyWidget(LPopupMenu);
+
+    LPopupMenu = CreateMotifMenu(Peer->MainWin, id, 2, PopupCallback);
     XtAddCallback(XtParent(LPopupMenu), XmNpopdownCallback, MenuPopdownCb, 0);
-    XmMenuPosition (LPopupMenu, (XButtonEvent *)&LastRelease);
-    XtManageChild (LPopupMenu);
+    XmMenuPosition(LPopupMenu, &LastRelease);
+    XtManageChild(LPopupMenu);
     return 1;
 }
 
+void GFrame::Show() {
+    Update();
+    Peer->MapFrame();
+}
+
+void GFrame::Activate() {
+    frames = this;
+    Update();
+    //Peer->ShowFrame();
+}
+
 // GUI
 
+int GUI::multiFrame() {
+    return 1;
+}
 
-GUI::GUI(int &argc, char **argv, int XSize, int YSize) {
-
+GUI::GUI(int &argc, char **argv, int XSize, int YSize) :
+    fArgc(argc),
+    fArgv(argv)
+{
     char *fs = getenv("VIOFONT");
     if (fs == 0 && WindowFont[0] != 0)
         fs = WindowFont;
@@ -1688,15 +1741,13 @@ GUI::GUI(int &argc, char **argv, int XSize, int YSize) {
                                  &argc, argv, NULL,
                                  XmNmappedWhenManaged, FALSE,
                                  NULL);
-
-    if (TopLevel == 0)
-        return ;
-
-    display = XtDisplay(TopLevel);
-    //XSynchronize(display, True);
-    if (display == NULL)
+    if (!TopLevel)
         return;
 
+    if (!(display = XtDisplay(TopLevel)))
+        return;
+    //XSynchronize(display, True);
+
     root = DefaultRootWindow(display);
     screen = DefaultScreen(display);
     colormap = DefaultColormap(display, screen);
@@ -1729,11 +1780,14 @@ GUI::GUI(int &argc, char **argv, int XSize, int YSize) {
     XtRealizeWidget(TopLevel);
 
     WM_DELETE_WINDOW = XInternAtom(display, "WM_DELETE_WINDOW", False);
+    XA_CLIPBOARD = XInternAtom(display, "CLIPBOARD", False);
 
     gui = this;
 }
 
 GUI::~GUI() {
+    //core: XtCloseDisplay(display);
+    //core: XtDestroyApplicationContext(AppContext);
     gui = 0;
 }
 
@@ -1748,9 +1802,9 @@ void GUI::DispatchEvent(GFrame *frame, GView *view, TEvent &Event) {
     }
 }
 
-int GUI::ConSuspend(void) { return 0; }
+int GUI::ConSuspend() { return 0; }
 
-int GUI::ConContinue(void) { return 0; }
+int GUI::ConContinue() { return 0; }
 
 int GUI::ConGetEvent(TEventMask EventMask, TEvent *Event, int WaitTime, int Delete, GView **view) {
     //return ::ConGetEvent(EventMask, Event, WaitTime, Delete, view);
@@ -1758,12 +1812,12 @@ int GUI::ConGetEvent(TEventMask EventMask, TEvent *Event, int WaitTime, int Dele
     return 0;
 }
 
-int GUI::ConPutEvent(TEvent Event) {
+int GUI::ConPutEvent(const TEvent& Event) {
     EventBuf = Event;
     return 0;
 }
 
-int GUI::ConFlush(void) {
+int GUI::ConFlush() {
     return 0;
 }
 
@@ -1783,11 +1837,20 @@ void GUI::ProcessEvent() {
 }
 
 int GUI::Run() {
-    frames->Peer->MapFrame();
+    if (Start(fArgc, fArgv) == 0) {
+        doLoop = 1;
+        frames->Show();
 
-    //XtAppMainLoop(AppContext);
-    while (frames != 0)
-        ProcessEvent();
+	//if (frames && frames->Peer)
+	//    frames->Peer->MapFrame();
+
+	//XtAppMainLoop(AppContext);
+	while (doLoop)
+	    ProcessEvent();
+
+	Stop();
+        return 0;
+    }
 
     return 0;
 }
@@ -1799,27 +1862,24 @@ int GUI::ShowEntryScreen() {
 int GUI::RunProgram(int mode, char *Command) {
     char Cmd[1024];
 
-    char* xterm = getenv("TERM");
-    if (NULL == xterm || 0 == *xterm)
-        xterm = "xterm";
-
-    strcpy(Cmd, xterm);
+    strlcpy(Cmd, XShellCommand, sizeof(Cmd));
 
     if (*Command == 0)  // empty string = shell
-        strcat(Cmd, " -ls &");
+        strlcat(Cmd, " -ls &", sizeof(Cmd));
     else {
-        strcat(Cmd, " -e ");
-        // buffer overflow problem: -2 for possible async.
-	strncat(Cmd, Command, sizeof(Cmd) - strlen(Cmd) - 2);
-        Cmd[sizeof(Cmd) - 3] = 0;
+        strlcat(Cmd, " -e ", sizeof(Cmd));
+	strlcat(Cmd, Command, sizeof(Cmd));
         if (mode == RUN_ASYNC)
-            strcat(Cmd, " &");
+            strlcat(Cmd, " &", sizeof(Cmd));
     }
-    rc = system(Cmd);
-    return rc;
+
+    return system(Cmd);
 }
 
-void PipeCallback(GPipe *pipe, int *source, XtInputId *input) {
+//void PipeCallback(GPipe *pipe, int *source, XtInputId *input)
+void PipeCallback(void *pipev, int *source, XtInputId *input)
+{
+    GPipe *pipe = (GPipe*)pipev;
     if (pipe && pipe->notify && *source == pipe->fd) {
         NextEvent.What = evNotify;
         NextEvent.Msg.View = frames->Active;
@@ -1831,7 +1891,7 @@ void PipeCallback(GPipe *pipe, int *source, XtInputId *input) {
     //fprintf(stderr, "Pipe %d\n", *source);
 }
 
-int GUI::OpenPipe(char *Command, EModel *notify) {
+int GUI::OpenPipe(const char *Command, EModel *notify) {
     int i;
 
     for (i = 0; i < MAX_PIPES; i++) {
@@ -1859,8 +1919,8 @@ int GUI::OpenPipe(char *Command, EModel *notify) {
                 fcntl(pfd[0], F_SETFL, O_NONBLOCK);
                 Pipes[i].fd = pfd[0];
             }
-            Pipes[i].input =
-                XtAppAddInput(AppContext, Pipes[i].fd, XtInputReadMask, PipeCallback, &Pipes[i]);
+            Pipes[i].input =                                 /* FIXME: */
+                XtAppAddInput(AppContext, Pipes[i].fd, (void*)XtInputReadMask, PipeCallback, &Pipes[i]);
             Pipes[i].used = 1;
             //fprintf(stderr, "Pipe Open: %d\n", i);
             return i;
@@ -1876,21 +1936,22 @@ int GUI::SetPipeView(int id, EModel *notify) {
         return -1;
     //fprintf(stderr, "Pipe View: %d %08X\n", id, notify);
     Pipes[id].notify = notify;
-    if (notify != Pipes[id].notify)
+    if (notify != Pipes[id].notify) {
         if (notify) {
-            Pipes[id].input =
-                XtAppAddInput(AppContext, Pipes[id].fd, XtInputReadMask, PipeCallback, &Pipes[id]);
+            Pipes[id].input =                               /* FIXME */
+                XtAppAddInput(AppContext, Pipes[id].fd, (void*)XtInputReadMask, PipeCallback, &Pipes[id]);
         } else {
             if (Pipes[id].input != 0) {
                 XtRemoveInput(Pipes[id].input);
                 Pipes[id].input = 0;
             }
-        }
+	}
+    }
     return 0;
 }
 
-int GUI::ReadPipe(int id, void *buffer, int len) {
-    int rc;
+ssize_t GUI::ReadPipe(int id, void *buffer, size_t len) {
+    ssize_t rc;
 
     if (id < 0 || id > MAX_PIPES)
         return -1;
@@ -1928,7 +1989,8 @@ int GUI::ClosePipe(int id) {
     return WEXITSTATUS(status);
 }
 
-int GetXSelection(int *len, char **data) {
+int GetXSelection(int *len, char **data, int clipboard) {
+    // XXX use clipboard?
     *data = XFetchBytes(display, len);
     if (*data == 0)
         return -1;
@@ -1936,9 +1998,24 @@ int GetXSelection(int *len, char **data) {
         return 0;
 }
 
-int SetXSelection(int len, char *data) {
+int SetXSelection(int len, char *data, int clipboard) {
+    Atom clip;
     XStoreBytes(display, data, len);
-    XSetSelectionOwner (display, XA_PRIMARY, None, CurrentTime);
+    switch (clipboard) {
+    case 0:
+        clip =  XA_CLIPBOARD;
+        break;
+    case 1:
+        clip = XA_PRIMARY;
+        break;
+    case 2:
+        clip = XA_SECONDARY;
+        break;
+    default:
+        // not supported
+        return -1;
+    }
+    XSetSelectionOwner(display, clip, None, CurrentTime);
     return 1;
 }
 
@@ -1994,10 +2071,10 @@ void DieError(int rc, const char *msg, ...) {
     exit(rc);
 }
 
-char ConGetDrawChar(int index) {
-    static char tab[] = "\x0D\x0C\x0E\x0B\x12\x19____+>\x1F\x01\x12 ";
+char ConGetDrawChar(unsigned int index) {
+    static const char tab[] = "\x0D\x0C\x0E\x0B\x12\x19____+>\x1F\x01\x12\x01\x01 \x02";
 
-    assert(index >= 0 && index < strlen(tab));
+    assert(index < sizeof(tab) - 1);
 
     return tab[index];
 }
