@@ -1,44 +1,40 @@
-/*    g_motif.cpp
+/*    g_qt.cpp
  *
  *    Copyright (c) 1994-1996, Marko Macek
+ *    Copyright (c) 2010, Zdenek Kabelac
  *
  *    You may distribute under the terms of either the GNU General Public
  *    License or the Artistic License, as specified in the README file.
  *
+ *    Only a demo code - not really fully functional
  */
 
-#include <stdio.h>
-#include <string.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <assert.h>
-#include <stdarg.h>
-#include <fcntl.h>
-#include <sys/wait.h>
-
-#include "sysdep.h"
-#include "console.h"
 #include "gui.h"
+#include "s_string.h"
+#include "sysdep.h"
 
-
-#include <qwidget.h>
-#include <qframe.h>
 #include <qapp.h>
+#include <qclipbrd.h>
+#include <qframe.h>
+#include <qkeycode.h>
 #include <qmenubar.h>
+#include <qpainter.h>
 #include <qscrbar.h>
 #include <qtimer.h>
-#include <qpainter.h>
-#include <qkeycode.h>
-#include <qclipbrd.h>
-//#include <qfiledlg.h> // conflicts with <Xlib.h> above
+#include <qwidget.h>
 
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 
-#include <X11/Xlib.h>
-#include <X11/Xutil.h>
+#include <fcntl.h>
+#include <stdarg.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <sys/wait.h>
+#include <unistd.h>
 
-#define DEBUGX(x) //printf x
+#define DEBUGX(x)
+//#define DEBUGX(x) printf x
 
 #define EDIT_BORDER 2
 #define SCROLLBAR_SIZE 16
@@ -64,14 +60,14 @@ static GPipe Pipes[MAX_PIPES] = {
 class QEText: public QWidget {
     Q_OBJECT
 public:
-    GViewPeer   *view;
+    GViewPeer *view;
     
     QEText(GViewPeer *peer, QWidget *parent = 0, const char *name = 0);
     virtual ~QEText();
 
-    virtual void handleKeyPressEvent(QKeyEvent *qe);
-    void ActiveEvent(TEvent &Event);
 protected:
+    void handleKeyPressEvent(QKeyEvent *qe);
+    void ActiveEvent(TEvent &Event);
     void handleMouse(QMouseEvent *qe);
         
     virtual void resizeEvent(QResizeEvent *qe);
@@ -180,7 +176,7 @@ public:
     int PMShowCursor();
     int PMHideCursor();
     int PMSetCursorPos();
-};      
+};
 
 class GFramePeer {
 public:
@@ -190,8 +186,8 @@ public:
     GFramePeer(GFrame *aFrame, int Width, int Height);
     ~GFramePeer();
     
-    int ConSetTitle(char *Title, char *STitle);
-    int ConGetTitle(char *Title, int MaxLen, char *STitle, int SMaxLen);
+    int ConSetTitle(const char *Title, const char *STitle);
+    int ConGetTitle(char *Title, size_t MaxLen, char *STitle, size_t SMaxLen);
     
     int ConSetSize(int X, int Y);
     int ConQuerySize(int *X, int *Y);
@@ -212,35 +208,35 @@ static GView *FocusCapture = 0;
 
 static int cxChar = 1;
 static int cyChar = 1;
-static int fmAscent;
+//static int fmAscent;
 static int FinalExit = 0;
 
-TEvent EventBuf = { evNone };
+static TEvent EventBuf = { evNone };
 
-typedef struct _qEvent {
+struct qEvent {
     TEvent event;
-    struct _qEvent *next;
-} qEvent;
+    struct qEvent *next;
+};
 
 TEvent NextEvent = { evNone };
-static QColor colors[16] = {
-    black,
-    darkBlue,
-    darkGreen,
-    darkCyan,
-    darkRed,
-    darkMagenta,
-    darkYellow,
-    gray,
+static const QColor colors[16] = {
+    Qt::black,
+    Qt::darkBlue,
+    Qt::darkGreen,
+    Qt::darkCyan,
+    Qt::darkRed,
+    Qt::darkMagenta,
+    Qt::darkYellow,
+    Qt::gray,
     
-    darkGray,
-    blue,
-    green,
-    cyan,
-    red,
-    magenta,
-    yellow,
-    white,
+    Qt::darkGray,
+    Qt::blue,
+    Qt::green,
+    Qt::cyan,
+    Qt::red,
+    Qt::magenta,
+    Qt::yellow,
+    Qt::white
 };
 
 static XFontStruct *fontStruct;
@@ -248,7 +244,7 @@ static Display *display;
 
 static QPoint LastMousePos;
 
-static int LastMouseX = -1, LastMouseY = -1;
+//static int LastMouseX = -1, LastMouseY = -1;
 
 static qEvent *event_head = 0;
 static qEvent *event_tail = 0;
@@ -284,15 +280,15 @@ static void qGetEvent(TEvent &Event) {
     delete q;
 }
 
-QEView::QEView(GViewPeer *peer, QWidget *parent, const char *name): QFrame(parent, name)
+QEView::QEView(GViewPeer *peer, QWidget *parent, const char *name) :
+    QFrame(parent, name),
+    view(peer),
+    text(new QEText(peer, this)),
+    horz(new QScrollBar(QScrollBar::Horizontal, parent, 0)),
+    vert(new QScrollBar(QScrollBar::Vertical, parent, 0))
 {
-    view = peer;
-
-    text = new QEText(peer, this);
     CHECK_PTR(text);
-    horz = new QScrollBar(QScrollBar::Horizontal, parent, 0);
     CHECK_PTR(horz);
-    vert = new QScrollBar(QScrollBar::Vertical, parent, 0);
     CHECK_PTR(vert);
     horz->show();
     vert->show();
@@ -337,6 +333,7 @@ void QEView::sbHmoveLeft() {
     NextEvent.What = evCommand;
     NextEvent.Msg.View = view->View;
     NextEvent.Msg.Command = cmHScrollLeft;
+    NextEvent.Msg.Param1 = 1;
     ActiveEvent(NextEvent);
 }
 
@@ -344,6 +341,7 @@ void QEView::sbHmoveRight() {
     NextEvent.What = evCommand;
     NextEvent.Msg.View = view->View;
     NextEvent.Msg.Command = cmHScrollRight;
+    NextEvent.Msg.Param1 = 1;
     ActiveEvent(NextEvent);
 }
 
@@ -373,6 +371,7 @@ void QEView::sbVmoveUp() {
     NextEvent.What = evCommand;
     NextEvent.Msg.View = view->View;
     NextEvent.Msg.Command = cmVScrollUp;
+    NextEvent.Msg.Param1 = 1;
     ActiveEvent(NextEvent);
 }
 
@@ -380,6 +379,7 @@ void QEView::sbVmoveDown() {
     NextEvent.What = evCommand;
     NextEvent.Msg.View = view->View;
     NextEvent.Msg.Command = cmVScrollDown;
+    NextEvent.Msg.Param1 = 1;
     ActiveEvent(NextEvent);
 }
 
@@ -409,8 +409,9 @@ void QEView::sbVmoveTo(int pos) {
 QEText::QEText(GViewPeer *peer, QWidget *parent, const char *name): QWidget(parent, name) {
     view = peer;
 
-    setAcceptFocus(TRUE);
+    setFocusPolicy(QWidget::StrongFocus);
     setMinimumSize(100, 80);
+    DEBUGX(("QEText %p\n", this));
 }
 
 QEText::~QEText() {
@@ -419,7 +420,7 @@ QEText::~QEText() {
 void QEText::resizeEvent(QResizeEvent *qe) {
     int X, Y;
 
-    DEBUGX(("A: %X\n", qe));
+    DEBUGX(("A: %p\n", qe));
     QWidget::resizeEvent(qe);
     DEBUGX(("B\n"));
     
@@ -439,6 +440,7 @@ void QEText::resizeEvent(QResizeEvent *qe) {
 }
 
 void QEText::paintEvent(QPaintEvent *qe) {
+    DEBUGX(("Paint %p\n", qe));
     view->UpdateWindow(qe->rect().x(),
                        qe->rect().y(),
                        qe->rect().width(),
@@ -456,16 +458,16 @@ void QEText::handleMouse(QMouseEvent *qe) {
     LastMousePos = mapToGlobal(qe->pos());
 
     switch (qe->type()) {
-    case Event_MouseButtonPress:
-    case Event_MouseButtonDblClick:
+    case QEvent::MouseButtonPress:
+    case QEvent::MouseButtonDblClick:
         if (!view->View->IsActive())
             view->View->Parent->SelectView(view->View);
         NextEvent.What = evMouseDown;
         break;
-    case Event_MouseButtonRelease:
+    case QEvent::MouseButtonRelease:
         NextEvent.What = evMouseUp;
         break;
-    case Event_MouseMove:
+    case QEvent::MouseMove:
         NextEvent.What = evMouseMove;
         break;
     default:
@@ -497,7 +499,7 @@ void QEText::handleMouse(QMouseEvent *qe) {
         NextEvent.Mouse.KeyMask |= kfAlt;
     
     NextEvent.Mouse.Count = 1;
-    if (qe->type() == Event_MouseButtonDblClick)
+    if (qe->type() == QEvent::MouseButtonDblClick)
         NextEvent.Mouse.Count = 2;
     NextEvent.Mouse.X = X;
     NextEvent.Mouse.Y = Y;
@@ -517,48 +519,48 @@ void QEText::mouseReleaseEvent(QMouseEvent *qe) {
     handleMouse(qe);
 }
 
-static struct {
+static const struct {
     unsigned int q_code;
     TKeyCode keyCode;
 } key_table[] = {
-{ Key_Escape,     kbEsc },
-{ Key_Tab,        kbTab },
-{ Key_Backtab,    kbTab | kfShift },
-{ Key_Backspace,  kbBackSp },
-{ Key_Return,     kbEnter },
-{ Key_Enter,      kbEnter },
-{ Key_Insert,     kbIns },
-{ Key_Delete,     kbDel },
-{ Key_Pause,      kbPause },
-{ Key_Print,      kbPrtScr },
-{ Key_SysReq,     kbSysReq },
-{ Key_Home,       kbHome },
-{ Key_End,        kbEnd },
-{ Key_Left,       kbLeft },
-{ Key_Up,         kbUp },
-{ Key_Right,      kbRight },
-{ Key_Down,       kbDown },
-{ Key_Prior,      kbPgUp },
-{ Key_Next,       kbPgDn },
-{ Key_Shift,      kbShift | kfModifier },
-{ Key_Control,    kbCtrl | kfModifier },
-{ Key_Meta,       kbAlt | kfModifier },
-{ Key_Alt,        kbAlt | kfModifier },
-{ Key_CapsLock,   kbCapsLock | kfModifier },
-{ Key_NumLock,    kbNumLock | kfModifier },
-{ Key_ScrollLock, kbScrollLock | kfModifier },
-{ Key_F1,         kbF1 },
-{ Key_F2,         kbF2 },
-{ Key_F3,         kbF3 },
-{ Key_F4,         kbF4 },
-{ Key_F5,         kbF5 },
-{ Key_F6,         kbF6 },
-{ Key_F7,         kbF7 },
-{ Key_F8,         kbF8 },
-{ Key_F9,         kbF9 },
-{ Key_F10,        kbF10 },
-{ Key_F11,        kbF11 },
-{ Key_F12,        kbF12 },
+    { Qt::Key_Escape,     kbEsc },
+    { Qt::Key_Tab,        kbTab },
+    { Qt::Key_Backtab,    kbTab | kfShift },
+    { Qt::Key_Backspace,  kbBackSp },
+    { Qt::Key_Return,     kbEnter },
+    { Qt::Key_Enter,      kbEnter },
+    { Qt::Key_Insert,     kbIns },
+    { Qt::Key_Delete,     kbDel },
+    { Qt::Key_Pause,      kbPause },
+    { Qt::Key_Print,      kbPrtScr },
+    { Qt::Key_SysReq,     kbSysReq },
+    { Qt::Key_Home,       kbHome },
+    { Qt::Key_End,        kbEnd },
+    { Qt::Key_Left,       kbLeft },
+    { Qt::Key_Up,         kbUp },
+    { Qt::Key_Right,      kbRight },
+    { Qt::Key_Down,       kbDown },
+    { Qt::Key_Prior,      kbPgUp },
+    { Qt::Key_Next,       kbPgDn },
+    { Qt::Key_Shift,      kbShift | kfModifier },
+    { Qt::Key_Control,    kbCtrl | kfModifier },
+    { Qt::Key_Meta,       kbAlt | kfModifier },
+    { Qt::Key_Alt,        kbAlt | kfModifier },
+    { Qt::Key_CapsLock,   kbCapsLock | kfModifier },
+    { Qt::Key_NumLock,    kbNumLock | kfModifier },
+    { Qt::Key_ScrollLock, kbScrollLock | kfModifier },
+    { Qt::Key_F1,         kbF1 },
+    { Qt::Key_F2,         kbF2 },
+    { Qt::Key_F3,         kbF3 },
+    { Qt::Key_F4,         kbF4 },
+    { Qt::Key_F5,         kbF5 },
+    { Qt::Key_F6,         kbF6 },
+    { Qt::Key_F7,         kbF7 },
+    { Qt::Key_F8,         kbF8 },
+    { Qt::Key_F9,         kbF9 },
+    { Qt::Key_F10,        kbF10 },
+    { Qt::Key_F11,        kbF11 },
+    { Qt::Key_F12,        kbF12 },
 };
 
 void QEText::ActiveEvent(TEvent &Event) {
@@ -572,7 +574,7 @@ void QEText::handleKeyPressEvent(QKeyEvent *qe) {
     TKeyCode keyFlags;
     int state = qe->state();
     int ascii = qe->ascii();
-    int key = qe->key();
+    unsigned int key = qe->key();
     
     DEBUGX(("key: %d, ascii: %d(%c) state:%X\n",
            qe->key(),
@@ -589,12 +591,12 @@ void QEText::handleKeyPressEvent(QKeyEvent *qe) {
         keyFlags |= kfAlt;
 
     keyCode = 0;
-    for (int i = 0; i < (sizeof(key_table)/sizeof(key_table[0])); i++) {
+    for (size_t i = 0; i < FTE_ARRAY_SIZE(key_table); ++i)
         if (key == key_table[i].q_code) {
             keyCode = key_table[i].keyCode;
             break;
         }
-    }
+
     if (keyCode == 0 && ascii != 0) {
         if (keyFlags & (kfCtrl | kfAlt)) {
             keyCode = toupper(ascii);
@@ -607,7 +609,7 @@ void QEText::handleKeyPressEvent(QKeyEvent *qe) {
         return;
     }
 
-    DEBUGX(("key: %d, flags:%d\n", keyCode, keyFlags));
+    DEBUGX(("key: %d, flags:%d\n", (int)keyCode, (int)keyFlags));
 
     NextEvent.What = evKeyDown;
     NextEvent.Key.View = view->View;
@@ -624,13 +626,11 @@ void QEText::keyPressEvent(QKeyEvent *qe) {
 //}
 
 void QEText::focusInEvent(QFocusEvent *qe) {
-    //repaint(FALSE);
-    DEBUGX(("got focus\n"));
+    repaint(FALSE);
 }
 
 void QEText::focusOutEvent(QFocusEvent *qe) {
-    //repaint(FALSE);
-    DEBUGX(("lost focus\n"));
+    repaint(FALSE);
 }
 
 QEFrame::QEFrame(GFramePeer *peer, QWidget *parent, const char *name): QFrame(parent, name)
@@ -681,7 +681,7 @@ void QEFrame::resizeEvent(QResizeEvent *qe) {
 }
 
 void QEFrame::closeEvent(QCloseEvent *qe) {
-    DEBUGX(("Close Selected: %d\n", id));
+    DEBUGX(("Close Selected: %p\n", qe));
     NextEvent.What = evCommand;
     NextEvent.Msg.View = frame->Frame->Active;
     NextEvent.Msg.Command = cmClose;
@@ -756,39 +756,36 @@ static void CloseWindow(Widget w, GFramePeer *frame, XEvent *event, Boolean *con
 #endif
 ///////////////////////////////////////////////////////////////////////////
 
-GViewPeer::GViewPeer(GView *view, int XSize, int YSize) {
-    
-    View = view;
-    
-    //    wX = 0;
-    //    wY = 0;
-    wW = XSize;
-    wH = YSize;
-    sbVtotal = 0;
-    sbVstart = 0;
-    sbVamount = 0;
-    sbHtotal = 0;
-    sbHstart = 0;
-    sbHamount = 0;
-    wState = 0;
-    cVisible = 1;
-    cStart = 0; // %
-    cEnd = 100;
-    wRefresh = 0;
-    ScreenBuffer = 0;
-    cX = -1;
-    cY = -1;
-    VertPos = HorzPos = -1;
-
+GViewPeer::GViewPeer(GView *view, int XSize, int YSize) :
+    View(view),
+    wW(XSize),
+    wH(YSize),
+    wState(0),
+    wRefresh(0),
+    cX(-1),
+    cY(-1),
+    cVisible(1),
+    cStart(0), // %
+    cEnd(100),
+    sbVstart(0),
+    sbVamount(0),
+    sbVtotal(0),
+    sbHstart(0),
+    sbHamount(0),
+    sbHtotal(0),
+    VertPos(-1),
+    HorzPos(-1),
+    ScreenBuffer(0)
+{
     for (int jj = 0; jj < 256; jj++)
          GCs[jj] = 0;
-    
+
     qView = new QEView(this, frames->Peer->qFrame);
     qView->show();
 }
 
 GViewPeer::~GViewPeer() {
-    delete qView;
+    //delete qView; // auto deleted
 }
 
 int GViewPeer::AllocBuffer() {
@@ -828,7 +825,7 @@ void GViewPeer::DrawCursor(/*QPainter *painter, */int Show) {
     //    if (!XtIsManaged(TextWin)) return ;
 
     if (cVisible && cX >= 0 && cY >= 0) {
-        char *p = CursorXYPos(cX, cY);
+        unsigned char* p = CursorXYPos(cX, cY);
         unsigned char attr;
 
         attr = p[1];
@@ -949,7 +946,7 @@ int GViewPeer::ConPutBox(int X, int Y, int W, int H, PCell Cell) {
             len -= l;
         }
         p = CursorXYPos(X, Y + i);
-        memcpy(p, Cell, W * 2);
+        memmove(p, Cell, W * sizeof(TCell));
         if (i + Y == cY)
             DrawCursor(1);
         Cell += W;
@@ -1110,7 +1107,7 @@ int GViewPeer::ConSetSize(int X, int Y) {
     unsigned char *NewBuffer;
     unsigned char *p;
     int i;
-    unsigned int MX, MY;
+    int MX, MY;
     
     p = NewBuffer = (unsigned char *) malloc(X * Y * 2);
     if (NewBuffer == NULL) return -1;
@@ -1119,7 +1116,7 @@ int GViewPeer::ConSetSize(int X, int Y) {
         *p++ = 0x07;
     }
     if (ScreenBuffer) {
-        MX = wW; if (X < MX) MX = X;
+	MX = wW; if (X < MX) MX = X;
         MY = wH; if (Y < MY) MY = Y;
         if (X < MX) MX = X;
         p = NewBuffer;
@@ -1279,10 +1276,12 @@ int GViewPeer::PMSetCursorPos() {
 
 ///////////////////////////////////////////////////////////////////////////
 
-GView::GView(GFrame *parent, int XSize, int YSize) {
-    Parent = parent;
-    Prev = Next = 0;
-    Peer = new GViewPeer(this, XSize, YSize);
+GView::GView(GFrame *parent, int XSize, int YSize) :
+    Parent(parent),
+    Next(0),
+    Prev(0),
+    Peer(new GViewPeer(this, XSize, YSize))
+{
     if (Parent)
         Parent->AddView(this);
 }
@@ -1290,8 +1289,7 @@ GView::GView(GFrame *parent, int XSize, int YSize) {
 GView::~GView() {
     if (Parent)
         Parent->RemoveView(this);
-    if (Peer)
-        delete Peer;
+    delete Peer;
 }
 
 int GView::ConClear() {
@@ -1467,17 +1465,15 @@ int GFramePeer::ConQuerySize(int *X, int *Y) {
 //        --*X;
 //}
 
-int GFramePeer::ConSetTitle(char *Title, char *STitle) {
+int GFramePeer::ConSetTitle(const char *Title, const char *STitle) {
     qFrame->setCaption(Title);
     qFrame->setIconText(STitle);
     return 1;
 }
 
-int GFramePeer::ConGetTitle(char *Title, int MaxLen, char *STitle, int SMaxLen) {
-    strncpy(Title, qFrame->caption(), MaxLen);
-    Title[MaxLen - 1] = 0;
-    strncpy(STitle, qFrame->iconText(), SMaxLen);
-    STitle[SMaxLen - 1] = 0;
+int GFramePeer::ConGetTitle(char *Title, size_t MaxLen, char *STitle, size_t SMaxLen) {
+    strlcpy(Title, qFrame->caption(), MaxLen);
+    strlcpy(STitle, qFrame->iconText(), SMaxLen);
     return 1;
 }
 
@@ -1551,11 +1547,11 @@ GFrame::~GFrame() {
     Next = Prev = 0;
 }
 
-int GFrame::ConSetTitle(char *Title, char *STitle) {
+int GFrame::ConSetTitle(const char *Title, const char *STitle) {
     return Peer->ConSetTitle(Title, STitle);
 }
 
-int GFrame::ConGetTitle(char *Title, int MaxLen, char *STitle, int SMaxLen) {
+int GFrame::ConGetTitle(char *Title, size_t MaxLen, char *STitle, size_t SMaxLen) {
     return Peer->ConGetTitle(Title, MaxLen, STitle, SMaxLen);
 }
 
@@ -1746,7 +1742,7 @@ QPopupMenu *QEFrame::CreatePopup(QWidget *parent, int Id, int do_connect) {
     CHECK_PTR(menu);
 
     //menu->setFont(QFont("Helvetica", 12, QFont::Bold));
-    for (int i = 0; i < Menus[Id].Count; i++) {
+    for (unsigned i = 0; i < Menus[Id].Count; ++i) {
         if (Menus[Id].Items[i].Name) {
             //puts(Menus[Id].Items[i].Name);
             if (Menus[Id].Items[i].SubMenu != -1) {
@@ -1775,7 +1771,7 @@ QMenuBar *QEFrame::CreateMenuBar(QWidget *parent, int Id) {
     CHECK_PTR(menu);
 
     //menu->setFont(QFont("Helvetica", 12, QFont::Bold));
-    for (int i = 0; i < Menus[Id].Count; i++) {
+    for (unsigned i = 0; i < Menus[Id].Count; ++i) {
         if (Menus[Id].Items[i].Name) {
             //puts(Menus[Id].Items[i].Name);
 
@@ -1838,11 +1834,10 @@ void QEFrame::timerDone() {
 
 // GUI
 
-
-GUI::GUI(int &argc, char **argv, int XSize, int YSize) {
-
-    fArgc = argc;
-    fArgv = argv;
+GUI::GUI(int &argc, char **argv, int XSize, int YSize) :
+    fArgc(argc),
+    fArgv(argv)
+{
 
     new QApplication(argc, argv);
 
@@ -1859,7 +1854,7 @@ GUI::GUI(int &argc, char **argv, int XSize, int YSize) {
     if (fontStruct == NULL)
         fontStruct = XLoadQueryFont(display, "fixed");
     if (fontStruct == NULL) 
-        return ;
+        return;
 
     cxChar = fontStruct->max_bounds.width;
     cyChar = fontStruct->max_bounds.ascent + fontStruct->max_bounds.descent;
@@ -1882,9 +1877,9 @@ void GUI::DispatchEvent(GFrame *frame, GView *view, TEvent &Event) {
     }
 }
 
-int GUI::ConSuspend(void) { return 0; }
+int GUI::ConSuspend() { return 0; }
 
-int GUI::ConContinue(void) { return 0; }
+int GUI::ConContinue() { return 0; }
 
 int GUI::ConGetEvent(TEventMask EventMask, TEvent *Event, int WaitTime, int Delete, GView **view) {
     //return ::ConGetEvent(EventMask, Event, WaitTime, Delete, view);
@@ -1892,12 +1887,12 @@ int GUI::ConGetEvent(TEventMask EventMask, TEvent *Event, int WaitTime, int Dele
     return 0;
 }
 
-int GUI::ConPutEvent(TEvent Event) {
+int GUI::ConPutEvent(const TEvent& Event) {
     EventBuf = Event;
     return 0;
 }
 
-int GUI::ConFlush(void) {
+int GUI::ConFlush() {
     return 0;
 }
 
@@ -1928,7 +1923,7 @@ void GUI::ProcessEvent() {
     while (doLoop && qHasEvent()) {
         NextEvent.What = evNone;
         qGetEvent(NextEvent);
-        DEBUGX(("Got event: %d\n", NextEvent.What));
+        DEBUGX(("Got event: %d\n", (int)NextEvent.What));
         if (NextEvent.What == evMouseDown) {
             DEBUGX(("x:%d, y:%d, buttons:%d\n",
                     NextEvent.Mouse.X,
@@ -1959,22 +1954,21 @@ int GUI::ShowEntryScreen() {
     return 1;
 }
 
-int GUI::RunProgram(char *Command) {
-    char Cmd[1024];
-    int rc;
-    
+int GUI::RunProgram(int mode, char *Command) {
+    char Cmd[1024] = {0};
+
+    //strncpy(Cmd, XShellCommand, sizeof(Cmd));
+
     if (*Command == 0)  // empty string = shell
-        strcpy(Cmd, "xterm &");
+        strlcat(Cmd, " -ls &", sizeof(Cmd));
     else {
-        strcpy(Cmd, "xterm -e ");
-        strcat(Cmd, Command);
-        if (mode == RUN_ASYNC)
-            strcat(Cmd, " &");
+        strlcat(Cmd, " -e ", sizeof(Cmd));
+	strlcat(Cmd, Command, sizeof(Cmd));
+	if (mode == RUN_ASYNC)
+            strlcat(Cmd, " &", sizeof(Cmd));
     }
-    
-    rc = system(Cmd);
-    
-    return rc;
+
+    return system(Cmd);
 }
 
 /*void PipeCallback(GPipe *pipe, int *source, int *input) {
@@ -1989,7 +1983,7 @@ int GUI::RunProgram(char *Command) {
         DEBUGX(("Pipe %d\n", *source));
 }
   */
-int GUI::OpenPipe(char *Command, EModel *notify) {
+int GUI::OpenPipe(const char *Command, EModel *notify) {
     int i;
     
     for (i = 0; i < MAX_PIPES; i++) {
@@ -2032,9 +2026,9 @@ int GUI::SetPipeView(int id, EModel *notify) {
         return -1;
     if (Pipes[id].used == 0)
         return -1;
-    DEBUGX(("Pipe View: %d %08X\n", id, notify));
+    DEBUGX(("Pipe View: %d %p\n", id, notify));
     Pipes[id].notify = notify;
-    if (notify != Pipes[id].notify) 
+    if (notify != Pipes[id].notify) {
         if (notify) {
             //Pipes[id].input =
             //    XtAppAddInput(AppContext, Pipes[id].fd, XtInputReadMask, PipeCallback, &Pipes[id]);
@@ -2043,21 +2037,23 @@ int GUI::SetPipeView(int id, EModel *notify) {
             //    XtRemoveInput(Pipes[id].input);
           //      Pipes[id].input = 0;
             //}
-        }
+	}
+    }
+
     return 0;
 }
 
-int GUI::ReadPipe(int id, void *buffer, int len) {
-    int rc;
+ssize_t GUI::ReadPipe(int id, void *buffer, size_t len) {
+    ssize_t rc;
     
     if (id < 0 || id > MAX_PIPES)
         return -1;
     if (Pipes[id].used == 0)
         return -1;
-    DEBUGX(("Pipe Read: Get %d %d\n", id, len));
+    DEBUGX(("Pipe Read: Get %d %d\n", id, (int)len));
     
     rc = read(Pipes[id].fd, buffer, len);
-    DEBUGX(("Pipe Read: Got %d %d\n", id, len));
+    DEBUGX(("Pipe Read: Got %d %d\n", id, (int)len));
     if (rc == 0) {
         //if (Pipes[id].input != 0) {
             //XtRemoveInput(Pipes[id].input);
@@ -2088,15 +2084,27 @@ int GUI::ClosePipe(int id) {
 
 int GUI::multiFrame() { return 1; }
 
-int GetXSelection(int *len, char **data) {
+int GetXSelection(int *len, char **data, int clipboard) {
     QClipboard *cb = QApplication::clipboard();
     const char *text;
+    QClipboard::Mode mode;
+    switch (clipboard) {
+    case 0:
+        mode = QClipboard::Clipboard;
+        break;
+    case 1:
+        mode = QClipboard::Selection;
+        break;
+    default:
+        // not supported
+        return -1;
+    }
 
-    text = cb->text();
+    text = cb->text(mode);
     if (text == 0)
         return -1;
 
-    *len = strlen(text);
+    *len = (int)strlen(text);
     *data = (char *)malloc(*len);
     if (*data == 0)
         return -1;
@@ -2104,13 +2112,26 @@ int GetXSelection(int *len, char **data) {
     return 0;
 }
 
-int SetXSelection(int len, char *data) {
-    QClipboard *cb = QApplication::clipboard();
+int SetXSelection(int len, char *data, int clipboard) {
+    QClipboard* cb = QApplication::clipboard();
     char *text = (char *)malloc(len + 1);
+    QClipboard::Mode mode;
     if (text == 0)
         return -1;
+    switch (clipboard) {
+    case 0:
+        mode = QClipboard::Clipboard;
+        break;
+    case 1:
+        mode = QClipboard::Selection;
+        break;
+    default:
+        // not supported
+        return -1;
+    }
     memcpy(text, data, len);
     text[len] = 0;
+    cb->setText(text, mode);
     free(text);
     return 0;
 }
@@ -2124,10 +2145,10 @@ void DieError(int rc, const char *msg, ...) {
     exit(rc);
 }
 
-char ConGetDrawChar(int index) {
-    static char tab[] = "\x0D\x0C\x0E\x0B\x12\x19____+>\x1F\x01\x12 ";
+char ConGetDrawChar(unsigned int index) {
+    static const char tab[] = "\x0D\x0C\x0E\x0B\x12\x19____+>\x1F\x01\x12\x01\x01 \x02";
     
-    assert(index >= 0 && index < strlen(tab));
+    assert(index < sizeof(tab) - 1);
     
     return tab[index];
 }
