@@ -7,27 +7,14 @@
  *
  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <assert.h>
+#include "c_color.h"
+#include "c_mode.h"
+#include "gui.h"
+#include "sysdep.h"
+
 #include <signal.h>
 #include <stdarg.h>
-#if defined(MSVC)
-#include <malloc.h>
-#endif
-
-#include "console.h"
-#include "gui.h"
-#include "c_mode.h"
-#include "c_color.h"
-
-#if defined(_DEBUG) && defined(MSVC) && defined(MSVCDEBUG)
-#include <crtdbg.h>
-
-#define new new( _CLIENT_BLOCK, __FILE__, __LINE__)
-
-#endif //_DEBUG && MSVC && MSVCDEBUG
+#include <stdio.h>
 
 
 int ShowVScroll = 1;
@@ -79,7 +66,9 @@ public:
     int ConCursorVisible();
     int ConSetCursorSize(int Start, int End);
     
+#ifdef CONFIG_MOUSE
     int CaptureMouse(int grab);
+#endif
     int CaptureFocus(int grab);
     
     int QuerySbVPos();
@@ -99,8 +88,8 @@ public:
     GFramePeer(GFrame *aFrame, int Width, int Height);
     ~GFramePeer();
     
-    int ConSetTitle(char *Title, char *STitle);
-    int ConGetTitle(char *Title, int MaxLen, char *STitle, int SMaxLen);
+    int ConSetTitle(const char *Title, const char *STitle);
+    int ConGetTitle(char *Title, size_t MaxLen, char *STitle, size_t SMaxLen);
     
     int ConSetSize(int X, int Y);
     int ConQuerySize(int *X, int *Y);
@@ -109,30 +98,34 @@ public:
 
 ///////////////////////////////////////////////////////////////////////////
 
-GViewPeer::GViewPeer(GView *view, int XSize, int YSize) {
-    View = view;
-    wX = 0;
-    wY = 0;
-    wW = XSize;
-    wH = YSize;
-    sbVtotal = 0;
-    sbVstart = 0;
-    sbVamount = 0;
-    sbVupdate = 1;
-    sbHtotal = 0;
-    sbHstart = 0;
-    sbHamount = 0;
-    sbHupdate = 1;
-    wState = 0;
-    cVisible = 1;
-    cStart = 0; // %
-    cEnd = 100;
-    cX = cY = 0;
+GViewPeer::GViewPeer(GView *view, int XSize, int YSize) :
+    View(view),
+    wX(0),
+    wY(0),
+    wW(XSize),
+    wH(YSize),
+    wState(0),
+    cX(0),
+    cY(0),
+    cVisible(1),
+    cStart(0), // %
+    cEnd(100),
+    sbVstart(0),
+    sbVamount(0),
+    sbVtotal(0),
+    sbVupdate(1),
+    sbHstart(0),
+    sbHamount(0),
+    sbHtotal(0),
+    sbHupdate(1)
+{
 }
 
 GViewPeer::~GViewPeer() {
+#ifdef CONFIG_MOUSE
     if (MouseCapture == View)
         MouseCapture = 0;
+#endif
     if (FocusCapture == View)
         FocusCapture = 0;
 }
@@ -178,8 +171,8 @@ int GViewPeer::ConSetCursorPos(int X, int Y) {
     cY = Y;
     if (wState & sfFocus)
         return ::ConSetCursorPos(cX + wX, cY + wY);
-    else
-        return 1;
+
+    return 1;
 }
 
 int GViewPeer::ConQueryCursorPos(int *X, int *Y) {
@@ -192,16 +185,15 @@ int GViewPeer::ConShowCursor() {
     cVisible = 1;
     if (wState & sfFocus)
         return ::ConShowCursor();
-    else
-        return 1;
+    return 1;
 }
 
 int GViewPeer::ConHideCursor() {
     cVisible = 0;
     if (wState & sfFocus)
         return ::ConHideCursor();
-    else
-        return 1;
+
+    return 1;
 }
 
 int GViewPeer::ConCursorVisible() {
@@ -217,6 +209,7 @@ int GViewPeer::ConSetCursorSize(int Start, int End) {
         return 1;
 }
 
+#ifdef CONFIG_MOUSE
 int GViewPeer::CaptureMouse(int grab) {
     if (MouseCapture == 0) {
         if (grab)
@@ -231,6 +224,7 @@ int GViewPeer::CaptureMouse(int grab) {
     }
     return 1;
 }
+#endif
 
 int GViewPeer::CaptureFocus(int grab) {
     if (FocusCapture == 0) {
@@ -256,6 +250,7 @@ int GViewPeer::ExpandHeight(int DeltaY) {
         DeltaY = View->Next->Peer->wH - 3;
     View->Peer->ConSetSize(wW, wH + DeltaY);
     View->Next->Peer->wY += DeltaY;
+    assert(View->Next->Peer->wY >= 0);
     View->Next->Peer->ConSetSize(View->Next->Peer->wW, View->Next->Peer->wH - DeltaY);
     View->Resize(View->Peer->wW, View->Peer->wH);
     View->Next->Resize(View->Next->Peer->wW, View->Next->Peer->wH);
@@ -275,7 +270,6 @@ int GViewPeer::SetSbVPos(int Start, int Amount, int Total) {
         sbVamount = Amount;
         sbVtotal = Total;
         sbVupdate = 1;
-//        DrawScrollBar();
     }
     return 1;
 }
@@ -289,7 +283,6 @@ int GViewPeer::SetSbHPos(int Start, int Amount, int Total) {
         sbHamount = Amount;
         sbHtotal = Total;
         sbHupdate = 1;
-//        DrawScrollBar();
     }
     return 1;
 }
@@ -313,12 +306,12 @@ int GViewPeer::DrawScrollBar() {
     
     ConQuerySize(&W, &H);
 
-    if (ShowVScroll) {
+    if (ShowVScroll && H > 1) {
         MoveCh(B, ConGetDrawChar(DCH_AUP), hcScrollBar_Arrows, 1);
         ConPutBox(W, 0, 1, 1, B);
         MoveCh(B, ConGetDrawChar(DCH_ADOWN), hcScrollBar_Arrows, 1);
         ConPutBox(W, H - 1, 1, 1, B);
-        
+
         NRows = H - 2;
         
         if (sbVtotal <= NRows) {
@@ -337,7 +330,7 @@ int GViewPeer::DrawScrollBar() {
             ConPutBox(W, I + 1, 1, 1, B);
         }
     }
-    if (ShowHScroll) {
+    if (ShowHScroll && W > 1) {
         MoveCh(B, ConGetDrawChar(DCH_ALEFT), hcScrollBar_Arrows, 1);
         ConPutBox(0, H, 1, 1, B);
         MoveCh(B, ConGetDrawChar(DCH_ARIGHT), hcScrollBar_Arrows, 1);
@@ -373,18 +366,22 @@ int GViewPeer::DrawScrollBar() {
 
 ///////////////////////////////////////////////////////////////////////////
 
-GView::GView(GFrame *parent, int XSize, int YSize) {
-    Parent = parent;
-    Prev = Next = 0;
-    Peer = new GViewPeer(this, XSize, YSize);
+GView::GView(GFrame *parent, int XSize, int YSize) :
+    Parent(parent),
+    Next(0),
+    Prev(0),
+    Peer(new GViewPeer(this, XSize, YSize))
+{
     if (Parent)
         Parent->AddView(this);
+    //fprintf(stderr, "CREATE GVIEW  %p\n", this);
 }
 
 GView::~GView() {
     if (Parent)
         Parent->RemoveView(this);
     delete Peer;
+    //fprintf(stderr, "DESTROY GVIEW  %p\n", this);
 }
 
 int GView::ConClear() {
@@ -418,11 +415,11 @@ int GView::ConScroll(int Way, int X, int Y, int W, int H, TAttr Fill, int Count)
 }
 
 int GView::ConSetSize(int X, int Y) {
-    if (Peer->ConSetSize(X, Y))
-        Resize(X, Y);
-    else
-        return 0;
-    return 1;
+    if (Peer->ConSetSize(X, Y)) {
+	Resize(X, Y);
+	return 1;
+    }
+    return 0;
 }
 
 int GView::ConQuerySize(int *X, int *Y) {
@@ -453,9 +450,11 @@ int GView::ConSetCursorSize(int Start, int End) {
     return Peer->ConSetCursorSize(Start, End);
 }
 
+#ifdef CONFIG_MOUSE
 int GView::CaptureMouse(int grab) {
     return Peer->CaptureMouse(grab);
 }
+#endif
 
 int GView::CaptureFocus(int grab) {
     return Peer->CaptureFocus(grab);
@@ -558,55 +557,53 @@ int GFramePeer::ConQuerySize(int *X, int *Y) {
 //        --*X;
 //}
 
-int GFramePeer::ConSetTitle(char *Title, char *STitle) {
+int GFramePeer::ConSetTitle(const char *Title, const char *STitle) {
     ::ConSetTitle(Title, STitle);
     return 0;
 }
 
-int GFramePeer::ConGetTitle(char *Title, int MaxLen, char *STitle, int SMaxLen) {
+int GFramePeer::ConGetTitle(char *Title, size_t MaxLen, char *STitle, size_t SMaxLen) {
     return ::ConGetTitle(Title, MaxLen, STitle, SMaxLen);
 }
 
 ///////////////////////////////////////////////////////////////////////////
 
-GFrame::GFrame(int XSize, int YSize) {
-    Menu = 0;
+GFrame::GFrame(int XSize, int YSize) :
+    Top(0),
+    Active(0),
+    Menu(0)
+{
     if (frames == 0) {
         frames = Prev = Next = this;
     } else {
         Next = frames->Next;
-        Prev = frames;
+	Prev = frames;
         frames->Next->Prev = this;
         frames->Next = this;
         frames = this;
     }
-    Top = Active = 0;
     Peer = new GFramePeer(this, XSize, YSize);
 }
 
 GFrame::~GFrame() {
-    if (Peer) {
-        delete Peer;
-        Peer = 0;
-    }
+    //fprintf(stderr, "DELETE GFRAME %p  %p\n", Next, this);
+    delete Peer;
+
     if (Next == this) {
         frames = 0;
-//        printf("No more frames\x7\x7\n");
     } else {
         Next->Prev = Prev;
         Prev->Next = Next;
         frames = Next;
     }
-    Next = Prev = 0;
-    if (Menu)
-	free(Menu);
+    free(Menu);
 }
 
-int GFrame::ConSetTitle(char *Title, char *STitle) {
+int GFrame::ConSetTitle(const char *Title, const char *STitle) {
     return Peer->ConSetTitle(Title, STitle);
 }
 
-int GFrame::ConGetTitle(char *Title, int MaxLen, char *STitle, int SMaxLen) {
+int GFrame::ConGetTitle(char *Title, size_t MaxLen, char *STitle, size_t SMaxLen) {
     return Peer->ConGetTitle(Title, MaxLen, STitle, SMaxLen);
 }
 
@@ -627,6 +624,7 @@ int GFrame::ConSplitView(GView *view, GView *newview) {
     if (ShowVScroll) 
         newview->Peer->wW--;
     newview->Peer->wY = view->Peer->wY + view->Peer->wH / 2;
+    assert(newview->Peer->wY >= 0);
     newview->Peer->wH = view->Peer->wH - view->Peer->wH / 2;
     if (ShowHScroll) {
         newview->Peer->wY++;
@@ -648,45 +646,40 @@ int GFrame::ConResizeView(GView * /*view*/, int /*DeltaY*/) {
 }
 
 int GFrame::AddView(GView *view) {
-    if (Active != 0) {
+    //fprintf(stderr, "GFRAME ADDVIEW %p,  t %p   a %p\n", view, Top, Active);
+    if (Active != 0)
         return ConSplitView(Active, view);
-    } else {
-        int W, H;
-        
-        view->Parent = this;
-        view->Prev = view->Next = 0;
-        
-        view->Peer->wX = 0;
-        if (ShowMenuBar)
-            view->Peer->wY = 1;
-        else
-            view->Peer->wY = 0;
-        ConQuerySize(&W, &H);
-        if (ShowMenuBar)
-            H--;
-        if (ShowVScroll)
-            W--;
-        if (ShowHScroll)
-            H--;
-        view->ConSetSize(W, H);
-        InsertView(Top, view);
-        return 0;
-    }
+
+    int W, H;
+
+    view->Parent = this;
+    view->Prev = view->Next = 0;
+
+    view->Peer->wX = 0;
+    view->Peer->wY = (ShowMenuBar) ? 1 : 0;
+	ConQuerySize(&W, &H);
+    if (ShowMenuBar)
+	H--;
+    if (ShowVScroll)
+	W--;
+    if (ShowHScroll)
+	H--;
+    view->ConSetSize(W, H);
+    InsertView(Top, view);
+    return 0;
 }
 
 void GFrame::Update() {
-    GView *v = Active;
     
     UpdateMenu();
-    while (v) {
+    for (GView *v = Active; v; v = v->Next) {
         v->Update();
         if ((ShowVScroll || ShowHScroll) && (v->Peer->sbVupdate || v->Peer->sbHupdate)) {
             v->Peer->DrawScrollBar();
             v->Peer->sbVupdate = 0;
             v->Peer->sbHupdate = 0;
         }
-        v = v->Next;
-        if (v == Active) 
+        if (v->Next == Active)
             break;
     }
 }
@@ -697,19 +690,17 @@ void GFrame::UpdateMenu() {
 }
 
 void GFrame::Repaint() {
-    GView *v = Active;
     
     if (ShowMenuBar)
         DrawMenuBar();
-    while (v) {
+    for (GView *v = Active; v; v = v->Next) {
         v->Repaint();
         if (ShowVScroll || ShowHScroll) {
             v->Peer->DrawScrollBar();
             v->Peer->sbVupdate = 0;
             v->Peer->sbHupdate = 0;
         }
-        v = v->Next;
-        if (v == Active) 
+        if (v->Next == Active)
             break;
     }
 }
@@ -732,10 +723,11 @@ void GFrame::InsertView(GView *prev, GView *view) {
 }
 
 void GFrame::RemoveView(GView *view) {
-    if (!view) return ;
+    if (!view) return;
     
     if (Active == view)
         Active->Activate(0);
+
     if (view->Next == view) {
         Top = Active = 0;
         delete this;
@@ -746,6 +738,7 @@ void GFrame::RemoveView(GView *view) {
         if (Top == view) {
             Top = view->Next;
             Top->Peer->wY -= view->Peer->wH;
+            assert(Top->Peer->wY >= 0);
             Top->ConSetSize(Top->Peer->wW, Top->Peer->wH + view->Peer->wH + (ShowHScroll ? 1 : 0));
         } else {
             view->Prev->ConSetSize(view->Prev->Peer->wW,
@@ -800,33 +793,26 @@ int GFrame::SelectView(GView *view) {
 }
 
 void GFrame::Resize(int width, int height) {
-    GView *V;
-    int count = 0;
-    
-    
-    V = Top;
-    while (V) {
+    if (!Top)
+        return;
+
+    int count = 1;
+    for (GView *V = Top->Next; V != Top; V = V->Next)
         count++;
-        if (V == Top) break;
-    }
+
     if (height < 2 * count + 2 || width < 16) {
         ::ConSetSize(16, 2 * count + 1);
         return;
     }
-    
-    if (!Top)
-        return;
-    
+
     if (ShowVScroll)
         width--;
     if (ShowHScroll)
         height--;
+
+    //fprintf(stderr, "Resize: %d %d   c:%d\n", width, height, count);
     
-//    fprintf(stderr, "Resize: %d %d\n", width, height);
-    
-    V = Top->Prev;
-    
-    while (V != Top) {
+    for (GView *V = Top->Prev; V != Top; V = V->Prev) {
         int h, y;
         
         h = V->Peer->wH;
@@ -841,7 +827,6 @@ void GFrame::Resize(int width, int height) {
         V->Peer->wY = y;
         V->ConSetSize(width, h);
         height = y;
-        V = V->Prev;
     }
     if (ShowMenuBar)
         height--;
@@ -858,7 +843,7 @@ int GFrame::ExecMainMenu(char Sub) {
 }
 
 int GFrame::SetMenu(const char *Name) {
-    if (Menu) free(Menu);
+    free(Menu);
     Menu = strdup(Name);
     return 0;
 }
@@ -887,11 +872,11 @@ int GUI::ConGetEvent(TEventMask EventMask, TEvent *Event, int WaitTime, int Dele
     return ::ConGetEvent(EventMask, Event, WaitTime, Delete);
 }
 
-int GUI::ConPutEvent(TEvent Event) {
+int GUI::ConPutEvent(const TEvent& Event) {
     return ::ConPutEvent(Event);
 }
 
-int GUI::ConFlush(void) {
+int GUI::ConFlush() {
     return 0;
 }
 
@@ -905,12 +890,11 @@ static inline int scrollBreak(TEvent &E)
 #endif
 }
 
-void HandleVScroll(GView *view, TEvent &E) {
-    int y; //, x
+static void HandleVScroll(GView *view, TEvent &E) {
+    int y;
     int wY, wH;
     TEvent E1;
     
-    //x = E.Mouse.X;
     y = E.Mouse.Y;
     wY = view->Peer->wY;
     wH = view->Peer->wH;
@@ -919,6 +903,7 @@ void HandleVScroll(GView *view, TEvent &E) {
             E1.What = evCommand;
             E1.Msg.View = view;
             E1.Msg.Command = cmVScrollUp;
+            E1.Msg.Param1 = 1;
             gui->DispatchEvent(frames, view, E1);
             frames->Update();
             do {
@@ -933,6 +918,7 @@ void HandleVScroll(GView *view, TEvent &E) {
             E1.What = evCommand;
             E1.Msg.View = view;
             E1.Msg.Command = cmVScrollDown;
+            E1.Msg.Param1 = 1;
             gui->DispatchEvent(frames, view, E1);
             frames->Update();
             do {
@@ -992,12 +978,11 @@ void HandleVScroll(GView *view, TEvent &E) {
     E.What = evNone;
 }
 
-void HandleHScroll(GView *view, TEvent &E) {
-    int x; //, x
+static void HandleHScroll(GView *view, TEvent &E) {
+    int x;
     int wX, wW;
     TEvent E1;
     
-    //x = E.Mouse.X;
     x = E.Mouse.X;
     wX = view->Peer->wX;
     wW = view->Peer->wW;
@@ -1006,6 +991,7 @@ void HandleHScroll(GView *view, TEvent &E) {
             E1.What = evCommand;
             E1.Msg.View = view;
             E1.Msg.Command = cmHScrollLeft;
+            E1.Msg.Param1 = 1;
             gui->DispatchEvent(frames, view, E1);
             frames->Update();
             do {
@@ -1020,6 +1006,7 @@ void HandleHScroll(GView *view, TEvent &E) {
             E1.What = evCommand;
             E1.Msg.View = view;
             E1.Msg.Command = cmHScrollRight;
+            E1.Msg.Param1 = 1;
             gui->DispatchEvent(frames, view, E1);
             frames->Update();
             do {
@@ -1083,14 +1070,12 @@ void GUI::ProcessEvent() {
     TEvent E;
     
     E = NextEvent;
-    if (E.What != evNone) {
+    if (E.What != evNone)
         NextEvent.What = evNone;
-    }
-    if (E.What == evNone &&
-       ( ConGetEvent(evMouse | evCommand | evKeyboard, &E, 0, 1, 0) == -1 ||
-          E.What == evNone )
-       )
-    {
+
+    if (E.What == evNone
+	&& (ConGetEvent(evMouse | evCommand | evKeyboard, &E, 0, 1, 0) == -1
+	    || E.What == evNone)) {
         frames->Update();
         while(ConGetEvent(evMouse | evCommand | evKeyboard, &E, -1, 1, 0) == -1 ||
            (E.What == evMouseMove && E.Mouse.Buttons == 0));
@@ -1182,13 +1167,15 @@ void GUI::ProcessEvent() {
                 break;
             case cmPopupMenu:
                 {
-                    int id = E.Msg.Param1;
+                    long id = E.Msg.Param1;
                     int Cols, Rows;
                     
                     if (id == -1) return;
                     frames->ConQuerySize(&Cols, &Rows);
                     int x = Cols / 2, y = Rows / 2;
-                    ::ConQueryMousePos(&x, &y);
+#ifdef CONFIG_MOUSE
+                    ConQueryMousePos(&x, &y);
+#endif
                     
                     frames->Update(); // sync before menu
                     if (::ExecVertMenu(x, y, id, E, 0) != 1) {
