@@ -7,15 +7,15 @@
  *
  */
 
-#ifndef __GUI_H
-#define __GUI_H
+#ifndef GUI_H
+#define GUI_H
 
-#define RUN_WAIT 0
-#define RUN_ASYNC 1
+#include "console.h"
+
+#include <stdarg.h>
 
 class GFramePeer;
 class GViewPeer;
-
 class GUI;
 class GFrame;
 
@@ -76,8 +76,8 @@ public:
     GFrame(int XSize, int YSize);
     virtual ~GFrame();
     
-    int ConSetTitle(char *Title, char *STitle);
-    int ConGetTitle(char *Title, int MaxLen, char *STitle, int SMaxLen);
+    int ConSetTitle(const char *Title, const char *STitle);
+    int ConGetTitle(char *Title, size_t MaxLen, char *STitle, size_t SMaxLen);
     
     int ConSetSize(int X, int Y);
     int ConQuerySize(int *X, int *Y);
@@ -112,6 +112,11 @@ public:
 
 class GUI {
 public:
+    enum {
+	RUN_WAIT,
+	RUN_ASYNC
+    };
+
     GUI(int &argc, char **argv, int XSize, int YSize);
     virtual ~GUI();
     
@@ -121,9 +126,9 @@ public:
     
     void ProcessEvent();
     virtual void DispatchEvent(GFrame *frame, GView *view, TEvent &Event);
-    int ConGetEvent(TEventMask EventMask, TEvent *Event, int WaitTime, int Delete, GView **view);
-    int ConPutEvent(TEvent Event);
-    int ConFlush(void);
+    int ConGetEvent(TEventMask EventMask, TEvent* Event, int WaitTime, int Delete, GView **view);
+    int ConPutEvent(const TEvent& Event);
+    int ConFlush();
     int ConGrabEvents(TEventMask EventMask);
 
     virtual int Start(int &argc, char **argv);
@@ -134,13 +139,12 @@ public:
 
     int RunProgram(int mode, char *Command);
     
-    int OpenPipe(char *Command, EModel *notify);
+    int OpenPipe(const char *Command, EModel *notify);
     int SetPipeView(int id, EModel *notify);
-    int ReadPipe(int id, void *buffer, int len);
+    ssize_t ReadPipe(int id, void *buffer, size_t len);
     int ClosePipe(int id);
 
     int multiFrame();
-    void deleteFrame(GFrame *frame);
 
     int fArgc;
     char **fArgv;
@@ -177,7 +181,7 @@ int DLGPickChoice(GView *View, const char *ATitle, int NSel, va_list ap, int Fla
 #define SEARCH_BACK    0x00000001   // reverse (TODO for regexps)
 #define SEARCH_RE      0x00000002   // use regexp
 #define SEARCH_NCASE   0x00000004   // case
-#define SEARCH_GLOBAL  0x00000008   // start from beggining (or end if BACK)
+#define SEARCH_GLOBAL  0x00000008   // start from begining (or end if BACK)
 #define SEARCH_BLOCK   0x00000010   // search in block
 #define SEARCH_NEXT    0x00000020   // next match
 #define SEARCH_NASK    0x00000040   // ask before replacing
@@ -199,16 +203,22 @@ int DLGPickChoice(GView *View, const char *ATitle, int NSel, va_list ap, int Fla
 
 #define MAXSEARCH 512
 
-typedef struct {
+struct SearchReplaceOptions {
     int ok;
+    int resCount;
+    int lastInsertLen;
+    unsigned long Options;
+
+    //enum { MAXSEARCH = 512 };
     char strSearch[MAXSEARCH];
     char strReplace[MAXSEARCH];
-    unsigned long Options;
-    //
-    int resCount;
-} SearchReplaceOptions;
+
+    SearchReplaceOptions();
+};
 
 int DLGGetFind(GView *View, SearchReplaceOptions &sr);
 int DLGGetFindReplace(GView *View, SearchReplaceOptions &sr);
 
-#endif
+int WaitFdPipeEvent(TEvent *Event, int fd, int WaitTime);
+
+#endif // GUI_H
