@@ -11,45 +11,52 @@
 
 #ifdef CONFIG_HILIT_FTE
 
+#include "c_bind.h"
+#include "o_buflist.h"
+
+#include <ctype.h>
+
+// These states should match those in h_c.cpp to let autoindentation work
 #define hsFTE_Normal       0
 #define hsFTE_Comment      1
 #define hsFTE_Keyword      4
 #define hsFTE_String1     10
 #define hsFTE_String2     11
-#define hsFTE_Regexp      12
-#define hsFTE_KeySpec     13
+#define hsFTE_CPP         12
+#define hsFTE_Regexp      15
+#define hsFTE_KeySpec     16
 
-int Hilit_FTE(EBuffer *BF, int LN, PCell B, int Pos, int Width, ELine *Line, hlState &State, hsState *StateMap, int *ECol) {
-    ChColor *Colors = BF->Mode->fColorize->Colors;
+int Hilit_FTE(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line, hlState &State, hsState *StateMap, int *ECol) {
     int j = 0;
-    HILIT_VARS(Colors[CLR_Normal], Line);
+    HILIT_VARS(BF->Mode->fColorize->Colors, Line);
     int len1 = len;
     char *last = p + len1 - 1;
 
     C = 0;
     NC = 0;
-    for(i = 0; i < Line->Count;) {
+
+    for (i = 0; i < Line->Count;) {
         IF_TAB() else {
             switch(State) {
             default:
             case hsFTE_Normal:
-		if (isalpha(*p) || *p == '_') {
-		    j = 0;
-		    while (((i + j) < Line->Count) &&
-			   (isalnum(Line->Chars[i+j]) ||
-			    (Line->Chars[i + j] == '_'))
-			   ) j++;
-		    if (BF->GetHilitWord(j, &Line->Chars[i], Color)) {
-			State = hsFTE_Keyword;
-		    } else {
-                        Color = Colors[CLR_Normal];
-			State = hsFTE_Normal;
-		    }
-		    if (StateMap)
-			memset(StateMap + i, State, j);
-                    if (B) 
-			MoveMem(B, C - Pos, Width, Line->Chars + i, Color, j);
-		    i += j;
+                if (isalpha(*p) || *p == '_') {
+                    j = 0;
+                    while (((i + j) < Line->Count) &&
+                           (isalnum(Line->Chars[i+j]) ||
+                            (Line->Chars[i + j] == '_'))
+                          ) j++;
+                    if (BF->GetHilitWord(Color, &Line->Chars[i], j)) {
+                        State = hsFTE_Keyword;
+                    } else {
+                        Color = CLR_Normal;
+                        State = hsFTE_Normal;
+                    }
+                    if (StateMap)
+                        memset(StateMap + i, State, j);
+                    if (B)
+                        MoveMem(B, C - Pos, Width, Line->Chars + i, HILIT_CLRD(), j);
+                    i += j;
                     len -= j;
                     p += j;
                     C += j;
@@ -57,10 +64,14 @@ int Hilit_FTE(EBuffer *BF, int LN, PCell B, int Pos, int Width, ELine *Line, hlS
                     continue;
                 } else if (*p == '#') {
                     State = hsFTE_Comment;
-                    Color = Colors[CLR_Comment];
+                    Color = CLR_Comment;
+                    goto hilit;
+                } else if (*p == '%') {
+                    State = hsFTE_CPP;
+                    Color = CLR_CPreprocessor;
                     goto hilit;
                 } else if (isdigit(*p)) {
-                    Color = Colors[CLR_Number];
+                    Color = CLR_Number;
                     ColorNext();
                     while (len && (isdigit(*p) || *p == 'e' || *p == 'E' || *p == '.')) ColorNext();
                     if (len && (toupper(*p) == 'U')) ColorNext();
@@ -68,33 +79,36 @@ int Hilit_FTE(EBuffer *BF, int LN, PCell B, int Pos, int Width, ELine *Line, hlS
                     continue;
                 } else if (*p == '\'') {
                     State = hsFTE_String1;
-                    Color = Colors[CLR_String];
+                    Color = CLR_String;
                     goto hilit;
                 } else if (*p == '[') {
                     State = hsFTE_KeySpec;
-                    Color = Colors[CLR_Command];
+                    Color = CLR_Command;
                     goto hilit;
                 } else if (*p == '"') {
                     State = hsFTE_String2;
-                    Color = Colors[CLR_String];
+                    Color = CLR_String;
                     goto hilit;
                 } else if (*p == '/') {
                     State = hsFTE_Regexp;
-                    Color = Colors[CLR_Regexp];
+                    Color = CLR_Regexp;
                     goto hilit;
                 } else if (ispunct(*p)) {
-                    Color = Colors[CLR_Punctuation];
+                    Color = CLR_Punctuation;
                     ColorNext();
                     continue;
                 }
-                Color = Colors[CLR_Normal];
+                Color = CLR_Normal;
                 ColorNext();
                 continue;
             case hsFTE_Comment:
-                Color = Colors[CLR_Comment];
+                Color = CLR_Comment;
+                goto hilit;
+            case hsFTE_CPP:
+                Color = CLR_CPreprocessor;
                 goto hilit;
             case hsFTE_String1:
-                Color = Colors[CLR_String];
+                Color = CLR_String;
                 if ((len >= 2) && (*p == '\\')) {
                 hilit2:
                     ColorNext();
@@ -108,7 +122,7 @@ int Hilit_FTE(EBuffer *BF, int LN, PCell B, int Pos, int Width, ELine *Line, hlS
                 }
                 goto hilit;
             case hsFTE_String2:
-                Color = Colors[CLR_String];
+                Color = CLR_String;
                 if ((len >= 2) && (*p == '\\')) {
                     goto hilit2;
                 } else if (*p == '"') {
@@ -118,7 +132,7 @@ int Hilit_FTE(EBuffer *BF, int LN, PCell B, int Pos, int Width, ELine *Line, hlS
                 }
                 goto hilit;
             case hsFTE_KeySpec:
-                Color = Colors[CLR_Command];
+                Color = CLR_Command;
                 if ((len >= 2) && (*p == '\\')) {
                     goto hilit2;
                 } else if (*p == ']') {
@@ -128,7 +142,7 @@ int Hilit_FTE(EBuffer *BF, int LN, PCell B, int Pos, int Width, ELine *Line, hlS
                 }
                 goto hilit;
             case hsFTE_Regexp:
-                Color = Colors[CLR_Regexp];
+                Color = CLR_Regexp;
                 if ((len >= 2) && (*p == '\\')) {
                     goto hilit2;
                 } else if (*p == '/') {
@@ -140,10 +154,13 @@ int Hilit_FTE(EBuffer *BF, int LN, PCell B, int Pos, int Width, ELine *Line, hlS
             }
         }
     }
+
     if (State == hsFTE_Comment)
         State = hsFTE_Normal;
+
     if ((len1 == 0) || (*last != '\\')) {
         switch(State) {
+        case hsFTE_CPP:
         case hsFTE_String1:
         case hsFTE_String2:
         case hsFTE_KeySpec:
@@ -155,4 +172,4 @@ int Hilit_FTE(EBuffer *BF, int LN, PCell B, int Pos, int Width, ELine *Line, hlS
     *ECol = C;
     return 0;
 }
-#endif
+#endif // CONFIG_HILIT_FTE
