@@ -7,13 +7,18 @@
  *
  */
 
-#ifndef __EXCOMPLETE_H
-#define __EXCOMPLETE_H
+#ifndef I_COMPLETE_H
+#define I_COMPLETE_H
+
+#include "i_oview.h"
+#include "e_buffer.h"
+
+#include <ctype.h>
 
 // maximum words which will be presented to the user
 #define MAXCOMPLETEWORDS 300
 
-class ExComplete: public ExView {
+class ExComplete: public ExViewNext {
     EPoint Orig;
     EBuffer *Buffer;
     int WordsLast;
@@ -25,23 +30,19 @@ class ExComplete: public ExView {
     size_t WordFixedCount;
 
     int RefreshComplete();
-    inline int CheckASCII(int c) {
-	return ((c < 256)
-		&& (isalnum(c) || (c == '_') || (c == '.'))) ? 1 : 0;
-    }
+    int CheckASCII(char c) const { return (isalnum(c) || (c == '_') || (c == '.')); }
     void FixedUpdate(int add);
 
 public:
 
     ExComplete(EBuffer *B);
     virtual ~ExComplete();
-    virtual void Activate(int gotfocus);
-    virtual ExView* GetViewContext() { return Next; }
-    virtual int BeginMacro();
+
     virtual void HandleEvent(TEvent &Event);
-    virtual void UpdateView();
-    virtual void RepaintView();
-    virtual void UpdateStatus();
     virtual void RepaintStatus();
+
+    bool IsSimpleCase();
+    int DoCompleteWord();
 };
-#endif
+
+#endif // I_COMPLETE_H
