@@ -7,9 +7,26 @@
  *
  */
 
-#include "fte.h"
+#include "c_color.h"
+#include "c_config.h"
+#include "c_history.h"
+#include "e_mark.h"
+#include "e_tags.h"
+#include "e_undo.h"
+#include "ftever.h"
+#include "i_modelview.h"
+#include "i_view.h"
+#include "o_cvsdiff.h"
+#include "o_messages.h"
+#include "o_svndiff.h"
+#include "s_files.h"
+#include "s_string.h"
+#include "s_util.h"
 
-SearchReplaceOptions LSearch = { 0 };
+#include <ctype.h>
+#include <time.h>
+
+SearchReplaceOptions LSearch;
 int suspendLoads = 0;
 
 EViewPort *EBuffer::CreateViewPort(EView *V) {
@@ -23,6 +40,13 @@ EViewPort *EBuffer::CreateViewPort(EView *V) {
         if (CompilerMsgs)
             CompilerMsgs->FindFileErrors(this);
 #endif
+#ifdef CONFIG_OBJ_CVS
+        if (CvsDiffView) CvsDiffView->FindFileLines(this);
+#endif
+
+#ifdef CONFIG_OBJ_SVN
+        if (SvnDiffView) SvnDiffView->FindFileLines(this);
+#endif
 
         markIndex.retrieveForBuffer(this);
 
@@ -34,19 +58,24 @@ EViewPort *EBuffer::CreateViewPort(EView *V) {
         //printf("setting to c:%d r:%d f:%s", c, r, FileName);
         V->Port->GetPos();
         V->Port->ReCenter = 1;
+
+#ifdef CONFIG_BOOKMARKS
+        if (BFI (this,BFI_SaveBookmarks)==3) RetrieveBookmarks(this);
+#endif
 #endif
     }
     return V->Port;
 }
 
-EEditPort::EEditPort(EBuffer *B, EView *V): EViewPort(V) {
-    Buffer = B;
-    Rows = Cols = 0;
-    OldTP.Row = -1;
-    OldTP.Col = -1;
+EEditPort::EEditPort(EBuffer *B, EView *V) :
+    EViewPort(V),
+    Buffer(B),
+    OldTP(-1, -1),
+    Rows(0),
+    Cols(0)
+{
     GetPos();
-    TP = B->TP;
-    CP = B->CP;
+
     if (V && V->MView && V->MView->Win) {
         V->MView->ConQuerySize(&Cols, &Rows);
         Rows--;
@@ -143,11 +172,11 @@ void EEditPort::HandleEvent(TEvent &Event) {
     case evCommand:
         switch (Event.Msg.Command) {
         case cmVScrollUp:
-            Buffer->ScrollDown(1);
+            Buffer->ScrollDown(Event.Msg.Param1);
             Event.What = evNone;
             break;
         case cmVScrollDown:
-            Buffer->ScrollUp(1);
+            Buffer->ScrollUp(Event.Msg.Param1);
             Event.What = evNone;
             break;
         case cmVScrollPgUp:
@@ -171,11 +200,11 @@ void EEditPort::HandleEvent(TEvent &Event) {
             Event.What = evNone;
             break;
         case cmHScrollLeft:
-            Buffer->ScrollRight(1);
+            Buffer->ScrollRight(Event.Msg.Param1);
             Event.What = evNone;
             break;
         case cmHScrollRight:
-            Buffer->ScrollLeft(1);
+            Buffer->ScrollLeft(Event.Msg.Param1);
             Event.What = evNone;
             break;
         case cmHScrollPgLt:
@@ -199,14 +228,17 @@ void EEditPort::HandleEvent(TEvent &Event) {
             break;
         }
         break;
+#ifdef CONFIG_MOUSE
     case evMouseDown:
     case evMouseMove:
     case evMouseAuto:
     case evMouseUp:
         HandleMouse(Event);
         break;
+#endif
     }
 }
+#ifdef CONFIG_MOUSE
 void EEditPort::HandleMouse(TEvent &Event) {
     int x, y, xx, yy, W, H;
 
@@ -234,6 +266,7 @@ void EEditPort::HandleMouse(TEvent &Event) {
             View->MView->MouseMoved = 0;
 
             if (Event.Mouse.Buttons == 1) {
+                // left mouse button down
                 Buffer->SetNearPos(xx, yy);
                 switch (Event.Mouse.Count % 5) {
                 case 1:
@@ -250,11 +283,13 @@ void EEditPort::HandleMouse(TEvent &Event) {
                 }
                 //            Window->Buffer->Redraw();
                 if (SystemClipboard) {
+                    // note: copy to second clipboard
                     Buffer->NextCommand();
-                    Buffer->BlockCopy(0);
+                    Buffer->BlockCopy(0, 1);
                 }
                 Event.What = evNone;
             } else if (Event.Mouse.Buttons == 2) {
+                // right mouse button down
                 Buffer->SetNearPos(xx, yy);
             }
             break;
@@ -262,6 +297,7 @@ void EEditPort::HandleMouse(TEvent &Event) {
         case evMouseMove:
             if (View->MView->MouseCaptured) {
                 if (Event.Mouse.Buttons == 1) {
+                    // left mouse button move
                     if (!View->MView->MouseMoved) {
                         if (Event.Mouse.KeyMask == kfCtrl) Buffer->BlockMarkColumn();
                         else if (Event.Mouse.KeyMask == kfAlt) Buffer->BlockMarkLine();
@@ -274,6 +310,7 @@ void EEditPort::HandleMouse(TEvent &Event) {
                     Buffer->SetNearPos(xx, yy);
                     Buffer->BlockExtendEnd();
                 } else if (Event.Mouse.Buttons == 2) {
+                    // right mouse button move
                     if (Event.Mouse.KeyMask == kfAlt) {
                     } else {
                         Buffer->SetNearPos(xx, yy);
@@ -295,13 +332,16 @@ void EEditPort::HandleMouse(TEvent &Event) {
                 break;
             View->MView->MouseCaptured = 0;
             if (Event.Mouse.Buttons == 1) {
+                // left mouse button up
                 if (View->MView->MouseMoved)
                     if (SystemClipboard) {
+                        // note: copy to second clipboard
                         Buffer->NextCommand();
-                        Buffer->BlockCopy(0);
+                        Buffer->BlockCopy(0, 1);
                     }
             }
             if (Event.Mouse.Buttons == 2) {
+                // right mouse button up
                 if (!View->MView->MouseMoved) {
                     EEventMap *Map = View->MView->Win->GetEventMap();
                     const char *MName = 0;
@@ -314,14 +354,16 @@ void EEditPort::HandleMouse(TEvent &Event) {
                 }
             }
             if (Event.Mouse.Buttons == 4) {
+                // middle mouse button up
                 if (SystemClipboard) {
+                    // note: copy to second clipboard
                     Buffer->NextCommand();
                     if (Event.Mouse.KeyMask == 0)
-                        Buffer->BlockPasteStream();
+                        Buffer->BlockPasteStream(1);
                     else if (Event.Mouse.KeyMask == kfCtrl)
-                        Buffer->BlockPasteColumn();
+                        Buffer->BlockPasteColumn(1);
                     else if (Event.Mouse.KeyMask == kfAlt)
-                        Buffer->BlockPasteLine();
+                        Buffer->BlockPasteLine(1);
                 }
             }
             Event.What = evNone;
@@ -329,6 +371,7 @@ void EEditPort::HandleMouse(TEvent &Event) {
         }
     }
 }
+#endif
 
 void EEditPort::UpdateView() {
     Buffer->Redraw();
@@ -353,7 +396,8 @@ int EBuffer::BeginMacro() {
     return NextCommand();
 }
 
-int EBuffer::ExecCommand(int Command, ExState &State) {
+// *INDENT-OFF*
+int EBuffer::ExecCommand(ExCommands Command, ExState &State) {
     switch (Command) {
     case ExMoveUp:                return MoveUp();
     case ExMoveDown:              return MoveDown();
@@ -462,8 +506,26 @@ int EBuffer::ExecCommand(int Command, ExState &State) {
     case ExClipClear:             return ClipClear();
     case ExBlockPaste:            return BlockPaste();
     case ExBlockKill:             return BlockKill();
-    case ExBlockIndent:           return BlockIndent();
-    case ExBlockUnindent:         return BlockUnindent();
+    case ExBlockIndent:
+        {
+            int saved_persistence, ret_code;
+
+            saved_persistence = BFI(this, BFI_PersistentBlocks);
+            BFI_SET(this, BFI_PersistentBlocks, 1);
+            ret_code = BlockIndent();
+            BFI_SET(this, BFI_PersistentBlocks, saved_persistence);
+            return ret_code;
+        }
+    case ExBlockUnindent:
+        {
+            int saved_persistence, ret_code;
+
+            saved_persistence = BFI(this, BFI_PersistentBlocks);
+            BFI_SET(this, BFI_PersistentBlocks, 1);
+            ret_code = BlockUnindent();
+            BFI_SET(this, BFI_PersistentBlocks, saved_persistence);
+            return ret_code;
+        }
     case ExBlockClear:            return BlockClear();
     case ExBlockMarkStream:       return BlockMarkStream();
     case ExBlockMarkLine:         return BlockMarkLine();
@@ -496,6 +558,7 @@ int EBuffer::ExecCommand(int Command, ExState &State) {
     case ExBlockPasteStream:      return BlockPasteStream();
     case ExBlockPasteLine:        return BlockPasteLine();
     case ExBlockPasteColumn:      return BlockPasteColumn();
+    case ExBlockPasteOver:        return BlockPasteOver();
     case ExShowPosition:          return ShowPosition();
     case ExFoldCreate:            return FoldCreate(VToR(CP.Row));
     case ExFoldDestroy:           return FoldDestroy(VToR(CP.Row));
@@ -542,6 +605,9 @@ int EBuffer::ExecCommand(int Command, ExState &State) {
     case ExToggleWordWrap:        return ToggleWordWrap();
     case ExToggleTrim:            return ToggleTrim();
     case ExToggleShowMarkers:     return ToggleShowMarkers();
+    case ExToggleHilitTags:       return ToggleHilitTags();
+    case ExToggleShowBookmarks:   return ToggleShowBookmarks();
+    case ExToggleMakeBackups:     return ToggleMakeBackups();
     case ExSetLeftMargin:         return SetLeftMargin();
     case ExSetRightMargin:        return SetRightMargin();
     case ExSetIndentWithTabs:     return SetIndentWithTabs(State);
@@ -618,9 +684,12 @@ int EBuffer::ExecCommand(int Command, ExState &State) {
     case ExInsertDate:          return InsertDate(State);
     case ExInsertUid:           return InsertUid();
     case ExShowHelpWord:        return ShowHelpWord(State);
+    default:
+        ;
     }
     return EModel::ExecCommand(Command, State);
 }
+// *INDENT-ON*
 
 void EBuffer::HandleEvent(TEvent &Event) {
     EModel::HandleEvent(Event);
@@ -663,6 +732,79 @@ int EBuffer::FoldCreateByRegexp(ExState &State) {
 }
 
 #ifdef CONFIG_BOOKMARKS
+int EBuffer::PlaceUserBookmark(const char *n,EPoint P) {
+    char name[256+4] = "_BMK";
+    int result;
+    EPoint prev;
+
+    strcpy (name+4,n);
+    if (GetBookmark (name,prev)==0) {
+        prev.Row=-1;prev.Col=-1;
+    }
+    result=PlaceBookmark(name, P);
+    if (result) {
+        if (BFI (this,BFI_ShowBookmarks)) {
+            FullRedraw ();
+        }
+        if (BFI (this,BFI_SaveBookmarks)==1||BFI (this,BFI_SaveBookmarks)==2) {
+            if (!Modify ()) return result;   // Never try to save to read-only
+#ifdef CONFIG_UNDOREDO
+            if (BFI(this, BFI_Undo)) {
+                if (PushULong(prev.Row) == 0) return 0;
+                if (PushULong(prev.Col) == 0) return 0;
+                if (PushUData(n, strlen(n) + 1) == 0) return 0;
+                if (PushULong(strlen (n)+1) == 0) return 0;
+                if (PushUChar(ucPlaceUserBookmark) == 0) return 0;
+            }
+#endif
+        }
+    }
+    return result;
+}
+
+int EBuffer::RemoveUserBookmark(const char *n) {
+    char name[256+4] = "_BMK";
+    int result;
+    EPoint p;
+
+    strcpy (name+4,n);
+    GetBookmark (name,p);       // p is valid only if remove is successful
+    result=RemoveBookmark(name);
+    if (result) {
+        if (BFI (this,BFI_ShowBookmarks)) {
+            FullRedraw ();
+        }
+        if (BFI (this,BFI_SaveBookmarks)==1||BFI (this,BFI_SaveBookmarks)==2) {
+            if (!Modify ()) return result;   // Never try to save to read-only
+#ifdef CONFIG_UNDOREDO
+            if (PushULong(p.Row) == 0) return 0;
+            if (PushULong(p.Col) == 0) return 0;
+            if (PushUData((void *)n,strlen (n)+1) == 0) return 0;
+            if (PushULong(strlen (n)+1) == 0) return 0;
+            if (PushUChar(ucRemoveUserBookmark) == 0) return 0;
+#endif
+        }
+    }
+    return result;
+}
+
+int EBuffer::GotoUserBookmark(const char *n) {
+    char name[256+4] = "_BMK";
+
+    strcpy (name+4,n);
+    return GotoBookmark(name);
+}
+
+int EBuffer::GetUserBookmarkForLine(int searchFrom, int searchForLine, const EBookmark* &eb) {
+    int i = searchFrom;
+
+    while ((i = GetBookmarkForLine(i, searchForLine, eb) != -1))
+        if (strncmp(eb->GetName(), "_BMK", 4) == 0)
+            return i;
+
+    return -1;
+}
+
 int EBuffer::PlaceBookmark(ExState &State) {
     char name[256] = "";
     EPoint P = CP;
@@ -671,7 +813,7 @@ int EBuffer::PlaceBookmark(ExState &State) {
 
     if (State.GetStrParam(View, name, sizeof(name)) == 0)
         if (View->MView->Win->GetStr("Place Bookmark", sizeof(name), name, HIST_BOOKMARK) == 0) return 0;
-    return PlaceBookmark(name, P);
+    return PlaceUserBookmark(name, P);
 }
 
 int EBuffer::RemoveBookmark(ExState &State) {
@@ -679,7 +821,7 @@ int EBuffer::RemoveBookmark(ExState &State) {
 
     if (State.GetStrParam(View, name, sizeof(name)) == 0)
         if (View->MView->Win->GetStr("Remove Bookmark", sizeof(name), name, HIST_BOOKMARK) == 0) return 0;
-    return RemoveBookmark(name);
+    return RemoveUserBookmark(name);
 }
 
 int EBuffer::GotoBookmark(ExState &State) {
@@ -687,7 +829,7 @@ int EBuffer::GotoBookmark(ExState &State) {
 
     if (State.GetStrParam(View, name, sizeof(name)) == 0)
         if (View->MView->Win->GetStr("Goto Bookmark", sizeof(name), name, HIST_BOOKMARK) == 0) return 0;
-    return GotoBookmark(name);
+    return GotoUserBookmark(name);
 }
 #endif
 
@@ -711,7 +853,7 @@ int EBuffer::PushGlobalBookmark() {
     P.Row = VToR(P.Row);
     EMark *m = markIndex.pushMark(this, P);
     if (m)
-         Msg(S_INFO, "Placed bookmark %s", m->getName());
+         Msg(S_INFO, "Placed bookmark %s", m->GetName());
     return m ? 1 : 0;
 }
 
@@ -785,10 +927,10 @@ int EBuffer::FileReload(ExState &/*State*/) {
     return Reload();
 }
 
-int EBuffer::FileSaveAs(char *FName) {
+int EBuffer::FileSaveAs(const char *FName) {
     char Name[MAXPATH];
 
-    if (ExpandPath(FName, Name) == -1) {
+    if (ExpandPath(FName, Name, sizeof(Name)) == -1) {
         View->MView->Win->Choice(GPC_ERROR, "Error", 1, "O&K", "Invalid path: %s.", FName);
         return 0;
     }
@@ -829,10 +971,10 @@ int EBuffer::FileSaveAs(ExState &State) {
     return FileSaveAs(FName);
 }
 
-int EBuffer::FileWriteTo(char *FName) {
+int EBuffer::FileWriteTo(const char *FName) {
     char Name[MAXPATH];
 
-    if (ExpandPath(FName, Name) == -1) {
+    if (ExpandPath(FName, Name, sizeof(Name)) == -1) {
         View->MView->Win->Choice(GPC_ERROR, "Error", 1, "O&K", "Invalid path: %s.", FName);
         return 0;
     }
@@ -872,12 +1014,12 @@ int EBuffer::BlockReadX(ExState &State, int blockMode) {
     char Name[MAXPATH];
     char FName[MAXPATH];
 
-    if (JustDirectory(FileName, FName) == -1) return 0;
+    if (JustDirectory(FileName, FName, sizeof(FName)) == -1) return 0;
     SlashDir(FName);
     if (State.GetStrParam(View, FName, sizeof(FName)) == 0)
         if (View->MView->Win->GetFile("Read block", sizeof(FName), FName, HIST_PATH, GF_OPEN) == 0) return 0;
 
-    if (ExpandPath(FName, Name) == -1) {
+    if (ExpandPath(FName, Name, sizeof(Name)) == -1) {
         View->MView->Win->Choice(GPC_ERROR, "Error", 1, "O&K", "Invalid path: %s.", FName);
         return 0;
     }
@@ -905,13 +1047,13 @@ int EBuffer::BlockWrite(ExState &State) {
     char FName[MAXPATH];
     int Append = 0;
 
-    if (JustDirectory(FileName, FName) == -1) return 0;
+    if (JustDirectory(FileName, FName, sizeof(FName)) == -1) return 0;
     SlashDir(FName);
     if (State.GetStrParam(View, FName, sizeof(FName)) == 0)
         if (View->MView->Win->GetFile("Write block", sizeof(FName), FName, HIST_PATH, GF_SAVEAS) == 0)
             return 0;
 
-    if (ExpandPath(FName, Name) == -1) {
+    if (ExpandPath(FName, Name, sizeof(Name)) == -1) {
         View->MView->Win->Choice(GPC_ERROR, "Error", 1, "O&K", "Invalid path: %s.", FName);
         return 0;
     }
@@ -1052,14 +1194,10 @@ int EBuffer::FindRepeat(ExState &State) {
 int EBuffer::FindRepeatReverse(ExState &State) {
     int rc;
 
-    if (LSearch.ok == 0) { // ???
-        LSearch.Options ^= SEARCH_BACK;
-        rc = Find(State);
-        LSearch.Options ^= SEARCH_BACK;
-        return rc;
-    }
+    if (LSearch.ok == 0) return Find(State);
     LSearch.Options |= SEARCH_NEXT;
     LSearch.Options &= ~SEARCH_GLOBAL;
+    LSearch.Options ^= SEARCH_BACK;
     rc = Find(LSearch);
     LSearch.Options ^= SEARCH_BACK;
     return rc;
@@ -1239,37 +1377,42 @@ int EBuffer::ConfQuit(GxView *V, int multiFile) {
     return 1;
 }
 
-void EBuffer::GetName(char *AName, int MaxLen) {
-    strncpy(AName, FileName, MaxLen);
-    AName[MaxLen - 1] = 0;
+void EBuffer::GetName(char *AName, size_t MaxLen) {
+    strlcpy(AName, FileName, MaxLen);
 }
 
-void EBuffer::GetPath(char *APath, int MaxLen) {
-    JustDirectory(FileName, APath);
+void EBuffer::GetPath(char *APath, size_t MaxLen) {
+    JustDirectory(FileName, APath, MaxLen);
 }
 
-void EBuffer::GetInfo(char *AInfo, int MaxLen) {
+void EBuffer::GetInfo(char *AInfo, size_t /*MaxLen*/) {
+    char buf[256] = {0};
+    char winTitle[256] = {0};
+
+    JustFileName(FileName, buf, sizeof(buf));
+    if (buf[0] == '\0') // if there is no filename, try the directory name.
+        JustLastDirectory(FileName, buf, sizeof(buf));
+
+    if (buf[0] != 0) // if there is a file/dir name, stick it in here.
+    {
+        strlcat(winTitle, buf, sizeof(winTitle));
+        strlcat(winTitle, " - ", sizeof(winTitle));
+    }
+    strlcat(winTitle, FileName, sizeof(winTitle));
+
     sprintf(AInfo,
             "%2d %04d:%03d%c%-150s ",
             ModelNo,
             1 + CP.Row, 1 + CP.Col,
             Modified ? '*': ' ',
-            FileName);
+            winTitle);
 }
 
-void EBuffer::GetTitle(char *ATitle, int MaxLen, char *ASTitle, int SMaxLen) {
-    char *p;
+void EBuffer::GetTitle(char *ATitle, size_t MaxLen, char *ASTitle, size_t SMaxLen) {
 
-    strncpy(ATitle, FileName, MaxLen - 1);
-    ATitle[MaxLen - 1] = 0;
-    p = SepRChr(FileName);
-    if (p) {
-        strncpy(ASTitle, p + 1, SMaxLen - 1);
-        ASTitle[SMaxLen - 1] = 0;
-    } else {
-        strncpy(ASTitle, FileName, SMaxLen - 1);
-        ASTitle[SMaxLen - 1] = 0;
-    }
+    strlcpy(ATitle, FileName, MaxLen);
+    char *p = SepRChr(FileName);
+    strlcpy(ASTitle, (p) ? p + 1 : FileName, SMaxLen);
 }
 
 #ifdef CONFIG_I_ASCII
@@ -1363,19 +1506,22 @@ int EBuffer::InsertDate(ExState &State) {
         p = ctime(&t);
         sprintf(buf, "%.10s %.4s", p, p + 20);
     }
-    puts(buf);
+    //puts(buf);
 
     return InsertString(buf, strlen(buf));
 }
 
 
 int EBuffer::InsertUid() {
-    char *p = getenv("USER");
+    const char *p = getenv("USER");
     if (p == 0) p = getenv("NAME");
     if (p == 0) p = getenv("ID");
+    // mostly for Windows.  Why they can't just be standard, I don't know...
+    if (p == 0) p = getenv("USERNAME");
     if (p == 0) {
         Msg(S_INFO, "User ID not set ($USER).");
-        return 0;
+        //return 0;
+	p = "UNKNOWN USER";
     }
     return InsertString(p, strlen(p));
 }
@@ -1426,8 +1572,7 @@ int EBuffer::ShowHelpWord(ExState &State) {
     return View->SysShowHelp(State, buf[0] ? buf : 0);
 }
 
-int EBuffer::GetStrVar(int var, char *str, int buflen) {
-    assert(buflen >= 0);
+int EBuffer::GetStrVar(int var, char *str, size_t buflen) {
     if (buflen == 0)
         return 0;
     //puts("variable EBuffer\x7");
@@ -1439,50 +1584,140 @@ int EBuffer::GetStrVar(int var, char *str, int buflen) {
         return 1;
 
     case mvFileName:
-        JustFileName(FileName, str);
+        JustFileName(FileName, str, buflen);
         return 1;
 
     case mvFileDirectory:
-        JustDirectory(FileName, str);
+        JustDirectory(FileName, str, buflen);
         return 1;
     case mvFileBaseName:
         {
             char buf[MAXPATH];
             char *dot, *dot2;
 
-            JustFileName(FileName, buf);
+            JustFileName(FileName, buf, sizeof(buf));
 
             dot = strchr(buf, '.');
             while ((dot2 = strchr(dot + 1, '.')) != NULL)
                 dot = dot2;
             if (dot)
                 *dot = 0;
-            strcpy(str, buf);
+            strlcpy(str, buf, buflen);
         }
         return 1;
 
     case mvFileExtension:
         {
             char buf[MAXPATH];
-            char *dot, *dot2;
+            char *dot;
 
-            JustFileName(FileName, buf);
+            JustFileName(FileName, buf, sizeof(buf));
 
-            dot = strchr(buf, '.');
-            while ((dot2 = strchr(dot + 1, '.')) != NULL)
-                dot = dot2;
+            dot = strrchr(buf, '.');
             if (dot)
-                strcpy(str, dot);
+                strlcpy(str, dot, buflen);
             else
                 str[0] = 0;
         }
         return 1;
 
     case mvChar:
+        {
+            PELine L;
+            int P;
+
+            L = RLine(CP.Row);
+            P = CharOffset(L, CP.Col);
+
+            strlcpy(str, "", buflen);
+
+            if (ChClass(L->Chars[P]))
+            {
+                char tmp[2];
+
+                // make copy of character
+                tmp[0] = L->Chars[P];
+                tmp[1] = 0;
+
+                strlcat(str, tmp, buflen);
+            }
+        }
+        return 1;
+
     case mvWord:
+        {
+            PELine L;
+            int P, C;
+            int wordBegin, wordEnd;
+
+            L = RLine(CP.Row);
+            P = CharOffset(L, CP.Col);
+
+            strlcpy(str, "", buflen);
+
+            if (ChClass(L->Chars[P]))
+            {
+                C = ChClassK(L->Chars[P]);
+
+                // search start of word
+                while ((P>0) && (C == ChClassK(L->Chars[P-1]))) P--;
+
+                wordBegin = P;
+
+                // search end of word
+                while ((P< L->Count) && (C == ChClassK(L->Chars[P]))) P++;
+
+                wordEnd = P;
+
+                // calculate total length for buffer copy
+                size_t length = wordEnd - wordBegin;
+
+                if ((length + 1) < buflen)
+                {
+                    length++;
+                } else
+                {
+                    length = buflen;
+                }
+
+                // copy word to buffer
+                strlcpy(str, &L->Chars[wordBegin], length);
+            }
+        }
+        return 1;
+
     case mvLine:
-        return 0;
+        {
+            PELine L;
+
+            L = RLine(CP.Row);
+
+            strlcpy(str, "", buflen);
+
+            if (L->Count > 0)
+            {
+                // calculate total length for buffer copy
+                size_t length = L->Count;
+
+                if ((length + 1) < buflen)
+                {
+                    length++;
+                } else
+                {
+                    length = buflen;
+                }
+
+                // copy word to buffer
+                strlcpy(str, L->Chars, length);
+            }
+        }
+        return 1;
+
+    case mvFTEVer:
+        strlcpy(str, VERSION, buflen);
+        return 1;
     }
+
     return EModel::GetStrVar(var, str, buflen);
 }
 
