@@ -7,15 +7,37 @@
  *
  */
 
-#include "fte.h"
-#include "s_files.h"
-#include "c_commands.h"
+#include "o_messages.h"
 
 #ifdef CONFIG_OBJ_MESSAGES
+
+#include "c_commands.h"
+#include "c_config.h"
+#include "i_view.h"
+#include "o_buflist.h"
+#include "s_files.h"
+#include "s_string.h"
+#include "s_util.h"
+
+#include <stdio.h>
+
 #define MAXREGEXP  32
 
 EMessages *CompilerMsgs = 0;
 
+struct Error {
+    EBuffer *Buf;
+    StlString file;
+    StlString msg;
+    StlString text;
+    int line;
+    int hilit;
+
+    Error(const char *fn, int ln, const char *mesg, const char *txt, int hlt) :
+	Buf(0), file(fn), msg(mesg), text(txt), line(ln), hilit(hlt)
+    {}
+};
+
 static int NCRegexp = 0;
 static struct {
     int RefFile;
@@ -25,6 +47,7 @@ static struct {
 } CRegexp[MAXREGEXP];
 
 int AddCRegexp(int file, int line, int msg, const char *regexp) {
+    //fprintf(stderr, "ADD EXP %d: %s   %d %d %d\n", NCRegexp, regexp, file, line, msg);
     if (NCRegexp >= MAXREGEXP) return 0;
     CRegexp[NCRegexp].RefFile = file;
     CRegexp[NCRegexp].RefLine = line;
@@ -39,78 +62,59 @@ int AddCRegexp(int file, int line, int msg, const char *regexp) {
 void FreeCRegexp()
 {
     while(NCRegexp--)
-    {
         RxFree(CRegexp[NCRegexp].rx);
-    }
 }
 
-EMessages::EMessages(int createFlags, EModel **ARoot, char *ADir, char *ACommand): EList(createFlags, ARoot, "Messages") {
+EMessages::EMessages(int createFlags, EModel **ARoot, const char *ADir, const char *ACommand) :
+    EList(createFlags, ARoot, "Messages"),
+    Running(1),
+    BufLen(0),
+    BufPos(0),
+    ReturnCode(-1),
+    MatchCount(0)
+{
     CompilerMsgs = this;
-    ErrCount = 0;
-    ErrList = 0;
-    Running = 0;
-    BufLen = 0;
-    BufPos = 0;
-    Command = 0;
-    Directory = 0;
-    MatchCount = 0;
-    ReturnCode = -1;
-    Running = 1;
-    curr_dir = 0;
     RunPipe(ADir, ACommand);
 }
 
 EMessages::~EMessages() {
     gui->ClosePipe(PipeId);
     FreeErrors();
-    free(Command);
-    free(Directory);
     CompilerMsgs = 0;
-    freeDirStack();
-}
-
-void EMessages::freeDirStack()
-{
-    while(curr_dir != 0)
-    {
-        aDir *a = curr_dir;
-        curr_dir = curr_dir->next;
-        free(a->name);
-        delete a;
-    }
 }
 
 void EMessages::NotifyDelete(EModel *Deleting) {
-    for (int i = 0; i < ErrCount; i++) {
-        if (ErrList[i]->Buf == Deleting) {
+    vector_iterate(Error*, ErrList, it) {
+        if ((*it)->Buf == Deleting) {
             /* NOT NEEDED!
              char bk[16];
              sprintf(bk, "_MSG.%d", i);
              ((EBuffer *)Deleting)->RemoveBookmark(bk);
              */
-            ErrList[i]->Buf = 0;
+            (*it)->Buf = 0;
         }
     }
 }
 
 void EMessages::FindErrorFiles() {
-    for (int i = 0; i < ErrCount; i++)
-        if (ErrList[i]->Buf == 0 && ErrList[i]->file != 0)
-            FindErrorFile(i);
+    unsigned i = 0;
+    vector_iterate(Error*, ErrList, it) {
+        if ((*it)->Buf == 0 && (*it)->file.empty())
+	    FindErrorFile(i);
+        ++i;
+    }
 }
 
-void EMessages::FindErrorFile(int err) {
-    assert(err >= 0 && err < ErrCount);
-    if (ErrList[err]->file == 0)
-        return ;
-    
-    EBuffer *B;
+void EMessages::FindErrorFile(unsigned err) {
+    assert(err < ErrList.size());
+    if (ErrList[err]->file.empty())
+        return;
 
     ErrList[err]->Buf = 0;
-    
-    B = FindFile(ErrList[err]->file);
+
+    EBuffer *B = FindFile(ErrList[err]->file.c_str());
     if (B == 0)
-        return ;
+        return;
 
     if (B->Loaded == 0)
         return;
@@ -118,16 +122,13 @@ void EMessages::FindErrorFile(int err) {
     AddFileError(B, err);
 }
 
-void EMessages::AddFileError(EBuffer *B, int err) {
+void EMessages::AddFileError(EBuffer *B, unsigned err) {
+
+    assert(err < ErrList.size());
+
     char bk[16];
-    EPoint P;
-
-    assert(err >= 0 && err < ErrCount);
-
     sprintf(bk, "_MSG.%d", err);
-    P.Col = 0;
-    P.Row = ErrList[err]->line - 1; // offset 0
-
+    EPoint P(ErrList[err]->line - 1, 0);
 
     if (P.Row >= B->RCount)
         P.Row = B->RCount - 1;
@@ -139,45 +140,39 @@ void EMessages::AddFileError(EBuffer *B, int err) {
 }
 
 void EMessages::FindFileErrors(EBuffer *B) {
-    for (int i = 0; i < ErrCount; i++)
-        if (ErrList[i]->Buf == 0 && ErrList[i]->file != 0) {
-            if (filecmp(B->FileName, ErrList[i]->file) == 0) {
+    unsigned i = 0;
+    vector_iterate(Error*, ErrList, it) {
+        if ((*it)->Buf == 0 && !(*it)->file.empty()) {
+            if (filecmp(B->FileName, (*it)->file.c_str()) == 0)
                 AddFileError(B, i);
-            }
-        }
+	}
+	++i;
+    }
 }
 
-int EMessages::RunPipe(char *ADir, char *ACommand) {
+int EMessages::RunPipe(const char *ADir, const char *ACommand) {
     if (!KeepMessages)
         FreeErrors();
     
-    free(Command);
-    free(Directory);
-        
-    Command = strdup(ACommand);
-    Directory = strdup(ADir);
+    Command = ACommand;
+    Directory = ADir;
     
     MatchCount = 0;
     ReturnCode = -1;
     Running = 1;
     BufLen = BufPos = 0;
-    Row = ErrCount - 1;
+    Row = (int)ErrList.size() - 1;
 
-    {
-        char s[2 * MAXPATH * 4];
+    char s[2 * MAXPATH * 4];
 
-        sprintf(s, "[running '%s' in '%s']", Command, Directory);
-        AddError(0, -1, 0, s);
-    }
+    sprintf(s, "[running '%s' in '%s']", ACommand, ADir);
+    AddError(0, -1, 0, s);
 
-    {
-        char s[MAXPATH * 2];
-        sprintf(s, "Messages [%s]: %s", Directory, Command);
-        SetTitle(s);
-    }
+    sprintf(s, "Messages [%s]: %s", ADir, ACommand);
+    SetTitle(s);
     
-    ChangeDir(Directory);
-    PipeId = gui->OpenPipe(Command, this);
+    ChangeDir(ADir);
+    PipeId = gui->OpenPipe(ACommand, this);
     return 0;
 }
 
@@ -185,7 +180,7 @@ EEventMap *EMessages::GetEventMap() {
     return FindEventMap("MESSAGES");
 }
 
-int EMessages::ExecCommand(int Command, ExState &State) {
+int EMessages::ExecCommand(ExCommands Command, ExState &State) {
     switch (Command) {
     case ExChildClose:
         if (Running == 0 || PipeId == -1)
@@ -193,80 +188,56 @@ int EMessages::ExecCommand(int Command, ExState &State) {
         ReturnCode = gui->ClosePipe(PipeId);
         PipeId = -1;
         Running = 0;
-        {
-            char s[30];
-            
-            sprintf(s, "[aborted, status=%d]", ReturnCode);
-            AddError(0, -1, 0, s);
-        }
+        char s[30];
+        sprintf(s, "[aborted, status=%d]", ReturnCode);
+        AddError(0, -1, 0, s);
         return ErOK;
-        
     case ExActivateInOtherWindow:
         ShowError(View->Next, Row);
         return ErOK;
+    case ExFind:
+        fprintf(stderr, "FIND\n");
+        return ErOK;
+    default:
+        ;
     }
     return EList::ExecCommand(Command, State);
 }
 
-void EMessages::AddError(Error *p) {
-    ErrCount++;
-    ErrList = (Error **) realloc(ErrList, sizeof(void *) * ErrCount);
-    ErrList[ErrCount - 1] = p;
-    ErrList[ErrCount - 1]->Buf = 0;
-    FindErrorFile(ErrCount - 1);
+void EMessages::AddError(const char *file, int line, const char *msg, const char *text, int hilit) {
+    ErrList.push_back(new Error(file, line, msg, text, hilit));
+    //fprintf(stderr, "Error %s %d %s %s\n", file, line, msg, text);
+    FindErrorFile((unsigned)ErrList.size() - 1);
 
-    if (ErrCount > Count)
-        if (Row >= Count - 1) {
-            //if (ErrCount > 1 && !ErrList[TopRow]->file)
-                Row = ErrCount - 1;
-        }
+    if ((int)ErrList.size() > Count)
+        if (Row >= Count - 1)
+	    Row = (int)ErrList.size() - 1;
 
     UpdateList();
 }
 
-void EMessages::AddError(char *file, int line, char *msg, const char *text, int hilit) {
-    Error *pe;
-
-    pe = (Error *) malloc(sizeof(Error));
-    if (pe == 0)
-        return ;
-
-    pe->file = file ? strdup(file) : 0;
-    pe->line = line;
-    pe->msg = msg ? strdup(msg) : 0;
-    pe->text = text ? strdup(text) : 0;
-    pe->hilit = hilit;
-
-    AddError(pe);
-}
-
 void EMessages::FreeErrors() {
-    if (ErrList) {
-        for (int i = 0; i < ErrCount; i++) {
-            if (ErrList[i]->Buf != 0) {
-                char bk[16];
-                sprintf(bk, "_MSG.%d", i);
-                ((EBuffer *)(ErrList[i]->Buf))->RemoveBookmark(bk);
-            }
-            free(ErrList[i]->msg);
-            free(ErrList[i]->text);
-            free(ErrList[i]->file);
-            free(ErrList[i]);
-        }
-        free(ErrList);
+    unsigned i = 0;
+    vector_iterate(Error*, ErrList, it) {
+	if ((*it)->Buf != 0) {
+	    char bk[16];
+	    sprintf(bk, "_MSG.%d", i);
+	    (*it)->Buf->RemoveBookmark(bk);
+	}
+	delete *it;
+        i++;
     }
-    ErrCount = 0;
-    ErrList = 0;
+    ErrList.clear();
     BufLen = BufPos = 0;
 }
 
-int EMessages::GetLine(char *Line, int maxim) {
-    int rc;
+int EMessages::GetLine(char *Line, size_t maxim) {
+    ssize_t rc;
     char *p;
     int l;
     
     //fprintf(stderr, "GetLine: %d\n", Running);
-    
+
     *Line = 0;
     if (Running && PipeId != -1) {
         rc = gui->ReadPipe(PipeId, MsgBuf + BufLen, sizeof(MsgBuf) - BufLen);
@@ -286,7 +257,8 @@ int EMessages::GetLine(char *Line, int maxim) {
     p = (char *)memchr(MsgBuf + BufPos, '\n', l);
     if (p) {
         *p = 0;
-        strcpy(Line, MsgBuf + BufPos);
+        UnEscStr(Line, maxim, MsgBuf + BufPos, p - (MsgBuf + BufPos));
+        //strcpy(Line, MsgBuf + BufPos);
         l = strlen(Line);
         if (l > 0 && Line[l - 1] == '\r')
             Line[l - 1] = 0;
@@ -301,7 +273,8 @@ int EMessages::GetLine(char *Line, int maxim) {
     } else {
         if (l == 0) 
             return 0;
-        memcpy(Line, MsgBuf + BufPos, l);
+        UnEscStr(Line, maxim, MsgBuf + BufPos, l);
+        //memcpy(Line, MsgBuf + BufPos, l);
         Line[l] = 0;
         if (l > 0 && Line[l - 1] == '\r')
             Line[l - 1] = 0;
@@ -316,7 +289,7 @@ int EMessages::GetLine(char *Line, int maxim) {
 }
 
 
-static void getWord(char* dest, char*& pin)
+static void getWord(char* dest, const char* pin)
 {
     char *pout, *pend;
     char ch, ec;
@@ -364,46 +337,61 @@ void EMessages::GetErrors() {
     char fn[256];
     
     //fprintf(stderr, "Reading pipe\n");
-    while (GetLine((char *)line, sizeof(line))) {
-        if (strlen(line) > 0 && line[strlen(line)-1] == '\n') line[strlen(line)-1] = 0;
+    while (GetLine(line, sizeof(line))) {
+        size_t len = strlen(line);
+        if (len > 0 && line[len - 1] == '\n')
+	    line[--len] = 0;
         didmatch = 0;
         for (i = 0; i < NCRegexp; i++) {
-            if (RxExec(CRegexp[i].rx, line, strlen(line), line, &RM) == 1) {
-                char ln[256] = "";
-                char msg[256] = "";
-                char fn1[256] = "";
-                char fn2[256] = "";
-                char *file = 0;
-                *fn = 0;
-                
-                memset(fn, 0, 256);
-                memset(ln, 0, 256);
-                memset(msg, 0, 256);
-                n = CRegexp[i].RefFile;
-                if (RM.Close[n] - RM.Open[n] < 256)
-                    memcpy(fn, line + RM.Open[n], RM.Close[n] - RM.Open[n]);
-                n = CRegexp[i].RefLine;
-                if (RM.Close[n] - RM.Open[n] < 256) 
-                    memcpy(ln, line + RM.Open[n], RM.Close[n] - RM.Open[n]);
-                n = CRegexp[i].RefMsg;
-                if (RM.Close[n] - RM.Open[n] < 256)
-                    memcpy(msg, line + RM.Open[n], RM.Close[n] - RM.Open[n]);
+            if (RxExec(CRegexp[i].rx, line, len, line, &RM) == 1) {
+		char ln[256];
+		char msg[256];
+		char fn1[256];
+		char fn2[256];
+		char *file;
 
-                if (IsFullPath(fn))
-                    file = fn;
-                else {
-                    if (curr_dir == 0)
-                        strcpy(fn1, Directory);
-                    else
-                        strcpy(fn1, curr_dir->name);
+		n = CRegexp[i].RefFile;
+		unsigned s = RM.Close[n] - RM.Open[n];
+		if (s < sizeof(fn))
+		    memcpy(fn, line + RM.Open[n], s);
+		else
+		    s = 0;
+		fn[s] = 0;
+
+		n = CRegexp[i].RefLine;
+		s = RM.Close[n] - RM.Open[n];
+		if (s < sizeof(ln))
+		    memcpy(ln, line + RM.Open[n], s);
+		else
+		    s = 0;
+		ln[s] = 0;
+
+		n = CRegexp[i].RefMsg;
+		s = RM.Close[n] - RM.Open[n];
+		if (s < sizeof(msg))
+		    memcpy(msg, line + RM.Open[n], s);
+		else
+		    s = 0;
+		msg[s] = 0;
+		//fprintf(stderr, "File:%s msg:%s rex:%d c:%d o:%d>%s<8\nTXT:%s\n", fn, ln, i, RM.Close[n], RM.Open[n], msg, line);
+		if (IsFullPath(fn))
+		    file = fn;
+		else {
+		    /*
+		     * for now - try only with top most dir
+		     * later we might try to find the file in all stacked dirs
+		     * as with parallel makes it's hard to guess the right directory path
+                     */
+		    strlcpy(fn1, DirLevel.size() ? DirLevel.back().c_str() : Directory.c_str(),
+			    sizeof(fn1));
                     Slash(fn1, 1);
-                    strcat(fn1, fn);
-                    if (ExpandPath(fn1, fn2) == 0)
+                    strlcat(fn1, fn, sizeof(fn1));
+                    if (ExpandPath(fn1, fn2, sizeof(fn2)) == 0)
                         file = fn2;
                     else
                         file = fn1;
                 }
-                AddError(file, atoi(ln), msg[0] ? msg : 0, line, 1);
+                AddError(file, atoi(ln), msg, line, 1);
                 didmatch = 1;
                 MatchCount++;
                 break;
@@ -415,17 +403,14 @@ void EMessages::GetErrors() {
             //** Quicky: check for gnumake 'entering directory'
             //** make[x]: entering directory `xxx'
             //** make[x]: leaving...
-            static char t1[] = "entering directory";
-            static char t2[] = "leaving directory";
-            char*   pin;
+            static const char t1[] = "entering directory";
+            static const char t2[] = "leaving directory";
+            const char *pin;
             
             if ( (pin = strstr(line, "]:")) != 0)
             {
                 //** It *is* some make line.. Check for 'entering'..
                 pin += 2;
-                
-                //while(*pin != ':' && *pin != 0) pin++;
-                //pin++;
                 while (*pin == ' ')
                     pin++;
                 if (strnicmp(pin, t1, sizeof(t1)-1) == 0) {  // Entering?
@@ -433,44 +418,34 @@ void EMessages::GetErrors() {
                     pin += sizeof(t1)-1;
                     getWord(fn, pin);
                     //dbg("entering %s", fn);
-                    
-                    if (*fn) {
-                        //** Indeedy entering directory! Link in list,
-                        aDir * a = new aDir;
-                        assert(a != 0);
-                        a->name= strdup(fn);
-                        assert(a->name != 0);
-                        a->next = curr_dir;
-                        curr_dir = a;
-                    }
+
+		    if (*fn)
+                        //** Indeed entering directory! Link in list,
+			DirLevel.push_back(fn);
                 } else if (strnicmp(pin, t2, sizeof(t2)-1) == 0) {  // Leaving?
                     pin += sizeof(t2)-1;
                     getWord(fn, pin);                   // Get dirname,
                     //dbg("leaving %s", fn);
-                    
-                    aDir *a;
-                    
-                    a = curr_dir;
-                    if (a != 0)
-                        curr_dir = curr_dir->next;       // Remove from stack,
-                    if (a != 0 && stricmp(a->name, fn) == 0) {
-                        //** Pop this item.
-                        free(a->name);
-                        delete a;
-                    } else {
+		    int found = 0;
+		    for (unsigned i = DirLevel.size(); i-- > 0;) {
+			/*
+			 * remove leaved director from our list of Dirs
+			 * as many users runs make in parallel mode
+			 * we might get pretty mangled order of dirs
+                         * so remove the last added with the same name
+                         */
+			if (stricmp(DirLevel[i].c_str(), fn) == 0) {
+			    DirLevel.erase(DirLevel.begin() + i);
+                            found++;
+			    break;
+			}
+		    }
+		    if (!found) {
                         //** Mismatch filenames -> error, and revoke stack.
                         //dbg("mismatch on %s", fn);
                         AddError(0, -1, 0, "fte: mismatch in directory stack!?");
-                        
-                        //** In this case we totally die the stack..
-                        while(a != 0)
-                        {
-                            free(a->name);
-                            delete a;
-			    a = curr_dir;
-                            if (a != 0)
-				curr_dir = curr_dir->next;
-                        }
+			//** In this case we totally die the stack..
+                        DirLevel.clear();
                     }
                 }
             }
@@ -488,57 +463,51 @@ void EMessages::GetErrors() {
 }
 
 int EMessages::CompilePrevError(EView *V) {
-    if (ErrCount > 0) {
-        bad:
-        if (Row > 0) {
-            Row--;
-            if (ErrList[Row]->line == -1 || !ErrList[Row]->file) goto bad;
-            ShowError(V, Row);
-        } else {
-            V->Msg(S_INFO, "No previous error.");
-            return 0;
-        }
-    } else {
+    if (!ErrList.size()) {
         V->Msg(S_INFO, "No errors.");
         return 0;
     }
-    return 1;
+
+    while (Row > 0) {
+        Row--;
+        if (ErrList[Row]->line != -1 && !ErrList[Row]->file.empty()) {
+            ShowError(V, Row);
+            return 1;
+        }
+    }
+
+    V->Msg(S_INFO, "No previous error.");
+    return 0;
 }
 
 int EMessages::CompileNextError(EView *V) {
-    if (ErrCount > 0) {
-        bad:        
-        if (Row < ErrCount - 1) {
-            Row++;
-            if (ErrList[Row]->line == -1 || !ErrList[Row]->file) goto bad;
-            ShowError(V, Row);
-        } else {
-            if (Running)
-                V->Msg(S_INFO, "No more errors (yet).");
-            else
-                V->Msg(S_INFO, "No more errors.");
-            return 0;
-        }
-    } else { 
-        if (Running) 
-            V->Msg(S_INFO, "No errors (yet).");
-        else
-            V->Msg(S_INFO, "No errors.");
+    if (!ErrList.size()) {
+        V->Msg(S_INFO, (Running) ? "No errors (yet)." : "No errors.");
         return 0;
     }
-    return 1;
+
+    while ((Row + 1) < (int)ErrList.size()) {
+        Row++;
+        if (ErrList[Row]->line != -1 && !ErrList[Row]->file.empty()) {
+            ShowError(V, Row);
+            return 1;
+        }
+    }
+
+    V->Msg(S_INFO, (Running) ? "No more errors (yet)." : "No more errors.");
+    return 0;
 }
 
 int EMessages::Compile(char * /*Command*/) {
     return 0;
 }
 
-void EMessages::ShowError(EView *V, int err) {
-    if (err >= 0 && err < ErrCount) {
-        if (ErrList[err]->file) {
+void EMessages::ShowError(EView *V, unsigned err) {
+    if (err < ErrList.size()) {
+        if (!ErrList[err]->file.empty()) {
             // should check if relative path
             // possibly scan for (gnumake) directory info in output
-            if (ErrList[err]->Buf != 0) {
+            if (ErrList[err]->Buf) {
                 char bk[16];
 
                 V->SwitchToModel(ErrList[err]->Buf);
@@ -546,49 +515,47 @@ void EMessages::ShowError(EView *V, int err) {
                 sprintf(bk, "_MSG.%d", err);
                 ErrList[err]->Buf->GotoBookmark(bk);
             } else {
-                if (FileLoad(0, ErrList[err]->file, 0, V) == 1) {
+                if (FileLoad(0, ErrList[err]->file.c_str(), 0, V) == 1) {
                     V->SwitchToModel(ActiveModel);
                     ((EBuffer *)ActiveModel)->CenterNearPosR(0, ErrList[err]->line - 1);
                 }
             }
-            if (ErrList[err]->msg != 0)
-                V->Msg(S_INFO, "%s", ErrList[err]->msg);
+            if (!ErrList[err]->msg.empty())
+                V->Msg(S_INFO, "%s", ErrList[err]->msg.c_str());
             else
-                V->Msg(S_INFO, "%s", ErrList[err]->text);
+                V->Msg(S_INFO, "%s", ErrList[err]->text.c_str());
         }
     }
 }
 
 void EMessages::DrawLine(PCell B, int Line, int Col, ChColor color, int Width) {
-    if (Line < ErrCount)
-        if (Col < int(strlen(ErrList[Line]->text))) {
+    if (Line < (int)ErrList.size())
+        if (Col < int(ErrList[Line]->text.size())) {
             char str[1024];
-            int len;
+            size_t len;
 
             len = UnTabStr(str, sizeof(str),
-                           ErrList[Line]->text,
-                           strlen(ErrList[Line]->text));
+                           ErrList[Line]->text.c_str(),
+                           ErrList[Line]->text.size());
 
-            if (len > Col)
+            if ((int)len > Col)
                 MoveStr(B, 0, Width, str + Col, color, Width);
         }
 }
 
 char* EMessages::FormatLine(int Line) {
-    char *p;
-    if (Line < ErrCount)
-        p = strdup(ErrList[Line]->text);
-    else
-        p = 0;
-    return p;
+    if (Line < (int)ErrList.size())
+        return strdup(ErrList[Line]->text.c_str());
+
+    return 0;
 }
 
 int EMessages::IsHilited(int Line) {
-    return (Line >= 0 && Line < ErrCount) ? ErrList[Line]->hilit : 0;
+    return (Line >= 0 && Line < (int)ErrList.size()) ? ErrList[Line]->hilit : 0;
 }
 
 void EMessages::UpdateList() {
-    Count = ErrCount;
+    Count = (int)ErrList.size();
     EList::UpdateList();
 }
 
@@ -600,10 +567,10 @@ int EMessages::Activate(int /*No*/) {
 }
 
 int EMessages::CanActivate(int Line) {
-    int ok = 0;
-    if (Line < ErrCount)
-        if (ErrList[Line]->file || ErrList[Line]->line != -1) ok = 1;
-    return ok;
+    //return (Line < (int)ErrList.size());
+    return (Line < (int)ErrList.size()
+            && (!ErrList[Line]->file.empty()
+                || ErrList[Line]->line != -1)) ? 1 : 0;
 }
     
 void EMessages::NotifyPipe(int APipeId) {
@@ -612,40 +579,32 @@ void EMessages::NotifyPipe(int APipeId) {
         GetErrors();
 }
 
-void EMessages::GetName(char *AName, int MaxLen) {
-    strncpy(AName, "Messages", MaxLen);
+void EMessages::GetName(char *AName, size_t MaxLen) {
+    strlcpy(AName, "Messages", MaxLen);
 }
 
-void EMessages::GetInfo(char *AInfo, int /*MaxLen*/) {
-    sprintf(AInfo, 
-            "%2d %04d/%03d Messages: %d (%s) ",
-            ModelNo,
-            Row, Count,
-            MatchCount,
-            Command);
+void EMessages::GetInfo(char *AInfo, size_t /*MaxLen*/) {
+    sprintf(AInfo, "%2d %04d/%03d Messages: %d (%s)",
+            ModelNo,Row, Count, MatchCount, Command.c_str());
 }
 
-void EMessages::GetPath(char *APath, int MaxLen) {
-    strncpy(APath, Directory, MaxLen);
-    APath[MaxLen - 1] = 0;
+void EMessages::GetPath(char *APath, size_t MaxLen) {
+    strlcpy(APath, Directory.c_str(), MaxLen);
     Slash(APath, 0);
 }
 
-void EMessages::GetTitle(char *ATitle, int /*MaxLen*/, char *ASTitle, int SMaxLen) {
-    sprintf(ATitle, "Messages: %s", Command);
-    strncpy(ASTitle, "Messages", SMaxLen);
-    ASTitle[SMaxLen - 1] = 0;
+void EMessages::GetTitle(char *ATitle, size_t MaxLen, char *ASTitle, size_t SMaxLen) {
+    snprintf(ATitle, MaxLen, "Messages: %s", Command.c_str());
+    strlcpy(ASTitle, "Messages", SMaxLen);
 }
 
 // get row length for specified row, used in MoveLineEnd to get actual row length
-int EMessages::GetRowLength(int ARow)
+size_t EMessages::GetRowLength(int ARow)
 {
-    if ((ARow >= 0) && (ARow < ErrCount))
-    {
-	return strlen(ErrList[ARow]->text);
-    }
+    if ((ARow >= 0) && (ARow < (int)ErrList.size()))
+        return ErrList[ARow]->text.size();
 
     return 0;
 }
 
-#endif
+#endif // CONFIG_OBJ_MESSAGES
