@@ -7,28 +7,34 @@
  *
  */
 
-#include "fte.h"
+#include "o_routine.h"
 
 #ifdef CONFIG_OBJ_ROUTINE
-RoutineView::RoutineView(int createFlags, EModel **ARoot, EBuffer *AB): EList(createFlags, ARoot, "Routines") {
-    Buffer = AB;
+
+#include "i_view.h"
+#include "o_buflist.h"
+#include "s_string.h"
+#include "sysdep.h"
+
+#include <stdio.h>
+
+RoutineView::RoutineView(int createFlags, EModel **ARoot, EBuffer *AB) :
+    EList(createFlags, ARoot, "Routines"),
+    Buffer(AB)
+{
     if (Buffer->rlst.Count == 0)
         Buffer->ScanForRoutines();
-    Row = 0;
+
     int Row = Buffer->VToR(Buffer->CP.Row);
     for (int i = Buffer->rlst.Count - 1; i >= 0; --i)
         if (Row >= Buffer->rlst.Lines[i]) {
             Row = i;
             break;
         }
-    {
-        char CTitle[256];
-    
-        sprintf(CTitle, "Routines %s: %d",
-                Buffer->FileName,
-                Buffer->rlst.Count);
-        SetTitle(CTitle);
-    }
+
+    char CTitle[256];
+    snprintf(CTitle, sizeof(CTitle), "Routines %s: %d", Buffer->FileName, Buffer->rlst.Count);
+    SetTitle(CTitle);
 };
 
 RoutineView::~RoutineView() {
@@ -39,13 +45,12 @@ EEventMap *RoutineView::GetEventMap() {
     return FindEventMap("ROUTINES");
 }
 
-int RoutineView::ExecCommand(int Command, ExState &State) {
+int RoutineView::ExecCommand(ExCommands Command, ExState &State) {
     switch (Command) {
     case ExRescan:
         Buffer->ScanForRoutines();
         UpdateList();
         return ErOK;
-
     case ExActivateInOtherWindow:
         if (Row < Buffer->rlst.Count) {
             View->Next->SwitchToModel(Buffer);
@@ -53,32 +58,32 @@ int RoutineView::ExecCommand(int Command, ExState &State) {
             return ErOK;
         }
         return ErFAIL;
-        
     case ExCloseActivate:
         return ErFAIL;
+    default:
+        ;
     }
     return EList::ExecCommand(Command, State);
 }
     
 void RoutineView::DrawLine(PCell B, int Line, int Col, ChColor color, int Width) {
-    if (Buffer->RLine(Buffer->rlst.Lines[Line])->Count > Col) {
+    if ((int)Buffer->RLine(Buffer->rlst.Lines[Line])->Count > Col) {
         char str[1024];
-        int len;
+        size_t len;
 
         len = UnTabStr(str, sizeof(str),
                        Buffer->RLine(Buffer->rlst.Lines[Line])->Chars,
                        Buffer->RLine(Buffer->rlst.Lines[Line])->Count);
-                    
-        if (len > Col)
+
+        if ((int)len > Col)
             MoveStr(B, 0, Width, str + Col, color, len - Col);
     }
 }
 
 char* RoutineView::FormatLine(int Line) {
-    char *p = 0;
     PELine L = Buffer->RLine(Buffer->rlst.Lines[Line]);
     
-    p = (char *) malloc(L->Count + 1);
+    char *p = (char *) malloc(L->Count + 1);
     if (p) {
         memcpy(p, L->Chars, L->Count);
         p[L->Count] = 0;
@@ -87,12 +92,12 @@ char* RoutineView::FormatLine(int Line) {
 }
 
 int RoutineView::Activate(int No) {
-    if (No < Buffer->rlst.Count) {
-        View->SwitchToModel(Buffer);
-        Buffer->CenterPosR(0, Buffer->rlst.Lines[No]);
-        return 1;
-    }
-    return 0;
+    if (No >= Buffer->rlst.Count)
+	return 0;
+
+    View->SwitchToModel(Buffer);
+    Buffer->CenterPosR(0, Buffer->rlst.Lines[No]);
+    return 1;
 }
 
 void RoutineView::RescanList() {
@@ -109,17 +114,18 @@ int RoutineView::GetContext() {
     return CONTEXT_ROUTINES;
 }
 
-void RoutineView::GetName(char *AName, int MaxLen) {
-    strncpy(AName, "Routines", MaxLen);
+void RoutineView::GetName(char *AName, size_t MaxLen) {
+    strlcpy(AName, "Routines", MaxLen);
 }
 
-void RoutineView::GetInfo(char *AInfo, int /*MaxLen*/) {
-    sprintf(AInfo, "%2d %04d/%03d Routines (%s)", ModelNo, Row + 1, Count, Buffer->FileName);
+void RoutineView::GetInfo(char *AInfo, size_t MaxLen) {
+    snprintf(AInfo, MaxLen, "%2d %04d/%03d Routines (%s)",
+	    ModelNo, Row + 1, Count, Buffer->FileName);
 }
 
-void RoutineView::GetTitle(char *ATitle, int /*MaxLen*/, char *ASTitle, int SMaxLen) {
-    sprintf(ATitle, "Routines: %s", Buffer->FileName);
-    strncpy(ASTitle, "Routines", SMaxLen);
-    ASTitle[SMaxLen - 1] = 0;
+void RoutineView::GetTitle(char *ATitle, size_t MaxLen, char *ASTitle, size_t SMaxLen) {
+    snprintf(ATitle, MaxLen, "Routines: %s", Buffer->FileName);
+    strlcpy(ASTitle, "Routines", SMaxLen);
 }
-#endif
+
+#endif // CONFIG_OBJ_ROUTINE
