@@ -9,7 +9,13 @@
 
 // TODO: on OS/2 choose between UNIX and OS/2 style patterns (now OS/2 only)
 // TODO: on OS/2 fetch multiple entries at once and cache them for speed
-#include "fte.h"
+
+#include "s_direct.h"
+
+#include "s_files.h"
+
+#include <sys/stat.h>
+#include <unistd.h>
 
 #ifdef OS2
 #define INCL_BASE
@@ -46,33 +52,33 @@ static int my_stat(const char *name, struct stat *s)
 #endif
 
 
-FileInfo::FileInfo(char *Name, int Type, off_t Size, time_t MTime) {
-    name = new char[strlen (Name) + 1];
-    if (name)
-        strcpy(name, Name);
-    size = Size;
-    type = Type;
-    mtime = MTime;
+FileInfo::FileInfo(const char *Name, int Type, off_t Size, time_t MTime) :
+    name(Name),
+    size(Size),
+    mtime(MTime),
+    type(Type)
+{
 }
 
-FileInfo::~FileInfo() {
-    delete [] name;
+FileInfo::~FileInfo()
+{
 }
 
-FileFind::FileFind(const char *aDirectory, const char *aPattern, int aFlags) {
-    Directory = new char[strlen(aDirectory) + 1];
-    if (Directory) {
-        strcpy(Directory, aDirectory);
-        Slash(Directory, 0);
-    }
+FileFind::FileFind(const char *aDirectory, const char *aPattern, int aFlags) :
+    Directory(new char[strlen(aDirectory) + 1]),
+    Flags(aFlags)
+{
+    strcpy(Directory, aDirectory);
+    Slash(Directory, 0);
+
     if (aPattern) {
         Pattern = new char[strlen(aPattern) + 1];
-        if (Pattern)
-            strcpy(Pattern, aPattern);
+
+        strcpy(Pattern, aPattern);
     } else {
         Pattern = 0;
     }
-    Flags = aFlags;
+
 #if defined(USE_DIRENT)
     //#if defined(UNIX) || defined(DOSP32) || defined(NT)
     dir = 0;
@@ -128,10 +134,7 @@ int FileFind::FindFirst(FileInfo **fi) {
     if (Flags & ffHIDDEN)
         attr |= FILE_HIDDEN | FILE_SYSTEM; // separate ?
 
-    if (Pattern)
-        JoinDirFile(fullpattern, Directory, Pattern);
-    else
-        JoinDirFile(fullpattern, Directory, "*");
+    JoinDirFile(fullpattern, Directory, (Pattern) ? Pattern : "*");
 
     if ((rc = DosFindFirst(fullpattern,
                            &hdir,
@@ -151,7 +154,7 @@ int FileFind::FindFirst(FileInfo **fi) {
         JoinDirFile(fullpath, Directory, name);
         name = fullpath;
     }
-    memset((void *)&t, 0, sizeof(t));
+    memset(&t, 0, sizeof(t));
     t.tm_year = find.fdateLastWrite.year + 80; // ugh!
     t.tm_mon = find.fdateLastWrite.month - 1;
     t.tm_mday = find.fdateLastWrite.day;
@@ -226,10 +229,7 @@ int FileFind::FindFirst(FileInfo **fi) {
      if (Flags & ffHIDDEN)
      attr |= FILE_HIDDEN | FILE_SYSTEM; // separate ?
      */
-    if (Pattern)
-        JoinDirFile(fullpattern, Directory, Pattern);
-    else
-        JoinDirFile(fullpattern, Directory, "*");
+    JoinDirFile(fullpattern, Directory, (Pattern) ? Pattern : "*");
 
     if ((rc = (int) FindFirstFile(fullpattern, &find)) < 0)
     {
