@@ -7,10 +7,12 @@
  *
  */
 
-#ifndef __FILESYS_H__
-#define __FILESYS_H__
+#ifndef S_FILES_H
+#define S_FILES_H
 
-#define SDOT   "."    
+#include <sys/types.h>
+
+#define SDOT   "."
 
 #ifdef UNIX
 #define SLASH  '/'
@@ -35,20 +37,20 @@
 
 char *Slash(char *Path, int Add);
 char *SlashDir(char *Path);
-int ExpandPath(const char *Path, char *Expanded);
+int ExpandPath(const char *Path, char *Expanded, size_t ExpandSize);
 int CompletePath(const char *Path, char *Completed, int Num);
 int IsSameFile(const char *Path1, const char *Path2);
-int JustDirectory(const char *Path, char *Dir);
-int JustFileName(const char *Path, char *Name);
-int JustLastDirectory(const char *Path, char *Dir);
-int JustRoot(const char *Path, char *Root);
+int JustDirectory(const char *Path, char *Dir, size_t DirSize);
+int JustFileName(const char *Path, char *Name, size_t NameSize);
+int JustLastDirectory(const char *Path, char *Dir, size_t DirSize);
+int JustRoot(const char *Path, char *Root, size_t RootSize);
 int FileExists(const char *Path);
 int IsFullPath(const char *Path);
 int IsDirectory(const char *Path);
-const char *ShortFName(const char *Path, int len);
-int ChangeDir(char *Dir);
+const char *ShortFName(const char *Path, size_t len);
+int ChangeDir(const char *Dir);
 int JoinDirFile(char *Dest, const char *Dir, const char *Name);
-char *SepRChr(char *Dir);
+char *SepRChr(const char *Dir);
 int RelativePathName(const char *Dir, const char *Path, char *RelPath);
 
-#endif
+#endif // S_FILES_H
