@@ -7,27 +7,10 @@
  *
  */
 
-#ifndef __SYSDEP_H
-#define __SYSDEP_H
+#ifndef SYSDEP_H
+#define SYSDEP_H
 
-#include <assert.h>
-#include <errno.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <string.h>
-#if defined(AIX) || defined(SCO) || defined(NCR)
-#include <strings.h>
-#endif
-#include <stdio.h>
-#include <fcntl.h>
-#include <ctype.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <time.h>
-
-#ifdef DBMALLOC
-#include <malloc.h>
-#endif
+#include "fte.h"
 
 #if !defined(OS2) && \
     !defined(NT) && \
@@ -42,6 +25,17 @@
 #    error Target not supported.
 #endif
 
+
+#include <stdlib.h>
+
+#if defined(AIX) || defined(SCO) || defined(NCR)
+#include <strings.h>
+#endif
+
+#ifdef DBMALLOC
+#include <malloc.h>
+#endif
+
 #if defined(UNIX) || defined(DJGPP)
 #    define USE_DIRENT
 #endif
@@ -174,6 +168,10 @@
 #define HAVE_BOOL
 #endif
 
+#if defined __GNUC__ && (__GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 7))
+#define HAVE_BOOL
+#endif
+
 #if defined _G_HAVE_BOOL
 #define HAVE_BOOL
 #endif
@@ -182,6 +180,12 @@
 #define HAVE_BOOL
 #endif
 
+#if defined __BORLANDC__ && defined __OS2__
+#define popen _popen
+#define pclose _pclose
+#define ftruncate _ftruncate
+#endif
+
 #undef HAVE_STRLCPY
 #undef HAVE_STRLCAT
 
@@ -191,5 +195,4 @@
 #define false 0
 #endif
 
-
-#endif
+#endif // SYSDEP_H
