@@ -7,13 +7,39 @@
  *
  */
 
-#include "fte.h"
+#include "c_color.h"
+#include "c_history.h"
+#include "e_cvslog.h"
+#include "e_mark.h"
+#include "e_svnlog.h"
+#include "e_tags.h"
+#include "ftever.h"
+#include "i_modelview.h"
+#include "i_view.h"
+#include "o_buflist.h"
+#include "o_cvs.h"
+#include "o_cvsdiff.h"
+#include "o_directory.h"
+#include "o_messages.h"
+#include "o_modemap.h"
+#include "o_routine.h"
+#include "o_svn.h"
+#include "o_svndiff.h"
+#include "s_files.h"
+#include "s_string.h"
+#include "s_util.h"
 
 EView *ActiveView = 0;
 
 extern BufferView *BufferList;
 
-EView::EView(EModel *AModel) {
+EView::EView(EModel *AModel) :
+    MView(0),
+    Model(AModel),
+    NextView(0),
+    Port(0),
+    CurMsg(0)
+{
     if (ActiveView) {
         Prev = ActiveView;
         Next = ActiveView->Next;
@@ -21,12 +47,9 @@ EView::EView(EModel *AModel) {
         Next->Prev = this;
     } else
         Prev = Next = this;
+
     ActiveView = this;
-    Model = AModel;
-    NextView = 0;
-    Port = 0;
-    MView = 0;
-    CurMsg = 0;
+
     if (Model)
         Model->CreateViewPort(this);
 }
@@ -39,19 +62,20 @@ EView::~EView() {
             ActiveView = Next;
     } else
         ActiveView = 0;
-    if (MView)
-        MView->View = 0;
+
     if (Model)
-        Model->RemoveView(this);
-    if (Port)
-        delete Port;
+	Model->RemoveView(this);
+
+    delete Port;
+    //delete MView;
+    free(CurMsg);
 }
 
 int EView::CanQuit() {
     if (Model)
         return Model->CanQuit();
-    else
-        return 1;
+
+    return 1;
 }
 
 void EView::FocusChange(int GetFocus) {
@@ -140,7 +164,7 @@ int EView::BeginMacro() {
     return Model ? Model->BeginMacro() : 0;
 }
 
-int EView::ExecCommand(int Command, ExState &State) {
+int EView::ExecCommand(ExCommands Command, ExState &State) {
     switch (Command) {
     case ExSwitchTo:            return SwitchTo(State);
     case ExFilePrev:            return FilePrev();
@@ -201,6 +225,30 @@ int EView::ExecCommand(int Command, ExState &State) {
     case ExViewCvsLog:          return ErFAIL;
 #endif
 
+#ifdef CONFIG_OBJ_SVN
+    case ExSvn:                 return Svn(State);
+    case ExRunSvn:              return RunSvn(State);
+    case ExViewSvn:             return ViewSvn(State);
+    case ExClearSvnMessages:    return ClearSvnMessages(State);
+    case ExSvnDiff:             return SvnDiff(State);
+    case ExRunSvnDiff:          return RunSvnDiff(State);
+    case ExViewSvnDiff:         return ViewSvnDiff(State);
+    case ExSvnCommit:           return SvnCommit(State);
+    case ExRunSvnCommit:        return RunSvnCommit(State);
+    case ExViewSvnLog:          return ViewSvnLog(State);
+#else
+    case ExSvn:                 return ErFAIL;
+    case ExRunSvn:              return ErFAIL;
+    case ExViewSvn:             return ErFAIL;
+    case ExClearSvnMessages:    return ErFAIL;
+    case ExSvnDiff:             return ErFAIL;
+    case ExRunSvnDiff:          return ErFAIL;
+    case ExViewSvnDiff:         return ErFAIL;
+    case ExSvnCommit:           return ErFAIL;
+    case ExRunSvnCommit:        return ErFAIL;
+    case ExViewSvnLog:          return ErFAIL;
+#endif
+
     case ExViewBuffers:         return ViewBuffers(State);
 
     case ExShowKey:             return ShowKey(State);
@@ -222,6 +270,8 @@ int EView::ExecCommand(int Command, ExState &State) {
     case ExRemoveGlobalBookmark:return RemoveGlobalBookmark(State);
     case ExGotoGlobalBookmark:  return GotoGlobalBookmark(State);
     case ExPopGlobalBookmark:   return PopGlobalBookmark();
+    default:
+        ;
     }
     return Model ? Model->ExecCommand(Command, State) : 0;
 }
@@ -294,7 +344,6 @@ void EView::DeleteModel(EModel *M) {
     }
     delete M;
     SetMsg(0);
-    return;
 }
 
 /////////////////////////////////////////////////////////////////////////////
@@ -331,41 +380,39 @@ int EView::FileLast() {
 }
 
 int EView::SwitchTo(ExState &State) {
-    EModel *M;
     int No;
 
     if (State.GetIntParam(this, &No) == 0) {
         char str[10] = "";
 
-        if (MView->Win->GetStr("Obj.Number", sizeof(str), (char *)str, 0) == 0) return 0;
+        if (MView->Win->GetStr("Obj.Number", sizeof(str), str, 0) == 0) return 0;
         No = atoi(str);
     }
-    M = Model;
-    while (M) {
+
+    for (EModel* M = Model; M; M = M->Next) {
         if (M->ModelNo == No) {
             SwitchToModel(M);
             return 1;
         }
-        M = M->Next;
-        if (M == Model)
-            return 0;
+        if (M->Next == Model)
+	    break;
     }
     return 0;
 }
 
 
 int EView::FileSaveAll() {
-    EModel *M = Model;
-    while (M) {
+    for (EModel *M = Model; M; M = M->Next) {
         if (M->GetContext() == CONTEXT_FILE) {
             EBuffer *B = (EBuffer *)M;
             if (B->Modified) {
                 SwitchToModel(B);
-                if (B->Save() == 0) return 0;
+		if (B->Save() == 0)
+		    return 0;
             }
         }
-        M = M->Next;
-        if (M == Model) break;
+	if (M->Next == Model)
+	    break;
     }
     return 1;
 }
@@ -415,7 +462,7 @@ int EView::FileOpenInMode(ExState &State) {
 }
 
 int EView::SetPrintDevice(ExState &State) {
-    char Dev[MAXPATH];
+    char Dev[sizeof(PrintDevice)];
 
     strcpy(Dev, PrintDevice);
     if (State.GetStrParam(this, Dev, sizeof(Dev)) == 0)
@@ -438,16 +485,17 @@ int EView::ShowKey(ExState &/*State*/) {
     ks.Mask = 0;
     ks.Key = MView->Win->GetChar(0);
 
-    GetKeyName(buf, ks);
+    GetKeyName(buf, sizeof(buf), ks);
     Msg(S_INFO, "Key: '%s' - '%8X'", buf, ks.Key);
     return 1;
 }
 
 void EView::Msg(int level, const char *s, ...) {
+    char msgbuftmp[MSGBUFTMP_SIZE];
     va_list ap;
 
     va_start(ap, s);
-    vsprintf(msgbuftmp, s, ap);
+    vsnprintf(msgbuftmp, sizeof(msgbuftmp), s, ap);
     va_end(ap);
 
     if (level != S_BUSY)
@@ -455,12 +503,11 @@ void EView::Msg(int level, const char *s, ...) {
 }
 
 void EView::SetMsg(const char *msg) {
-    if (CurMsg)
-        free(CurMsg);
-    CurMsg = 0;
-    if (msg && strlen(msg))
-        CurMsg = strdup(msg);
-    if (CurMsg && msg && MView) {
+    free(CurMsg);
+
+    CurMsg = (msg && msg[0]) ? strdup(msg) : 0;
+
+    if (CurMsg && MView) {
         TDrawBuffer B;
         char SColor;
         int Cols, Rows;
@@ -468,9 +515,9 @@ void EView::SetMsg(const char *msg) {
         MView->ConQuerySize(&Cols, &Rows);
 
         if (MView->IsActive())
-            SColor = hcStatus_Active;
+            SColor = (char)hcStatus_Active;
         else
-            SColor = hcStatus_Normal;
+            SColor = (char)hcStatus_Normal;
 
         MoveChar(B, 0, Cols, ' ', SColor, Cols);
         MoveStr(B, 0, Cols, CurMsg, SColor, Cols);
@@ -488,22 +535,17 @@ int EView::ViewBuffers(ExState &/*State*/) {
         BufferList->UpdateList();
         BufferList->Row = 1;
         SwitchToModel(BufferList);
-        return 1;
     }
-    return 0;
+    return 1;
 }
 
 #ifdef CONFIG_OBJ_ROUTINE
 int EView::ViewRoutines(ExState &/*State*/) {
-    //int rc = 1;
-    //RoutineView *routines;
-    EModel *M;
-    EBuffer *Buffer;
+    EModel *M = Model;
 
-    M = Model;
     if (M->GetContext() != CONTEXT_FILE)
         return 0;
-    Buffer = (EBuffer *)M;
+    EBuffer *Buffer = (EBuffer *)M;
 
     if (Buffer->Routines == 0) {
         if (BFS(Buffer, BFS_RoutineRegexp) == 0) {
@@ -511,8 +553,6 @@ int EView::ViewRoutines(ExState &/*State*/) {
             return 0;
         }
         Buffer->Routines = new RoutineView(0, &ActiveModel, Buffer);
-        if (Buffer->Routines == 0)
-            return 0;
     } else {
         Buffer->Routines->UpdateList();
     }
@@ -528,30 +568,29 @@ int EView::DirOpen(ExState &State) {
     if (State.GetStrParam(this, Path, sizeof(Path)) == 0)
         if (GetDefaultDirectory(Model, Path, sizeof(Path)) == 0)
             return 0;
+
     return OpenDir(Path);
 }
 
-int EView::OpenDir(char *Path) {
+int EView::OpenDir(const char *Path) {
     char XPath[MAXPATH];
     EDirectory *dir = 0;
 
-    if (ExpandPath(Path, XPath) == -1)
+    if (ExpandPath(Path, XPath, sizeof(XPath)) == -1)
         return 0;
-    {
-        EModel *x = Model;
-        while (x) {
-            if (x->GetContext() == CONTEXT_DIRECTORY) {
-                if (filecmp(((EDirectory *)x)->Path, XPath) == 0)
-                {
-                    dir = (EDirectory *)x;
-                    break;
-                }
-            }
-            x = x->Next;
-            if (x == Model)
-                break;
-        }
+
+    for (EModel *x = Model; x; x = x->Next) {
+	if (x->GetContext() == CONTEXT_DIRECTORY) {
+	    if (filecmp(((EDirectory *)x)->Path.c_str(), XPath) == 0)
+	    {
+		dir = (EDirectory *)x;
+		break;
+	    }
+	}
+	if (x->Next == Model)
+	    break;
     }
+
     if (dir == 0)
         dir = new EDirectory(0, &ActiveModel, XPath);
     SelectModel(dir);
@@ -564,7 +603,7 @@ int EView::Compile(ExState &State) {
     static char Cmd[256] = "";
     char Command[256] = "";
 
-    if (CompilerMsgs != 0 && CompilerMsgs->Running) {
+    if (CompilerMsgs != 0 && CompilerMsgs->IsRunning()) {
         Msg(S_INFO, "Already running...");
         return 0;
     }
@@ -576,11 +615,11 @@ int EView::Compile(ExState &State) {
                 strcpy(Cmd, BFS(B, BFS_CompileCommand));
         }
         if (Cmd[0] == 0)
-            strcpy(Cmd, CompileCommand);
+            strlcpy(Cmd, CompileCommand, sizeof(Cmd));
 
         if (MView->Win->GetStr("Compile", sizeof(Cmd), Cmd, HIST_COMPILE) == 0) return 0;
 
-        strcpy(Command, Cmd);
+        strlcpy(Command, Cmd, sizeof(Cmd));
     } else {
         if (MView->Win->GetStr("Compile", sizeof(Command), Command, HIST_COMPILE) == 0) return 0;
     }
@@ -590,7 +629,7 @@ int EView::Compile(ExState &State) {
 int EView::RunCompiler(ExState &State) {
     char Command[256] = "";
 
-    if (CompilerMsgs != 0 && CompilerMsgs->Running) {
+    if (CompilerMsgs != 0 && CompilerMsgs->IsRunning()) {
         Msg(S_INFO, "Already running...");
         return 0;
     }
@@ -599,10 +638,10 @@ int EView::RunCompiler(ExState &State) {
         if (Model->GetContext() == CONTEXT_FILE) {
             EBuffer *B = (EBuffer *)Model;
             if (BFS(B, BFS_CompileCommand) != 0) 
-                strcpy(Command, BFS(B, BFS_CompileCommand));
+                strlcpy(Command, BFS(B, BFS_CompileCommand), sizeof(Command));
         }
         if (Command[0] == 0)
-            strcpy(Command, CompileCommand);
+            strlcpy(Command, CompileCommand, sizeof(Command));
     }
     return Compile(Command);
 }
@@ -612,7 +651,7 @@ int EView::Compile(char *Command) {
     EMessages *msgs;
     
     if (CompilerMsgs != 0) {
-        strcpy(Dir, CompilerMsgs->Directory);
+        strlcpy(Dir, CompilerMsgs->GetDirectory(), sizeof(Dir));
         CompilerMsgs->RunPipe(Dir, Command);
         msgs = CompilerMsgs;
     } else {
@@ -626,23 +665,25 @@ int EView::Compile(char *Command) {
 }
 
 int EView::ViewMessages(ExState &/*State*/) {
-    if (CompilerMsgs != 0) {
-        SwitchToModel(CompilerMsgs);
-        return 1;
-    }
-    return 0;
+    if (!CompilerMsgs)
+        return 0;
+
+    SwitchToModel(CompilerMsgs);
+    return 1;
 }
 
 int EView::CompilePrevError(ExState &/*State*/) {
-    if (CompilerMsgs != 0)
-        return CompilerMsgs->CompilePrevError(this);
-    return 0;
+    if (!CompilerMsgs)
+        return 0;
+
+    return CompilerMsgs->CompilePrevError(this);
 }
 
 int EView::CompileNextError(ExState &/*State*/) {
-    if (CompilerMsgs != 0)
-        return CompilerMsgs->CompileNextError(this);
-    return 0;
+    if (!CompilerMsgs)
+        return 0;
+
+    return CompilerMsgs->CompileNextError(this);
 }
 #endif
 
@@ -655,20 +696,22 @@ int EView::ViewModeMap(ExState &/*State*/) {
     if (TheEventMapView != 0)
         TheEventMapView->ViewMap(GetEventMap());
     else
+	// FIXME: Complete MESS!!
         (void)new EventMapView(0, &ActiveModel, GetEventMap());
-    if (TheEventMapView != 0)
-        SwitchToModel(TheEventMapView);
-    else
-        return 0;
+
+    if (!TheEventMapView)
+	return 0;
+
+    SwitchToModel(TheEventMapView);
     return 1;
 }
 
 int EView::ClearMessages() {
-    if (CompilerMsgs != 0 && CompilerMsgs->Running) {
-        Msg(S_INFO, "Running...");
-        return 0;
-    }
-    if (CompilerMsgs != 0) {
+    if (CompilerMsgs) {
+        if (CompilerMsgs->IsRunning()) {
+            Msg(S_INFO, "Running...");
+            return 0;
+        }
         CompilerMsgs->FreeErrors();
         CompilerMsgs->UpdateList();
     }
@@ -680,17 +723,16 @@ int EView::TagLoad(ExState &State) {
     char Tag[MAXPATH];
     char FullTag[MAXPATH];
 
-    char const* pTagFile = getenv("TAGFILE");
+    const char* pTagFile = getenv("TAGFILE");
     if (pTagFile == NULL)
-    {
         pTagFile = "tags";
-    }
-    if (ExpandPath(pTagFile, Tag) == -1)
+
+    if (ExpandPath(pTagFile, Tag, sizeof(Tag)) == -1)
         return 0;
     if (State.GetStrParam(this, Tag, sizeof(Tag)) == 0)
         if (MView->Win->GetFile("Load tags", sizeof(Tag), Tag, HIST_TAGFILES, GF_OPEN) == 0) return 0;
 
-    if (ExpandPath(Tag, FullTag) == -1)
+    if (ExpandPath(Tag, FullTag, sizeof(FullTag)) == -1)
         return 0;
 
     if (!FileExists(FullTag)) {
@@ -710,14 +752,16 @@ int EView::ConfigRecompile(ExState &/*State*/) {
 
     char command[1024];
 
-    strcpy(command, "cfte ");
-    strcat(command, ConfigSourcePath);
-    strcat(command, " ");
+    strlcpy(command, "cfte \"", sizeof(command));
+    strlcat(command, ConfigSourcePath, sizeof(command));
+    strlcat(command, "\" ", sizeof(command));
 #ifdef UNIX
-    if (ExpandPath("~/.fterc", command + strlen(command)) != 0)
+    if (ExpandPath("~/.fterc", command + strlen(command), sizeof(command) - strlen(command)) != 0)
         return 0;
 #else
-    strcat(command, ConfigFileName);
+    strlcat(command, "\"", sizeof(command));
+    strlcat(command, ConfigFileName, sizeof(command));
+    strlcat(command, "\"", sizeof(command));
 #endif
     return Compile(command);
 }
@@ -753,7 +797,7 @@ int EView::PopGlobalBookmark() {
     return 1;
 }
 
-int EView::GetStrVar(int var, char *str, int buflen) {
+int EView::GetStrVar(int var, char *str, size_t buflen) {
     //switch (var) {
     //}
     return Model->GetStrVar(var, str, buflen);
@@ -815,7 +859,7 @@ int EView::Cvs(char *Options) {
 
     switch (Model->GetContext()) {
         case CONTEXT_FILE:
-            if (JustFileName(((EBuffer *)Model)->FileName, OnFiles) != 0) return 0;
+            if (JustFileName(((EBuffer *)Model)->FileName, OnFiles, sizeof(buf)) != 0) return 0; // OnFiles points to buf
             break;
         case CONTEXT_CVSDIFF:
             OnFiles = strdup(CvsDiffView->OnFiles);
@@ -852,11 +896,11 @@ int EView::ClearCvsMessages(ExState &/*State*/) {
 }
 
 int EView::ViewCvs(ExState &/*State*/) {
-    if (CvsView != 0) {
-        SwitchToModel(CvsView);
-        return 1;
-    }
-    return 0;
+    if (!CvsView)
+        return 0;
+
+    SwitchToModel(CvsView);
+    return 1;
 }
 
 int EView::CvsDiff(ExState &State) {
@@ -907,7 +951,7 @@ int EView::CvsDiff(char *Options) {
 
     switch (Model->GetContext()) {
         case CONTEXT_FILE:
-            if (JustFileName(((EBuffer *)Model)->FileName, OnFiles) != 0) return 0;
+            if (JustFileName(((EBuffer *)Model)->FileName, OnFiles, sizeof(buf)) != 0) return 0; // OnFiles points to buf
             break;
         case CONTEXT_CVSDIFF:
             OnFiles = strdup(CvsDiffView->OnFiles);
@@ -930,11 +974,11 @@ int EView::CvsDiff(char *Options) {
 }
 
 int EView::ViewCvsDiff(ExState &/*State*/) {
-    if (CvsDiffView != 0) {
-        SwitchToModel(CvsDiffView);
-        return 1;
-    }
-    return 0;
+    if (!CvsDiffView)
+        return 0;
+
+    SwitchToModel(CvsDiffView);
+    return 1;
 }
 
 int EView::CvsCommit(ExState &State) {
@@ -985,7 +1029,7 @@ int EView::CvsCommit(char *Options) {
 
     switch (Model->GetContext()) {
         case CONTEXT_FILE:
-            if (JustFileName(((EBuffer *)Model)->FileName, OnFiles) != 0) return 0;
+            if (JustFileName(((EBuffer *)Model)->FileName, OnFiles, sizeof(buf)) != 0) return 0; // OnFiles points to buf
             break;
         case CONTEXT_CVSDIFF:
             OnFiles = strdup(CvsDiffView->OnFiles);
@@ -1012,3 +1056,251 @@ int EView::ViewCvsLog(ExState &/*State*/) {
 }
 
 #endif
+
+#ifdef CONFIG_OBJ_SVN
+
+int EView::Svn(ExState &State) {
+    static char Opts[128] = "";
+    char Options[128] = "";
+
+    if (SvnView != 0 && SvnView->Running) {
+        Msg(S_INFO, "Already running...");
+        return 0;
+    }
+
+    if (State.GetStrParam(this, Options, sizeof(Options)) == 0) {
+        if (MView->Win->GetStr("SVN options", sizeof(Opts), Opts, HIST_SVN) == 0) return 0;
+        strcpy(Options, Opts);
+    } else {
+        if (MView->Win->GetStr("SVN options", sizeof(Options), Options, HIST_SVN) == 0) return 0;
+    }
+    return Svn(Options);
+}
+
+int EView::RunSvn(ExState &State) {
+    char Options[128] = "";
+
+    if (SvnView != 0 && SvnView->Running) {
+        Msg(S_INFO, "Already running...");
+        return 0;
+    }
+
+    State.GetStrParam(this, Options, sizeof(Options));
+    return Svn(Options);
+}
+
+int EView::Svn(char *Options) {
+    char Dir[MAXPATH] = "";
+    char Command[256] = "";
+    char buf[1024] = "";
+    char *OnFiles = buf;
+    ESvn *svn;
+
+    if (GetDefaultDirectory(Model, Dir, sizeof(Dir)) == 0) return 0;
+
+    strlcpy(Command, SvnCommand, sizeof(Command));
+    strlcat(Command, " ", sizeof(Command));
+    if (Options[0] != 0) {
+        strlcat(Command, Options, sizeof(Command));
+        strlcat(Command, " ", sizeof(Command));
+    }
+
+    switch (Model->GetContext()) {
+        case CONTEXT_FILE:
+            if (JustFileName(((EBuffer *)Model)->FileName, OnFiles, sizeof(buf)) != 0) return 0; // OnFiles points to buf
+            break;
+        case CONTEXT_SVNDIFF:
+            OnFiles = strdup(SvnDiffView->OnFiles);
+            break;
+        case CONTEXT_SVN:
+            OnFiles = ((ESvn *)Model)->MarkedAsList();
+            if (!OnFiles) OnFiles = strdup(((ESvn *)Model)->OnFiles);
+            break;
+    }
+
+    if (SvnView != 0) {
+        SvnView->RunPipe(Dir, Command, OnFiles);
+        svn = SvnView;
+    } else {
+        svn = new ESvn(0, &ActiveModel, Dir, Command, OnFiles);
+    }
+    if (OnFiles != buf) free(OnFiles);
+    SwitchToModel(svn);
+    return 1;
+}
+
+int EView::ClearSvnMessages(ExState &/*State*/) {
+    if (SvnView != 0) {
+        if (SvnView->Running) {
+            Msg(S_INFO, "Running...");
+            return 0;
+        } else {
+            SvnView->FreeLines();
+            SvnView->UpdateList();
+            return 1;
+        }
+    }
+    return 0;
+}
+
+int EView::ViewSvn(ExState &/*State*/) {
+    if (!SvnView)
+        return 0;
+
+    SwitchToModel(SvnView);
+    return 1;
+}
+
+int EView::SvnDiff(ExState &State) {
+    static char Opts[128] = "";
+    char Options[128] = "";
+
+    if (SvnDiffView != 0 && SvnDiffView->Running) {
+        Msg(S_INFO, "Already running...");
+        return 0;
+    }
+
+    if (State.GetStrParam(this, Options, sizeof(Options)) == 0) {
+        if (MView->Win->GetStr("SVN diff options", sizeof(Opts), Opts, HIST_SVNDIFF) == 0) return 0;
+        strcpy(Options, Opts);
+    } else {
+        if (MView->Win->GetStr("SVN diff options", sizeof(Options), Options, HIST_SVNDIFF) == 0) return 0;
+    }
+    return SvnDiff(Options);
+}
+
+int EView::RunSvnDiff(ExState &State) {
+    char Options[128] = "";
+
+    if (SvnDiffView != 0 && SvnDiffView->Running) {
+        Msg(S_INFO, "Already running...");
+        return 0;
+    }
+
+    State.GetStrParam(this, Options, sizeof(Options));
+    return SvnDiff(Options);
+}
+
+int EView::SvnDiff(char *Options) {
+    char Dir[MAXPATH] = "";
+    char Command[256] = "";
+    char buf[1024] = "";
+    char *OnFiles = buf;
+    ESvnDiff *diffs;
+
+    if (GetDefaultDirectory(Model, Dir, sizeof(Dir)) == 0) return 0;
+
+    strcpy(Command, SvnCommand);
+    strcat(Command, " diff ");
+    if (Options[0] != 0) {
+        strcat(Command, Options);
+        strcat(Command, " ");
+    }
+
+    switch (Model->GetContext()) {
+        case CONTEXT_FILE:
+            if (JustFileName(((EBuffer *)Model)->FileName, OnFiles, sizeof(buf)) != 0) return 0; // OnFiles points to buf
+            break;
+        case CONTEXT_SVNDIFF:
+            OnFiles = strdup(SvnDiffView->OnFiles);
+            break;
+        case CONTEXT_SVN:
+            OnFiles = ((ESvn *)Model)->MarkedAsList();
+            if (!OnFiles) OnFiles = strdup(((ESvn *)Model)->OnFiles);
+            break;
+    }
+
+    if (SvnDiffView != 0) {
+        SvnDiffView->RunPipe(Dir, Command, OnFiles);
+        diffs = SvnDiffView;
+    } else {
+        diffs = new ESvnDiff(0, &ActiveModel, Dir, Command, OnFiles);
+    }
+    if (OnFiles != buf) free(OnFiles);
+    SwitchToModel(diffs);
+    return 1;
+}
+
+int EView::ViewSvnDiff(ExState &/*State*/) {
+    if (!SvnDiffView)
+        return 0;
+
+    SwitchToModel(SvnDiffView);
+    return 1;
+}
+
+int EView::SvnCommit(ExState &State) {
+    static char Opts[128] = "";
+    char Options[128] = "";
+
+    if (SvnView != 0 && SvnView->Running) {
+        Msg(S_INFO, "Already running...");
+        return 0;
+    }
+
+    if (State.GetStrParam(this, Options, sizeof(Options)) == 0) {
+        if (MView->Win->GetStr("SVN commit options", sizeof(Opts), Opts, HIST_SVNCOMMIT) == 0) return 0;
+        strcpy(Options, Opts);
+    } else {
+        if (MView->Win->GetStr("SVN commit options", sizeof(Options), Options, HIST_SVNCOMMIT) == 0) return 0;
+    }
+    return SvnCommit(Options);
+}
+
+int EView::RunSvnCommit(ExState &State) {
+    char Options[128] = "";
+
+    if (SvnView != 0 && SvnView->Running) {
+        Msg(S_INFO, "Already running...");
+        return 0;
+    }
+
+    State.GetStrParam(this, Options, sizeof(Options));
+    return SvnCommit(Options);
+}
+
+int EView::SvnCommit(char *Options) {
+    char Dir[MAXPATH] = "";
+    char Command[256] = "";
+    char buf[1024] = "";
+    char *OnFiles = buf;
+    ESvn *svn;
+
+    if (GetDefaultDirectory(Model, Dir, sizeof(Dir)) == 0) return 0;
+
+    strcpy(Command, SvnCommand);
+    strcat(Command, " commit ");
+    if (Options[0] != 0) {
+        strcat(Command, Options);
+        strcat(Command, " ");
+    }
+
+    switch (Model->GetContext()) {
+        case CONTEXT_FILE:
+            if (JustFileName(((EBuffer *)Model)->FileName, OnFiles, sizeof(buf)) != 0) return 0; // OnFiles points to buf
+            break;
+        case CONTEXT_SVNDIFF:
+            OnFiles = strdup(SvnDiffView->OnFiles);
+            break;
+        case CONTEXT_SVN:
+            OnFiles = ((ESvn *)Model)->MarkedAsList();
+            if (!OnFiles) OnFiles = strdup(((ESvn *)Model)->OnFiles);
+            break;
+    }
+
+    svn = (!SvnView) ? new ESvn(0, &ActiveModel) : SvnView;
+    SwitchToModel(svn);
+    svn->RunCommit(Dir, Command, OnFiles);
+    if (OnFiles != buf) free(OnFiles);
+    return 1;
+}
+
+int EView::ViewSvnLog(ExState &/*State*/) {
+    if (!SvnLogView)
+        return 0;
+
+    SwitchToModel(SvnLogView);
+    return 1;
+}
+
+#endif
